/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include "CAVERN.hxx"

#ifdef WIN32
#define sleep(seconds) Sleep(seconds * 1000)
#endif

/* This is a callback to handle any incoming data on a key. This method follows
the traditional style of firing callbacks to handle events. See below for the
new way.
*/
void incomingData(char* path, char* key, void* userData)
{
	CAVERNdb_client_c *client = (CAVERNdb_client_c*) userData;
	int size;
	char* data = client->get(path, key, &size);

	printf("CALLBACK FIRED    %s %s %s\n", path, key, data);
	if (data) delete data;
}

/* This is contemporary way of handling incoming data on a key. This is an observer
class that is set up to listen for updates to the db client. This technique
follows the observer/observable (or observer/subject) design pattern- as used in
Java.
*/
class myObserver_c : public CAVERNmisc_observer_c 
{
	// You overload the update function to handle the incoming data.
	void update(CAVERNmisc_subject_c *client) {

		CAVERNdb_client_c *theClient = (CAVERNdb_client_c*) client;

		int size;
		char *path = theClient->getNotifyPathname();
		char *key = theClient->getNotifyKeyname();

		char* data = theClient->get(path, key, &size);

		printf("OBSERVER NOTIFIED %s %s %s\n", path, key, data);
		if (data) delete data;
	}

};


main(int argc, char** argv)
{
	CAVERNinit();

	myObserver_c myObserver;

	CAVERNdb_client_c client;
	client.init(argv[1]);

	/* There are 2 ways to get a notification on when new data is arriving.
	   Register a callback using the trigger() method.
	   Or attach an observer to the client using the attach() method.
	*/	  
    int perfDaemonStatus =1;
	if (argc == 3  && client.initSendStats(argv[2]) != CAVERNnet_perfMonitor_c::OK) {
		printf("Cannot connect to %s\n", argv[2]);
		exit(1);
	}
    if (argc <3) 
        perfDaemonStatus =0;

	client.trigger(incomingData, (void*) &client);
	client.attach(&myObserver);

	client.fetch("path1","key1");

	client.put("path1","key1", "hello", 6, CAVERNdb_client_c::UNRELIABLE_CONNECTION );

	printf("DONE PUT\n");
    client.commit("path1", "key1");
	client.remoteCommit("path1","key1");

	while(1) {
		int status = client.process();
        //Uncomment this for an instantaneous display of performance statistics
        //client.showStats("dbClient_monitoring","Some_local_DB_data");
     
        //Uncomment this for sending the statistics to a remote perfDaemon (the initSendStats must be done before this)
        /*if (client.sendStats("CAVE_avatar_on_clark", "New_avatar_entered") != CAVERNnet_perfMonitor_c::OK)
            printf("\nsendStats failed!\n");*/
        sleep(1);
		if (status == CAVERNdb_client_c::CONNECTION_TERMINATED) {
			client.reconnect();
		}
	}
}









