/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

/*
The server and client sample programs illustrate the use of CAVERNnet_datapack_c class. Miscellaneous data like floats, doubles, longs etc are packed in a single buffer and sent by the server across the network, to be read and unpacked by the client. The array packing and unpacking API's have also been used in these sample programs.
*/


#include "CAVERN.hxx"

#ifdef WIN32
#define sleep(seconds) Sleep(seconds * 1000)
#endif

void main (int argc, char *argv[])
{

	CAVERNnet_tcpServer_c *server;
	CAVERNnet_datapack_c packer;

	char mychar='A';
	int myint = 20000000;
	float myfloat = 3.33;
	double mydouble = 4.44444;
	long mylong = 1138;
	char mystr[8];
    strcpy(mystr,"thx1138");
    float floatArray[4] = {1.1, 2.2, 3.3, 4.4};
	printf("char=%d\n",packer.sizeof_char());
	printf("int=%d\n",packer.sizeof_int());
	printf("float=%d\n",packer.sizeof_float());
	printf("double=%d\n",packer.sizeof_double());
	printf("long=%d\n",packer.sizeof_long());

	int dataSize=packer.sizeof_char()+packer.sizeof_int()+packer.sizeof_float()+packer.sizeof_double() +  packer.sizeof_char(8) + packer.sizeof_long() + packer.sizeof_float()*4;

	char *sendBuffer;
	sendBuffer = new char[dataSize];

	CAVERNinit();

	server = new CAVERNnet_tcpServer_c;
	if (server->init(9999) == 0)	{
		printf("Can not start server\n");
		exit(1);
	}


	CAVERNnet_tcpClient_c *aClient;
	aClient=NULL;
	while(NULL==aClient){
		aClient = server->checkForNewConnections();
		sleep(1);

	}

	printf("Send Char: %c\n", mychar);
	printf("Send Int: %d\n", myint);
	printf("Send Long: %ld\n", mylong);
	printf("Send Float: %f\n", myfloat);
	printf("Send Double: %lf\n", mydouble);
	printf("Send String: %s\n", mystr);
	printf("Send Float Array - with the floats: ");
    for(int i=0; i<4; i++)
        printf("%f\t", floatArray[i]);
	packer.initPack(sendBuffer,dataSize);
	packer.packFloat(myfloat);
	packer.packInt(myint);
	packer.packDouble(mydouble);
	packer.packChar(mychar);
	packer.pack(mystr,8);
    packer.packFloatArray(floatArray, 4);
    packer.packLong(mylong);	
    int realSize = packer.getBufferFilledSize();
	printf("\n\nSIZE: %d/%d\n",realSize, dataSize);
	aClient->write(sendBuffer, &realSize, CAVERNnet_tcpClient_c::BLOCKING);

	aClient->close();
	server->close();
	delete sendBuffer;
}

