/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/
#include "CAVERN.hxx"

void main(int argc, char **argv)
{

	// webget site path filename

	if (argc < 6) {
		printf("%s WEBsite WEBpath WEBfilename Localpath Localfile\n", argv[0]);
		printf("e.g. %s www.evl.uic.edu spiff/images blazer.gif /netusr/evl/spiff blazer.gif\n", argv[0]);
		printf("This grabs: http://www.evl.uic.edu/spiff/images/blazer.gif\n");
		printf("and copies it to: /netusr/evl/spiff/blazer.gif\n");
		exit(1);
	}

	CAVERNinit();

	// Create a CAVERNnet_http_c object.
	CAVERNnet_http_c webobj;

	// Set its attributes for access.
	webobj.setRemoteSite(argv[1]);
	webobj.setRemotePath(argv[2]);
	webobj.setRemoteFile(argv[3]);
	webobj.setLocalPath(argv[4]);
	webobj.setLocalFile(argv[5]);

	// Set to CAVERNnet_http_c::NO_TIME_OUT for no time out waiting for data.
	// Time is in seconds.
	webobj.setTimeOut(5);

	// There are several modes of operation:
	// AUTO_LOAD will grab a remote file only if it does not exist locally
	// and if the remote one is newer than the local one.
	//
	// FORCED_LOAD will grab a remote file always.
	//
	// CHECK_BUT_DONT_LOAD will grab header info and return a status but
	// will not grab the file.

	webobj.setMode(CAVERNnet_http_c::AUTO_LOAD);
	//	webobj.SetMode(CAVERNnet_http_c::FORCED_LOAD);
	//	webobj.SetMode(CAVERNnet_http_c::CHECK_BUT_DONT_LOAD);


	// Once the parameters are set, GRAB THE FILE!!
	int status = webobj.grabWEBFile();

	// Here are the return codes from the call.
	switch(status) {
	case 200:
		printf("File grabbed ok!\n");
		break;
	case 304:
		printf("File on server is not new. No download attempted.\n");
		break;
	case 404:
		printf("File not found on WEB server.\n");
		break;
	case -1:
		printf("Connection timed out. Cached version available.\n");
		break;
	case -11:
		printf("Connection timed out. No cached version available.\n");
		break;
	case -2:
		printf("Cannot connect to WEB server.\n");
		break;
	case -3:
		printf("Download of file successful but renaming of temp file failed.\n");
		break;
	case -4:
		printf("Unable to write to local file.\n");
		break;

	case -5:
		printf("Remote file was NEW. But I didn't load it.\n");
		break;
	case -6:
		printf("Can't find content-length of remote file.\n");
		break;
	default:
		printf("WEB Server Error %d.\n",status);
	}
    FILE* filePtr = fopen("http_monitoring.log", "a");
    if (filePtr)
    {
        if (webobj.logStats("http_Monitoring", "monitoring_web_grab", filePtr) != CAVERNnet_perfMonitor_c::OK)
            printf("\nlogStats failed!\n");
    }
    
    webobj.showStats("Testing_http_perfMonitoring", "monitoring_web_grab");
}
