/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

/* Demo for mcast class.
 * Compile this with: make newclient
 *
 * On 2 separate machines type:
 * newclient 225.0.0.37 7777 hello
 * newclient 225.0.0.37 7777 there
 * Now watch the messages fly.
 *
 */

#include "CAVERN.hxx"
#ifdef WIN32
#include <winsock.h>
#else
#include <unistd.h>
#endif
#include <stdlib.h>

#define BUFSIZE 256

#ifdef WIN32
#define EWOULDBLOCK WSAEWOULDBLOCK 
#define sleep(seconds) Sleep(seconds * 1000)
#endif

main(int argc, char **argv)
{

	if (argc < 2) {
		printf("Usage: %s mcast_host (e.g. 225.0.0.37) mcast_port (e.g. # > 6000) <string>\n",argv[0]);
		exit(1);
	}

	CAVERNinit();

	/* Make an mcast object */
	CAVERNnet_mcast_c *client;

	client = new CAVERNnet_mcast_c;

	char selfIP[256];
	client->getSelfIP(selfIP);
	printf("I am: %s %lu\n",selfIP,client->getSelfIP());
	

	/* Open to mcast host and port */
	if (client->init(argv[1],atoi(argv[2])) == CAVERNnet_mcast_c::FAILED) {
		printf("Connection error. Bailing out\n"); exit(1);
	}

	/* Make it a non blocking connection */
	client->makeNonBlocking();

	char buffer[BUFSIZE];
	char outMsg[BUFSIZE];
	int counter = 0;
	while(1) {

		/* Try and receive something */
		if (client->receive(buffer, BUFSIZE) < 0) {
			if (errno != EWOULDBLOCK)
			printf("Receive Error #%d\n", errno);
		}

		/* Send your command line string */
		sprintf(outMsg,"%s %d\n", argv[3], counter++);
		if (client->send(outMsg,strlen(outMsg)+1) < 0) {
			printf("Send Error #%d\n", errno);
		}

		printf("MESSAGE: %s\n",buffer);
		client->printInfo(); /* Print connection info */

		sleep(1);
	}
}
