/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include "CAVERN.hxx" 

#ifndef WIN32
#include <iostream.h>
#else
// CAVERN defines std namespace and uses STL.
#include <iostream>
#include <sys\timeb.h>

// Simplified version of gettimeofday function.
static int gettimeofday(struct timeval *tp, void *tzp)
{
    struct _timeb t;

    _ftime(&t);
    tp->tv_sec = t.time;
    tp->tv_usec = t.millitm * 1000;
    return 0;
}
#endif

main( int a_iArgc, char *a_pcArgv[] )
{
  if( a_iArgc != 3 )
    {
      printf("usage: %s <port> <recv buffer size>\n ",a_pcArgv[0]);
      exit(1);
    }
  int iPort = atoi( a_pcArgv[1] );
  int iBufferSize = atoi( a_pcArgv[2] );

  int nTotalBytesReceived=0;
  char *pcBuffer = new char[iBufferSize];
  struct timeval tStartTime, tEndTime;
  float fElapsed=0;

  CAVERNinit();

  memset( pcBuffer, 0, iBufferSize );

  CAVERNnet_parallelTcpClient_c *client;
  CAVERNnet_parallelTcpServer_c uServer;

  uServer.init(iPort);

  client = uServer.checkForNewConnections();
  gettimeofday(&tStartTime, NULL);

  memset( pcBuffer, 0, iBufferSize ); 
  int j=0;
  client->read( pcBuffer, &iBufferSize );
  nTotalBytesReceived+=iBufferSize;

  cout << endl;
  printf("Total bytes received: %d\n",nTotalBytesReceived);

  gettimeofday(&tEndTime, NULL);

  fElapsed+=tEndTime.tv_sec-tStartTime.tv_sec;
  fElapsed+=(float)((float)(tEndTime.tv_usec-tStartTime.tv_usec)/1000000);

  printf("Approx Elapsed Seconds = %f\n",fElapsed);
  cout << "Done!" << endl;

  delete client;
  delete pcBuffer;
}

  
