/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

/* E.g. of client using TCP protocol.
 * This will connect to a server and send text to it.
 * Then on every 10 intervals, it will send the text word "UPDATE"
 * to the server and the server will return by sending text from
 * all its buffers.
 */

#include "CAVERN.hxx"

#include <stdlib.h>
#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#endif

/* This is the size of all messages sent from clients to servers. */
#define TRANSMITTED_DATA_SIZE 512

/* This is the size of all messages sent from servers to clients. */
#define BROADCAST_DATA_SIZE 1024

char sendBuffer[TRANSMITTED_DATA_SIZE];
char receiveBuffer[BROADCAST_DATA_SIZE];


void main (int argc, char *argv[])
{
	int tic = 0;
	int status, dataSize;

	CAVERNinit();

	CAVERNnet_tcpClient_c *client = new CAVERNnet_tcpClient_c;
	if (client->connectToServer(argv[1], 6544) < 0){
		exit(1);
	}

    /*to create a logfile for storing performance monitoring results 
      (logfile = "Machine_IP + Port")
     */
    char* fileName = new char[256];
    char IP[256];
    int port;
    int fileOpenOK =0;

    client->getSelfIP(IP);
    port = client->getSelfPort();
 
    sprintf(fileName, "%s_%d.log", IP, port);    
    FILE* filePtr;
    filePtr = fopen(fileName, "a");
    if (filePtr)
    {        
        fileOpenOK =1;
    }


	while(1) {

		char name[256];
		client->getSelfIP(name);
		printf(">>> %s,%u\n", name, client->getSelfPort());

		tic++;
		if (tic % 10)
		sprintf(sendBuffer,"TICK(%d)",tic);
		else
		sprintf(sendBuffer,"UPDATE");

		dataSize = TRANSMITTED_DATA_SIZE;
		status = client->write(sendBuffer, &dataSize,
				       CAVERNnet_tcpClient_c::BLOCKING);

if (status != CAVERNnet_tcpClient_c::OK) {
			client->showStatus(status, dataSize);
		}

		dataSize = BROADCAST_DATA_SIZE;

		status =  client->read(receiveBuffer, &dataSize,
				 CAVERNnet_tcpClient_c::BLOCKING);

		if (status == CAVERNnet_tcpClient_c::OK) {
			printf("Data received: %s\n", receiveBuffer);
            printf("\nPerformance Monitoring Statistics:\n");
            client->showStats("tcpClient_perf_monitoring", "Some_local_data");
          
            //uncomment for storing performance stats in a logfile
            //if (fileOpenOK)
            //if (client->logStats("Some_tcpClient_data", "New_local_data", filePtr)!= CAVERNnet_perfMonitor_c::OK)
            //printf("\nlogStats failed!\n");
   
		} else if (status != CAVERNnet_tcpClient_c::NON_BLOCKING_HAS_NO_DATA)
			client->showStatus(status, dataSize);

#ifdef WIN32
		Sleep(1000);
#else
		sleep(1);
#endif
	}
    if (filePtr)
        fclose(filePtr);
}
