/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

/* This server accepts N client connections.
 * It receives data from the clients and stores it away in their own buffers.
 * If it gets an UPDATE message it will broadcast all its buffers
 * to all the clients.
 */

#include "CAVERN.hxx"



#include <stdlib.h>
#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#endif

/* This is the size of all messages sent from clients to servers. */
#define TRANSMITTED_DATA_SIZE 512

/* This is the size of all messages sent from servers to clients. */
#define BROADCAST_DATA_SIZE 1024


#define MAX_NUM_CLIENTS 10
int numberOfClients = 0;

CAVERNnet_tcpClient_c *client[MAX_NUM_CLIENTS];
char clientData[MAX_NUM_CLIENTS][TRANSMITTED_DATA_SIZE];

void InitClientData()
{
	int i, j;
	for (i = 0; i < MAX_NUM_CLIENTS; i++) {
		for (j = 0; j < TRANSMITTED_DATA_SIZE; j++) {
			clientData[i][j] = '\0';
		}
	}
}

CAVERNnet_tcpServer_c *server;

void main (int argc, char *argv[])
{

	CAVERNinit();

	InitClientData();

	server = new CAVERNnet_tcpServer_c;
	if (server->init(6544) == 0)	{
		exit(1);
	}

	/* Now perform routing task between network connections. */
	void PerformRouting();

	PerformRouting();


}


void PerformRouting()
{

	char dataBuffers[MAX_NUM_CLIENTS][TRANSMITTED_DATA_SIZE];
	int i;
	void InterpretData(int clientNum, char *data);

	CAVERNnet_tcpClient_c *aClient;
	while(1) {

		/* If a connection is established, accept it and create
		 * a new socket for that connection to communicate with.
		 *
		 */
		aClient = server->checkForNewConnections();

		if (aClient) {
			printf("Client %d connected\n", numberOfClients); fflush(stdout);

			char name[256];
			aClient->getSelfIP(name);
			printf(">>> %s,%u\n", name, aClient->getSelfPort());
			client[numberOfClients] = aClient;
			numberOfClients++;
		}

		int dataSize;
		int status;

		for (i = 0; i < numberOfClients; i++) {
			dataSize = TRANSMITTED_DATA_SIZE;
			status = client[i]->read(dataBuffers[i], &dataSize, CAVERNnet_tcpClient_c::NON_BLOCKING);

			if (status == CAVERNnet_tcpClient_c::OK)  {
				printf("Read in data from client %d\n",i);
				InterpretData(i,dataBuffers[i]);
				printf("\nDisplaying Performance statistics\n");
                client[i]->showStats("tcpServer_perf_monitoring", "Some_remoteClient_data");
				fflush(stdout);
			} else if (status != CAVERNnet_tcpClient_c::NON_BLOCKING_HAS_NO_DATA)
				client[i]->showStatus(status, dataSize);
		}
	}
}



void InterpretData(int clientNum, char *data)
{
	char message[BROADCAST_DATA_SIZE];
	int i;
	char temp[1024];
	int dataSize;
	int status;

	strcpy(message,"Client #");

	/* If update command requested, then all buffered data from all clients
	 * sent to client.  Note: the originating client does not get his
	 * data back.
	 */

	if (strcmp(data,"UPDATE") == 0) {
		printf("UPDATE received\n");
		for (i = 0; i < numberOfClients; i++) {
			if (i != clientNum) {
				sprintf(temp,"%d ",i);
				strcat(message,temp);
				strcat(message," ");
				strcat(message,clientData[i]);
				strcat(message," ");
			}
		}
		dataSize = BROADCAST_DATA_SIZE;
		status = client[clientNum]->write(message,
						  &dataSize,
						  CAVERNnet_tcpClient_c::BLOCKING);

		if (status != CAVERNnet_tcpClient_c::OK) client[clientNum]->showStatus(status, dataSize);
		
	} else {
		strcpy(clientData[clientNum], data);

		strcpy(message,"ACKNOWLEDGED");
		dataSize = BROADCAST_DATA_SIZE;

		status =client[clientNum]->write(message,
					    &dataSize,
					    CAVERNnet_tcpClient_c::BLOCKING);

		if (status != CAVERNnet_tcpClient_c::OK) {
			printf("Writing ACKNOWLEDGED to client error.\n");
			client[clientNum]->showStatus(status, dataSize);
		}

		printf("Data for client %d stored: (%s)\n", clientNum, clientData[clientNum]);
	}

	

}

void CloseDownSockets()
{
	int count;

	for (count = 0; count < numberOfClients; count++) {
		client[count]->close();
	}
	server->close();
}
