/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include <stdlib.h>

#include "CAVERN.hxx"
#include "CAVERNnet_perfDaemon_c.hxx"
#include "getopt.hxx"

#define MAX_PATH_LENGTH 256

/** This program implements simple performance monitoring daemon program.
 Users can either save all performance messages into a log file using -f
 option and specifying the log file name or reflects all performance messages
 to the performance monitoring client such as QoSiMoto. 
 You can only use two options with this program. -h option will show you
 the descriptions of options that can be used with this program. If -f 
 option is used with a file name, all performance monitoring messages sent
 to this program will be saved into the log file named as the given file name.
 If no option is used at all, this program will send out all performance
 monitoring messages to the registered performance monitoring clients.
 If no performance monitoring client is connected to this program but several
 clients are connected to send out performance monitoring messages, the 
 messages will be consumed by this program and nothing will be sent out.
*/

char LogFileName[MAX_PATH_LENGTH] = "";
FILE* LogFile = NULL;

/** This function is going to be registered so that it must be called when
 this program exits normally. This must be used so that we can flush out all
 performance monitoring messages and save them into the log file. 
*/
void AtExit(void)
{
    if (LogFile) {
        fflush(LogFile);
	fclose(LogFile);
    }
}

// Shows usage
void Usage(char* ProgramName)
{
	printf("%s [-h|-f FILE_NAME]\n", ProgramName);
	printf("Options:\n");
	printf("h: Help. Shows this message\n");
	printf("f: Specifies log file name. Performance daemon will save all performance\n");
	printf("   monitoring message into the log file.\n");
}

void ProcessCommandLine(int argc, char *argv[])
{
    char optionStr[] = "f:h";
    char opt;
    char* optarg;

    opt = GetOption(argc, argv, optionStr, &optarg);
    while (opt != (char) -1 && opt != (char) 0) {
	switch (opt) {
            case 'f': 
                if (optarg == NULL) {
                    Usage(argv[0]);
                    exit(0);
                }	
		strcpy(LogFileName, optarg);
		break;

            case 'h':
            case (char)-1:
            case (char) 1:
		Usage(argv[0]);
		exit(0);
	}		
	opt = GetOption(argc, argv, optionStr, &optarg);
    }
}

void main(int argc, char* argv[])
{
    CAVERNnet_perfDaemon_c* cavernPerfDaemon;

    if (argc > 1) ProcessCommandLine(argc, argv);

    CAVERNinit();
    if (LogFileName[0] == '\0')
	cavernPerfDaemon = new CAVERNnet_perfDaemon_c;
    else {
        LogFile = fopen(LogFileName, "a+t");

        if (LogFile) {
            cavernPerfDaemon = new CAVERNnet_perfDaemon_c(LogFile);
            atexit(AtExit);
        }
 	else {
            printf("Cannot open file %s\n", LogFileName);
            exit(1);
        } 
    }

    // Initialize cavern reflector
    if (cavernPerfDaemon->init() != CAVERNnet_perfDaemon_c::OK) {
        printf("Cannot initialize CAVERN perfdaemon server\n");
        exit(1);
    }

    while(1) {
        // Perform data reflection services.
        cavernPerfDaemon->process();
    }
}





