/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

/* E.g. of TCP reflector client. This uses the CAVERNnet_tcpReflectorClient_c
 * class designed specifically for use with the CAVERNnet_tcpReflector_c
 * class.
 * This will connect to a server and send text to it. In order to be able to
 * see anything at all you need to have multiple clients connecting to a single
 * server.
 */
#include "CAVERN.hxx"

#include <stdlib.h>
#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#endif

/* This is the size of all messages sent from clients to servers. */
#define TRANSMITTED_DATA_SIZE 512

char sendBuffer[TRANSMITTED_DATA_SIZE];

void main (int argc, char *argv[])
{
	int tic = 0;
	int status, dataSize;

	CAVERNinit();
	CAVERNnet_tcpReflectorClient_c *client = new CAVERNnet_tcpReflectorClient_c;

	if (client->connectToServer(argv[1]) != CAVERNnet_tcpReflectorClient_c::OK)    {
		exit(1);
	}
    int perfDaemonStatus =1;
	if (argc > 2 && client->initSendStats(argv[2]) != CAVERNnet_perfMonitor_c::OK) {
		printf("Cannot connect to %s\n", argv[2]);
		delete client;
		exit(1);
	}
    if (argc <= 2) 
        perfDaemonStatus =0;

    /*to create a logfile for storing performance monitoring results 
      (logfile = "Machine_IP + Port")
     */
    char * fileName = new char[256];
    char IP[256];
    int port;
    int fileOpenOK =0;

    client->getSelfIP(IP);
    port = client->getSelfPort();
  
    
    sprintf(fileName, "%s_%d.log", IP, port);
    
    FILE* filePtr;
    filePtr = fopen(fileName, "a");
    if (filePtr)
    {        
        fileOpenOK =1;
    }
    
	while(1) {
	  		char name[256];
		client->getSelfIP(name);

		tic++;
		sprintf(sendBuffer,"%s TICK(%d)",name, tic);

		dataSize = TRANSMITTED_DATA_SIZE;
		printf("\n***Write***\n");
		status = client->write(sendBuffer, &dataSize);

		if (status != CAVERNnet_tcpReflectorClient_c::OK) {
		  		client->showStatus(status, dataSize);
		}

		dataSize = 0;
		char* receiveBuffer;

		printf("\n***Read***\n");
		status =  client->read(&receiveBuffer, &dataSize,
				 CAVERNnet_tcpReflectorClient_c::NON_BLOCKING);

		if (status == CAVERNnet_tcpReflectorClient_c::OK) {
			printf("Data received: %s\n", receiveBuffer);
	
            //Performance monitoring -ignore the first 10 packets
			if(tic >10)
				{
                    //Uncomment this for an instantaneous display locally
					//client->showStats("CAVE_avatar_on_clark", "New_avatar_entered");					
                    //if (perfDaemonStatus)
                    //{
                        //send stats to a perfdaemon
                        if (client->sendStats("CAVE_avatar_on_clark", "New_avatar_entered") != CAVERNnet_perfMonitor_c::OK)
                            printf("\nsendStats failed!\n");
                   // }
                    
                    //to log stats into a file
                    if (fileOpenOK)
                    {
                        if (client->logStats("CAVE_avatar_on_clark", "New_avatar_entered", filePtr) != CAVERNnet_perfMonitor_c::OK)
                            printf("\nlogStats Failed!\n");
                    }                    
				}
			delete receiveBuffer;

		} else if (status != CAVERNnet_tcpReflectorClient_c::NON_BLOCKING_HAS_NO_DATA)
			client->showStatus(status, dataSize);
        sleep(1);
	}
    fclose(filePtr);
	client->exitSendStats();
	delete client;
}



