/******************************************************************
* CAVERNsoft
* Copyright (C) 1994-2000 Electronic Visualization Laboratory,
* all rights reserved
* By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
* Stuart Bailey, Atul Nayak, Shalini Venkataraman
* University of Illinois at Chicago
* 
* This publication and its text and code may not be copied for commercial 
* use without the express written permission of the University of Illinois
* at Chicago.
* The contributors disclaim any representation of warranty: use this
* code at your own risk.
* Direct questions, comments etc to cavern@evl.uic.edu
******************************************************************/

/* UDP example to connect to a UDP server and send data to it. */

#include "CAVERN.hxx"

#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#endif
#include <stdlib.h>

#define MAXLINE 256

#ifdef WIN32
#define sleep(seconds)	Sleep(seconds * 1000)
#endif

int count = 0;

main(int argc, char** argv)
{
    int port;
	CAVERNinit();

	char recvline[MAXLINE];
	CAVERNnet_udp_c *mainUDPSocket = new CAVERNnet_udp_c;

	// Enable instrumentation
	mainUDPSocket->enableInstrumentation();

	/* Open a dynamic port since we are a client and don't need to
	 * commit to any one port.
	 */
	mainUDPSocket->init();
	
	// We recommend that you use the blocking mode option in 
	// the send() and recieve() instead of calling makeNonBlocking()
	// makeNonBlocking() will be slowly phased out.
	// The demo shows how you can use send() and receive() with blocking mode
	// Comment out the following line if you are using the blockingmode in send() and receive()
	mainUDPSocket->makeNonBlocking();

    if (argv[2])
        port = atoi(argv[2]);
    else port = 6000;        
	/* But we want to use the socket to send data to the server */
	mainUDPSocket->setSendAddress(argv[1],port);

	while(1) {

		char mesg[MAXLINE];
		int n;

		sprintf(mesg,"COUNT %d\n",count++);
		n = strlen(mesg) + 1;

		/* Send message to the server */
		if (mainUDPSocket->send(mesg, n) != n)
//		if (mainUDPSocket->send(mesg, n, CAVERNnet_udp_c::NON_BLOCKING) != n)
			printf("Client sendto error\n");

		/* To receive something in return from the server you can do something
		 * like:
		 */

		n = mainUDPSocket->receive(recvline, MAXLINE);
//		n = mainUDPSocket->receive(recvline, MAXLINE,CAVERNnet_udp_c::NON_BLOCKING);

		if (n < 0)  
			if (n == CAVERNnet_udp_c::NON_BLOCKING_HAS_NO_DATA)
				printf("Nonblocking has no data\n");
			else
				printf("Client receive error\n");
		else
		{
			printf("RECV SIZE %d ",n);
			printf(recvline);
		}
		// Remove this comment to see how performance monitoring is done.		
//		mainUDPSocket->showStats("UDP_Performance_monitoring", "Continuous_data_transfer_Client_Side");
		sleep(10);

	}

}
