/******************************************************************
* CAVERNsoft
* Copyright (C) 1994-2000 Electronic Visualization Laboratory,
* all rights reserved
* By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
* Stuart Bailey, Atul Nayak
* University of Illinois at Chicago
* 
* This publication and its text and code may not be copied for commercial 
* use without the express written permission of the University of Illinois
* at Chicago.
* The contributors disclaim any representation of warranty: use this
* code at your own risk.
* Direct questions, comments etc to cavern@evl.uic.edu
******************************************************************/

/* UDP Server example. Server sets itself up and just listens for
 * incoming data. Then it prints it to the screen and echoes
 * a message back to all the clients that are connected to it.
 */

#include "CAVERN.hxx"
extern int errno;

#ifdef WIN32
#include <string.h>
#else
#include <strings.h>
#endif

#define MAXSIZE 256


main (int argc, char** argv)
{

	CAVERNinit();

	CAVERNnet_udp_c *mainUDPSocket;

	mainUDPSocket = new CAVERNnet_udp_c;

	// Enable instrumentation
	mainUDPSocket->enableInstrumentation();

	/* Open a static port for others to connect to. Hence you are a server */
	mainUDPSocket->init(6000);

	mainUDPSocket->makeNonBlocking();
	
	char recvline[MAXSIZE];

	while(1) {
		while (mainUDPSocket->receive(recvline, MAXSIZE,CAVERNnet_udp_c::BLOCKING) > 0) {

			/* Show who the client is by printing the IP address & port */
			printf("RECEIVED: %s",recvline);
			mainUDPSocket->printInfo();

			/* See if the current client is the 1st time we've encountered it.
			 * If it is then save it away.
			 */
			void CheckClient(CAVERNnet_udp_c *);
			CheckClient(mainUDPSocket);

			/* Send a reply to all the clients. */
			void ReplyToClients();
			ReplyToClients();
	// Remove the following comment and compile the demos again to see 
	// performance statistics		
//	            mainUDPSocket->showStats("UDP_Performance_monitoring", "Continuous_data_transfer_Server_Side");
		}
#ifdef WIN32
		if (errno != WSAEWOULDBLOCK) {
			printf("RECEIVE ERROR\n");
		}
#else
		if (errno != EWOULDBLOCK) {
			printf("RECEIVE ERROR\n");
		}
#endif
	}

}


/* Array of pointers to clients that might connect with us. Allow 5 clients to connect. */
CAVERNnet_udp_c *bunchOfClients[5];
int numberOfClients = 0;

/* Check to see if the current client is one that we have not encountered
 * before.
 */
void CheckClient(CAVERNnet_udp_c *mainUDPSocket)
{

	CAVERNnet_udp_c *eachClient;

	for (int i = 0; i < numberOfClients; i++) {
		eachClient = bunchOfClients[i];
		if ((eachClient->getSendIP() == mainUDPSocket->getReceiveIP()) &&
		    (eachClient->getSendPort() == mainUDPSocket->getReceivePort())) return;
	}
	printf("NEW CLIENT\n\n");
	bunchOfClients[numberOfClients] = mainUDPSocket->clone();
	bunchOfClients[numberOfClients]->copyReceiveAddressToSendAddress();
	numberOfClients++;
}

/* Reply to all clients that are conencted to me. */
void ReplyToClients()
{
	static int ok = 0;

	ok++;
	char mesg[256];
	for(int i = 0 ; i<numberOfClients; i++) {
		sprintf(mesg,"HOWDY CLIENT# %d\tTOTAL CLIENTS: %d MESSAGE: %d\n", i, numberOfClients, ok);
		bunchOfClients[i]->send(mesg, strlen(mesg)+1,CAVERNnet_udp_c::NON_BLOCKING);
	}

}

