/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include <string.h>
#include <iostream.h>
#include "CAVERNdb_sharedState_c.hxx"

CAVERNdb_sharedState_c::CAVERNdb_sharedState_c()
{
	_path = new char[255];
	_path[0] = '\0';
	_key = new char[255];
	_key[0] = '\0';
	_dbclient = 0;
}

CAVERNdb_sharedState_c::CAVERNdb_sharedState_c( CAVERNdb_client_c *dbclient, char *path, char *key )
: _dbclient( dbclient ), _sharingEnabled( false )
{
	_path = new char[255];
	_key = new char[255];
	if (path)
		strcpy( _path, path );
	else
		path[0] = '\0';
	if (key)
		strcpy( _key, key );
	else
		cerr<<"ERROR! CAVERNdb_sharedState_c requires a valid key name.\n";
	if (_dbclient)
		_dbclient->attach( this );
}

CAVERNdb_sharedState_c::~CAVERNdb_sharedState_c()
{
	// do nothing since the pointer to the dbclient is external
	if (_dbclient)
		_dbclient->detach( this );
	delete[] _path;
	delete[] _key;
}

void CAVERNdb_sharedState_c::update( CAVERNmisc_subject_c *subj )
{
	CAVERNdb_client_c *theClient = (CAVERNdb_client_c*) subj;
	// check the path name
	char *path = theClient->getNotifyPathname();
	if (strcmp(_path, path) == 0)
	{
		// check the key name
		char *key = theClient->getNotifyKeyname();
		if (strcmp(_key, key) == 0)
		{
			// it's a match, so get the data
			int size;
			char *data = theClient->get( path, key, &size );
			this->unpackState( data );
			delete data;
		}
		delete key;
	}
	delete path;
}

int CAVERNdb_sharedState_c::refresh()
{
	if (_dbclient)
		return _dbclient->fetch( _path, _key );
	else
		return 0;
}
