/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include "CAVERN.hxx"

int _cvrnDebugLevel= 0;
void cvrnSetDebugFilter(int lvl) {_cvrnDebugLevel = lvl;}

void cvrnPrintf(char *fmt, ...)
{
#ifdef CAVERN_DO_NOT_USE_GLOBUS
	#ifdef WIN32
		EnterCriticalSection(&Win32StdioLock);
	#else
		pthread_mutex_lock(&pthread_stdio_lock); 
	#endif
#else
	globus_stdio_lock();
#endif
	va_list args;
	va_start(args,fmt);
	vprintf(fmt,args);
	fflush(stdout);
#ifdef CAVERN_DO_NOT_USE_GLOBUS
	#ifdef WIN32
		LeaveCriticalSection(&Win32StdioLock);
	#else
		pthread_mutex_unlock(&pthread_stdio_lock);
	#endif
#else
	globus_stdio_unlock();
#endif
}

void cvrnDebug(char* prefix, int lvl, char *fmt, ...)
{
#ifdef CAVERN_DO_NOT_USE_GLOBUS
	#ifdef WIN32
		EnterCriticalSection(&Win32StdioLock);
	#else
		pthread_mutex_lock(&pthread_stdio_lock);
	#endif
#else
		globus_stdio_lock();
#endif
		va_list args;
		va_start(args,fmt);
		if (prefix)
		printf("    cavern(%d) %s: ", lvl, prefix);
		else
		printf("    cavern(%d) ", lvl);
		vprintf(fmt,args);
		printf("\n");

		fflush(stdout);
#ifdef CAVERN_DO_NOT_USE_GLOBUS
	#ifdef WIN32
		LeaveCriticalSection(&Win32StdioLock);
	#else
		pthread_mutex_unlock(&pthread_stdio_lock);
	#endif
#else
		globus_stdio_unlock();
#endif
}

void cvrnMesg(char* prefix, char *fmt, ...)
{
#ifdef CAVERN_DO_NOT_USE_GLOBUS
	#ifdef WIN32
		EnterCriticalSection(&Win32StdioLock);
	#else
		pthread_mutex_lock(&pthread_stdio_lock);
	#endif
#else
	globus_stdio_lock();
#endif
	va_list args;
	va_start(args,fmt);
	if (prefix)
		printf("CAVERN: %s: ", prefix);
	else
		printf("CAVERN: ");
	vprintf(fmt,args);
	printf("\n");

	fflush(stdout);
#ifdef CAVERN_DO_NOT_USE_GLOBUS
	#ifdef WIN32
		LeaveCriticalSection(&Win32StdioLock);
	#else
		pthread_mutex_unlock(&pthread_stdio_lock);
	#endif
#else
	globus_stdio_unlock();
#endif
}

