/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/
#if (defined(__sgi)|| defined(linux) || defined(sun))

#include "CAVERNmisc_fileList_c.hxx"

const int CAVERNmisc_fileInfo_c::OK = 0;
const int CAVERNmisc_fileInfo_c::FAILED = -1;
const int CAVERNmisc_fileInfo_c::FILE_TYPE = FTW_F;
const int CAVERNmisc_fileInfo_c::DIRECTORY_TYPE = FTW_D;
const int CAVERNmisc_fileInfo_c::DIRECTORY_TYPE_CANNOT_BE_READ = FTW_DNR;

CAVERNmisc_fileInfo_c::CAVERNmisc_fileInfo_c()
{
    fileName = NULL;
    leafName = NULL;
    objectType = FILE_TYPE;
}

CAVERNmisc_fileInfo_c::~CAVERNmisc_fileInfo_c()
{
    if (fileName)
        delete [] fileName;
    if (leafName)
        delete [] leafName;
}

void CAVERNmisc_fileInfo_c::setLeafName(char* path)
{
     char slash;
     #ifndef WIN32
        slash = '/';
     #else
        slash = '\\';          
     #endif
                                                     
    if (fileName)
    {
        int currentDirectory = 0;
        int actualPathLen = strlen(path); 
        int leafLength = strlen(fileName) - strlen(path); 
        if (leafLength == 0) currentDirectory =1;  //path is the same as the filename
        if (leafLength == -1) currentDirectory =1;  //path is the same as the filename but the file name does not have the slash at the end
        
        if (fileName[actualPathLen] == slash)
        {
            if (leafLength == 1) currentDirectory =1;  //path is the same as the filename but does not have the slash at the end
            actualPathLen ++;
        }
        if (currentDirectory)     // leafname is given the value "."
        {
            leafName = new char[2];
            leafName[0] = '.';
            leafName[1] = '\0';
            return;
        }
        leafName = new char[leafLength+1]; 
        
        for (int i=0; i<leafLength; i++)
            leafName[i] = fileName[ actualPathLen + i];
        leafName[leafLength] = '\0';        
    } 
}
void CAVERNmisc_fileInfo_c::setFileInfo(char* path, char* fName, long fSize, long tStamp, int type)
{
    fileName = new char[strlen(fName) + 1];
    strcpy(fileName, fName);
    setLeafName(path);
    fileSize = fSize;
    timeStamp = tStamp;
    objectType = type;
    if (type == FTW_DP)         //nftw returns a FTW_DP type which means the directory has been visited
        objectType = DIRECTORY_TYPE;
}

void CAVERNmisc_fileInfo_c::getFileName(char*& fName)
{
    if (fileName)
    {
        fName = new char[strlen(fileName)+1];
        strcpy(fName, fileName);
    }
    else    
        fName =NULL;    
}

void CAVERNmisc_fileInfo_c::getLeafName(char*& lName)
{
    if (leafName)
    {
        lName = new char[strlen(leafName)+1];
        strcpy(lName, leafName);
    }
    else
        lName = NULL;
}

long CAVERNmisc_fileInfo_c::getFileSize()
{
    return fileSize;
}

long CAVERNmisc_fileInfo_c::getTimeStamp()
{
    return timeStamp;
}

int CAVERNmisc_fileInfo_c::getObjectType()
{
    return objectType;
}

int CAVERNmisc_fileInfo_c::getPackingSize()
{
    CAVERNnet_datapack_c packer;
    //return the packing size for file name size, file name, file size, time stamp and the object type
    return (packer.sizeof_int() + strlen(fileName) + packer.sizeof_long() + packer.sizeof_long() + packer.sizeof_int());
}

int CAVERNmisc_fileInfo_c::packFileInfo(CAVERNnet_datapack_c* packer)
{
    //Pack the members -check if space is enough and return status
   if (packer->packInt(strlen(fileName)) == CAVERNnet_datapack_c::FAILED)
        return FAILED;
   if (packer->pack(fileName, strlen(fileName)) == CAVERNnet_datapack_c::FAILED)
        return FAILED;
   if (packer->packLong(fileSize) == CAVERNnet_datapack_c::FAILED)
        return FAILED;
   if (packer->packLong(timeStamp) == CAVERNnet_datapack_c::FAILED)
        return FAILED;
   if (packer->packInt(objectType) == CAVERNnet_datapack_c::FAILED)
        return FAILED;
   return OK;     
}

int CAVERNmisc_fileInfo_c::unpackFileInfo(CAVERNnet_datapack_c* packer, char* path)
{
    //Un pack the file name size, file name, file size, time stamp and object type 
    int filenameSize;
    if (packer->unpackInt(&filenameSize) == CAVERNnet_datapack_c::FAILED)
        return FAILED;
    if (fileName)
    {
        delete [] fileName;
        fileName = NULL;
    }
    fileName = new char[filenameSize+1];
    if (packer->unpack(fileName, filenameSize) == CAVERNnet_datapack_c::FAILED)
        return FAILED;
    fileName[filenameSize] = '\0';
    //get the leaf name from the path and the file name    
    setLeafName(path);
    if (packer->unpackLong(&fileSize) == CAVERNnet_datapack_c::FAILED)
        return FAILED;
    if (packer->unpackLong(&timeStamp) == CAVERNnet_datapack_c::FAILED)
        return FAILED;
    if (packer->unpackInt(&objectType) == CAVERNnet_datapack_c::FAILED)
        return FAILED;
    return OK;
}

void CAVERNmisc_fileInfo_c::displayFileInfo()
{
    char* charTime = ctime((const time_t*)&timeStamp);
    printf("\n%s\t%ld  %s  ", leafName, fileSize, charTime);
    if (objectType == FILE_TYPE)
        printf("File");
    if (objectType == DIRECTORY_TYPE)
        printf("Directory");
    if (objectType == DIRECTORY_TYPE_CANNOT_BE_READ)
        printf("Directory (not readable)");
        
}
/*************************************************************/

const int CAVERNmisc_fileList_c::OK = 0;
const int CAVERNmisc_fileList_c::FAILED = -1;
const int CAVERNmisc_fileList_c::LIST_RECURSIVELY = 2;
const int CAVERNmisc_fileList_c::SKIP_RECURSION= 3;
const int CAVERNmisc_fileList_c::TRAVERSE_LINKS = 4;
const int CAVERNmisc_fileList_c::DO_NOT_TRAVERSE_LINKS = 5;

CAVERNmisc_fileList_c* CAVERNmisc_fileList_c::thisInstance = NULL;

CAVERNmisc_fileList_c::CAVERNmisc_fileList_c()
{
    fileList = NULL;
    I_ALLOCATED_MEMORY = 0; //filelist not yet allocated
    path = NULL;
    noFiles =0;
    depth = 25;
}

CAVERNmisc_fileList_c::~CAVERNmisc_fileList_c()
{
    if ( fileList && I_ALLOCATED_MEMORY)
        delete [] fileList;
    if (path)
        delete [] path;
}

void CAVERNmisc_fileList_c::setDepth(int newDepth)
{
    depth = newDepth;
}

int CAVERNmisc_fileList_c::buildFileList(char* dirName, int recursion, int linkTraversal)
{
    int flags;
    doRecursion = recursion;
    path = new char[strlen(dirName)+1];
    strcpy(path, dirName);
    //Do a File Tree Walk - giving a default depth of 25 - this would find out the number of file objects in the directory
    thisInstance = this;
    if (linkTraversal == DO_NOT_TRAVERSE_LINKS)
        flags =  FTW_PHYS;
    else
        flags = FTW_DEPTH;
        
    int result = nftw(path, ftwDummyCallBack, depth, flags);
    
    if (result != 0)   
        return FAILED;
    
    fileList = new CAVERNmisc_fileInfo_c[noFiles];    
    I_ALLOCATED_MEMORY = 1;      //Remember to delete
    fileCount = 0;
    thisInstance = this;
    result = nftw(path, ftwCallBack, depth, flags);
    if (result !=0)        
        return FAILED;
    return OK;    
}

int CAVERNmisc_fileList_c::ftwDummyCallBack(const char* fname, const struct stat* statBuf, int objectType, struct FTW* ftwstruct)
{
    if (thisInstance->doRecursion == LIST_RECURSIVELY)    //list all files recursively -search subdirectories
    {
        (thisInstance->noFiles)++;
        return 0;
    }
    else                                    //skip recursion                                
    {
        //Check the depth of the file - ignore files and directories with depth greater than 1     
        char* fileName = new char[strlen(fname)+1];
        strcpy(fileName, fname);
        char* absFilename = fileName + strlen(thisInstance->path);
        char slash;
        #ifndef WIN32
            slash = '/';
        #else
            slash = '\\';          
        #endif
        
        if (absFilename[0] == slash)          //If a '/' is not part of the given path
            absFilename++;
        char* nextSlash = strchr(absFilename, (int)slash);
        if (nextSlash != NULL)              //Depth of the file is greater than 1 - ignore the file
            {delete [] fileName; return 0;}
        delete [] fileName;
        (thisInstance->noFiles)++;                                 
        return 0;
    }    
}

int CAVERNmisc_fileList_c::ftwCallBack(const char* fname, const struct stat* statBuf, int objectType, struct FTW* ftwstruct)
{
    char* fileName = new char[strlen(fname)+1];
    strcpy(fileName, fname);            
    if (thisInstance->doRecursion == SKIP_RECURSION)    //skip recursion
    {
        //Check the depth of the file - ignore files and directories with depth greater than 1     
        char* absFilename = fileName + strlen(thisInstance->path);
        char slash;
        #ifndef WIN32
            slash = '/';
        #else
            slash = '\\';          
        #endif
        
        if (absFilename[0] == slash)          //If a '/' is not part of the given path
            absFilename++;
        char* nextSlash = strchr(absFilename, (int)slash);
        if (nextSlash != NULL)              //Depth of the file is greater than 1 - ignore the file
            {delete [] fileName; return 0;}
 	       
	#ifdef linux
		thisInstance->fileList[thisInstance->fileCount].setFileInfo(thisInstance->path, fileName, statBuf->st_size, (statBuf->st_ctime), objectType);
        #else
		thisInstance->fileList[thisInstance->fileCount].setFileInfo(thisInstance->path, fileName, statBuf->st_size, (statBuf->st_ctim).tv_sec, objectType);
	#endif

	(thisInstance->fileCount)++;
        delete [] fileName;
        return 0;        
    }
    else //doRecursion = LIST_RECURSIVELY    -> list all files recursively 
    {                  
	#ifdef linux
		thisInstance->fileList[thisInstance->fileCount].setFileInfo(thisInstance->path, fileName, statBuf->st_size, (statBuf->st_ctime), objectType);
        #else
		thisInstance->fileList[thisInstance->fileCount].setFileInfo(thisInstance->path, fileName, statBuf->st_size, (statBuf->st_ctim).tv_sec, objectType);
	#endif

        (thisInstance->fileCount)++;    
        delete fileName;
        return 0;
    }
}

void CAVERNmisc_fileList_c::displayFileList()
{
    for(int i=0; i<noFiles; i++)
        fileList[i].displayFileInfo();
    printf("\n");    
}

int CAVERNmisc_fileList_c::getNoFiles()
{
    return noFiles;
}

int CAVERNmisc_fileList_c::getDataPackingSize()
{
    int packingSize =0;
    for (int i=0; i<noFiles; i++)
        packingSize+= fileList[i].getPackingSize();
    return packingSize;
}

int CAVERNmisc_fileList_c::packFileList(CAVERNnet_datapack_c* packer)
{
    if (!packer)
        {
            printf("\n Data packing instance should be initialized with the CAVERNnet_datapack_c::initPack function. The datapacking size can be got by using the getDataPackingSize() function\n");
            return FAILED;
        }
    for (int i=0; i<noFiles; i++)
    {
        if (fileList[i].packFileInfo(packer) == CAVERNmisc_fileInfo_c::FAILED)
            return FAILED; 
    }
    return OK;
}

int CAVERNmisc_fileList_c::unpackfileList(CAVERNnet_datapack_c* packer, char* path, CAVERNmisc_fileInfo_c* list, int fileCount)
{
    if (!packer)
        {
            printf("\n Data packing instance should be initialized with the CAVERNnet_datapack_c::initPack function. The datapacking size can be got by using the getDataPackingSize() function\n");
            return FAILED;
        }
    noFiles = fileCount;
    if (fileList)
        delete [] fileList;
    fileList = list;
    I_ALLOCATED_MEMORY = 0;     //do not delete the filelist
    for (int i=0; i<noFiles; i++)
        {
             if (fileList[i].unpackFileInfo(packer, path) == CAVERNmisc_fileInfo_c::FAILED)
                return FAILED;
        }
    return OK;    
}
#endif
