/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman 
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include "CAVERNnet_datapack_c.hxx"
#include "CAVERNnet_extendedParallelTcp_c.hxx"

CAVERNnet_extendedParallelTcpClient_c::CAVERNnet_extendedParallelTcpClient_c()
{
	CAVERNnet_datapack_c packer;

	char_size = sizeof(char);	
	int_size = packer.sizeof_int();
	int32_size = packer.sizeof_int32();
	int64_size = packer.sizeof_int64();
	long_size = packer.sizeof_long();
	float_size = packer.sizeof_float();
	double_size = packer.sizeof_double();


	int_buffer = new char[int_size];
	int32_buffer = new char[int32_size];
	int64_buffer = new char[int64_size];
	long_buffer = new char[long_size];
	float_buffer = new char[float_size];
	double_buffer = new char[double_size];
}

CAVERNnet_extendedParallelTcpClient_c::~CAVERNnet_extendedParallelTcpClient_c()
{
	if (int_buffer) delete [] int_buffer;
	if (int32_buffer) delete [] int32_buffer;
	if (int64_buffer) delete [] int64_buffer;
	if (long_buffer) delete [] long_buffer;
	if (float_buffer) delete [] float_buffer;
	if (double_buffer) delete [] double_buffer;
}

int CAVERNnet_extendedParallelTcpClient_c::readChar(char& value)
{
	int nbytes = char_size;
	return read((char*) &value, &nbytes);
}

int CAVERNnet_extendedParallelTcpClient_c::writeChar(char value)
{
	int nbytes = sizeof(char);
	return write((char*) &value, &nbytes);
}

int CAVERNnet_extendedParallelTcpClient_c::readInt(int& value)
{
	CAVERNnet_datapack_c packer;
	int nbytes = int_size;
	int status  = read(int_buffer , &nbytes);
    packer.initUnpack(int_buffer,nbytes);
    packer.unpackInt(&value);          
	return status;
}

int CAVERNnet_extendedParallelTcpClient_c::readIntArray(int* intArray, int size)
{
    CAVERNnet_datapack_c packer;
	int nbytes = int_size*size;
	char* readBuffer = new char[nbytes];
	int status  = read(readBuffer , &nbytes);
    packer.initUnpack(readBuffer,nbytes);
    for (int index = 0; index < size; index++) {
        packer.unpackInt(&intArray[index]);
    }
    delete readBuffer;
	return status;
}

int CAVERNnet_extendedParallelTcpClient_c::writeInt(int value)
{
	CAVERNnet_datapack_c packer;
	int nbytes = int_size;                          
                                                                 
    packer.initPack(int_buffer, nbytes);             
    packer.packInt(value);
	int status = write(int_buffer, &nbytes);
	return status;
}

int CAVERNnet_extendedParallelTcpClient_c::writeIntArray(int* intArray, int size)
{
    CAVERNnet_datapack_c packer;
    int nbytes = int_size*size;
    char*sendBuffer = new char[nbytes];
    
    packer.initPack(sendBuffer, nbytes);
    for (int index=0; index<size; index++)
    {
        packer.packInt(intArray[index]);
    }
    int status = write(sendBuffer, &nbytes);
    delete sendBuffer;
    return status;
}

int CAVERNnet_extendedParallelTcpClient_c::readInt32(int32& value)
{
	CAVERNnet_datapack_c packer;
	int nbytes = int32_size;
	int status  = read(int32_buffer , &nbytes);
    packer.initUnpack(int32_buffer,nbytes);
    packer.unpackInt32(&value);          
	return status;
}

int CAVERNnet_extendedParallelTcpClient_c::readInt32Array(int32* int32Array, int size)
{
    CAVERNnet_datapack_c packer;
	int nbytes = int32_size*size;
    char* readBuffer = new char[nbytes];
    int status  = read(readBuffer , &nbytes);
    packer.initUnpack(readBuffer,nbytes);
    for (int index=0; index<size; index++)
    {
        packer.unpackInt32(&int32Array[index]);
    }
    delete readBuffer;
    return status;
}

int CAVERNnet_extendedParallelTcpClient_c::writeInt32(int32 value)
{
	CAVERNnet_datapack_c packer;
	int nbytes = int32_size;                          
                                                                 
    packer.initPack(int32_buffer, nbytes);             
    packer.packInt32(value);
	int status = write(int32_buffer, &nbytes);
	return status;
}

int CAVERNnet_extendedParallelTcpClient_c::writeInt32Array(int32* int32Array, int size)
{
    CAVERNnet_datapack_c packer;
    int nbytes = int32_size*size;
    char* sendBuffer = new char[nbytes];
    packer.initPack(sendBuffer, nbytes);
    for (int index =0; index<size; index++)
    {
        packer.packInt32(int32Array[index]);
    }
    int status = write(sendBuffer, &nbytes);
	delete sendBuffer;
    return status;
}

int CAVERNnet_extendedParallelTcpClient_c::readInt64(int64& value)
{
	CAVERNnet_datapack_c packer;
	int nbytes = int64_size;
	int status  = read(int64_buffer , &nbytes);
    packer.initUnpack(int64_buffer, nbytes);
    packer.unpackInt64(&value);          
	return status;
}

int CAVERNnet_extendedParallelTcpClient_c::readInt64Array(int64* int64Array, int size)
{
    CAVERNnet_datapack_c packer;
	int nbytes = int64_size*size;
    char* readBuffer = new char[nbytes];
    int status  = read(readBuffer , &nbytes);
    packer.initUnpack(readBuffer, nbytes);
    for (int index =0; index<size; index++)
    {
        packer.unpackInt64(&int64Array[index]);
    }
    delete readBuffer;
    return status;
}

int CAVERNnet_extendedParallelTcpClient_c::writeInt64(int64 value)
{
	CAVERNnet_datapack_c packer;
	int nbytes = int64_size;                                                               
    packer.initPack(int64_buffer, nbytes);             
    packer.packInt64(value);
	int status = write(int64_buffer, &nbytes);
	return status;
}

int  CAVERNnet_extendedParallelTcpClient_c::writeInt64Array(int64* int64Array, int size)
{
    CAVERNnet_datapack_c packer;
    int nbytes = int64_size *size;   
    char* sendBuffer = new char[nbytes];
    packer.initPack(sendBuffer, nbytes);
    for (int index=0; index<size;index++)
    {
        packer.packInt64(int64Array[index]);
    }
    int status = write(sendBuffer, &nbytes);
    delete sendBuffer;
    return status;
}

int CAVERNnet_extendedParallelTcpClient_c::readLong(long& value)
{
	CAVERNnet_datapack_c packer;
	int nbytes = long_size;
	int status  = read(long_buffer , &nbytes);
    packer.initUnpack(long_buffer,nbytes);
    packer.unpackLong(&value);          
	return status;
}

int CAVERNnet_extendedParallelTcpClient_c::readLongArray(long* longArray, int size)
{
    CAVERNnet_datapack_c packer;
	int nbytes = long_size*size;
    char* readBuffer = new char[nbytes];
    int status  = read(readBuffer , &nbytes);
    packer.initUnpack(readBuffer,nbytes);
    for (int index=0; index<size; index++)
    {
        packer.unpackLong(&longArray[index]);
    }
    delete readBuffer;
    return status;
}

int CAVERNnet_extendedParallelTcpClient_c::writeLong(long value)
{
	CAVERNnet_datapack_c packer;
	int nbytes = long_size;                                            
    packer.initPack(long_buffer, nbytes);             
    packer.packLong(value);
	int status = write(long_buffer, &nbytes);
	return status;
}

int CAVERNnet_extendedParallelTcpClient_c::writeLongArray(long* longArray, int size)
{
    CAVERNnet_datapack_c packer;
    int nbytes = long_size*size;
    char* sendBuffer = new char[nbytes];
    packer.initPack(sendBuffer, nbytes); 
    for (int index=0; index<size; index++)
    {
        packer.packLong(longArray[index]);
    }
    int status = write(sendBuffer, &nbytes);
    delete sendBuffer;
    return status;
}

int CAVERNnet_extendedParallelTcpClient_c::readFloat(float& value)
{
	CAVERNnet_datapack_c packer;
	int nbytes = float_size;
	int status  = read(float_buffer , &nbytes);
    packer.initUnpack(float_buffer,nbytes);
    packer.unpackFloat(&value);          
	return status;
}

int CAVERNnet_extendedParallelTcpClient_c::readFloatArray(float* floatArray, int size)
{
    CAVERNnet_datapack_c packer;
    int nbytes = float_size*size;
    char* readBuffer = new char[nbytes];
    int status  = read(readBuffer , &nbytes);
    packer.initUnpack(readBuffer,nbytes);
    for (int index=0; index<size; index++)
    {
        packer.unpackFloat(&floatArray[index]);
    }
    delete readBuffer;
    return status;
}

int CAVERNnet_extendedParallelTcpClient_c::writeFloat(float value)
{
	CAVERNnet_datapack_c packer;
	int nbytes = float_size;                          
                                                                 
    packer.initPack(float_buffer, nbytes);             
    packer.packFloat(value);
	int status = write(float_buffer, &nbytes);
	return status;
}

int CAVERNnet_extendedParallelTcpClient_c::writeFloatArray(float* floatArray, int size)
{
    CAVERNnet_datapack_c packer;
    int nbytes = float_size*size;
    char* sendBuffer = new char[nbytes];
    packer.initPack(sendBuffer, nbytes);  
    for (int index=0; index<size; index++)
    {
        packer.packFloat(floatArray[index]);
    }
    int status = write(sendBuffer, &nbytes);
    delete sendBuffer;
    return status;
}

int CAVERNnet_extendedParallelTcpClient_c::readDouble(double& value)
{
	CAVERNnet_datapack_c packer;
	int nbytes = double_size;
	int status  = read(double_buffer , &nbytes);
    packer.initUnpack(double_buffer,nbytes);
    packer.unpackDouble(&value);          
	return status;
}

int CAVERNnet_extendedParallelTcpClient_c::writeDouble(double value)
{
	CAVERNnet_datapack_c packer;
	int nbytes = double_size;                          
                                                                 
    packer.initPack(double_buffer, nbytes);             
    packer.packDouble(value);
	int status = write(double_buffer, &nbytes);
	return status;
}

int CAVERNnet_extendedParallelTcpClient_c::readDoubleArray(double* doubleArray, int size)
{
    CAVERNnet_datapack_c packer;
	int nbytes = double_size*size;
    char* readBuffer = new char[nbytes];
    int status  = read(readBuffer , &nbytes);
    packer.initUnpack(readBuffer,nbytes);
    for(int index=0; index<size; index++)
    {
        packer.unpackDouble(&doubleArray[index]);
    }
    delete readBuffer;
    return status;
}

int CAVERNnet_extendedParallelTcpClient_c::writeDoubleArray(double* doubleArray, int size)
{
    CAVERNnet_datapack_c packer;
	int nbytes = double_size*size;
    char* sendBuffer = new char[nbytes];
    packer.initPack(sendBuffer, nbytes);             
    for (int index=0; index<size;index++)
    {
        packer.packDouble(doubleArray[index]);
    }
    int status = write(sendBuffer, &nbytes);
	delete sendBuffer;
    return status;
}

CAVERNnet_extendedParallelTcpClient_c *CAVERNnet_extendedParallelTcpServer_c::checkForNewConnections()
{
	struct sockaddr_in clientInfo;
	int cliLen = sizeof(sockaddr_in);
	CAVERNnet_extendedParallelTcpClient_c* newClient;
	CAVERNnet_datapack_c packer;
	unsigned int iBufSize = packer.sizeof_int() + packer.sizeof_int();
	char* pcBuffer = new char[iBufSize];
	int iSize;
	int data1;
	int* sockets;
	int setupSockfd;
	
	memset(pcBuffer, 0, iBufSize);
	memset(&clientInfo, 0, sizeof(struct sockaddr_in));

	setupSockfd = accept(sockfd, (struct sockaddr*) &clientInfo, &cliLen);
#ifdef WIN32
	if (setupSockfd != INVALID_SOCKET) {
#else
	if (setupSockfd >= 0) {
#endif
		int len = recv(setupSockfd, pcBuffer, iBufSize, 0);
		packer.initUnpack(pcBuffer, iBufSize);
		packer.unpackInt(&data1);
		packer.unpackInt(&iSize);

		sockets = new int[iSize];
		for (int i = 0; i < iSize; i++) {
		  if (connectToClient(clientInfo.sin_addr.s_addr, data1, sockets[i]) == CAVERNnet_parallelTcpServer_c::FAILED) {
#ifdef WIN32
			  closesocket(setupSockfd);
				for (int j = 0; j < i; j++) closesocket(sockets[j]);
#else
				::close(setupSockfd);
				for (int j = 0; j < i; j++) ::close(sockets[j]);
#endif
				delete [] pcBuffer;
				return NULL;
			}
			else {
			  makeNonBlocking(sockets[i]);
			}
		}
		newClient = new CAVERNnet_extendedParallelTcpClient_c;
		newClient->setSockets(sockets, iSize);
		delete [] pcBuffer;
#ifdef WIN32
		closesocket(setupSockfd);
#else
		::close(setupSockfd);
#endif
		return newClient;
	}
	delete pcBuffer;
	return NULL;
}
