/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include "CAVERNnet_datapack_c.hxx"
#include "CAVERNnet_extendedTcp_c.hxx"

CAVERNnet_extendedTcpClient_c::CAVERNnet_extendedTcpClient_c()
{
	CAVERNnet_datapack_c packer;

	char_size = sizeof(char);	
	int_size = packer.sizeof_int();
	int32_size = packer.sizeof_int32();
	int64_size = packer.sizeof_int64();
	long_size = packer.sizeof_long();
	float_size = packer.sizeof_float();
	double_size = packer.sizeof_double();


	int_buffer = new char[int_size];
	int32_buffer = new char[int32_size];
	int64_buffer = new char[int64_size];
	long_buffer = new char[long_size];
	float_buffer = new char[float_size];
	double_buffer = new char[double_size];
}

CAVERNnet_extendedTcpClient_c::~CAVERNnet_extendedTcpClient_c()
{
	if (int_buffer) delete int_buffer;
	if (int32_buffer) delete int32_buffer;
	if (int64_buffer) delete int64_buffer;
	if (long_buffer) delete long_buffer;
	if (float_buffer) delete float_buffer;
	if (double_buffer) delete double_buffer;
}

int CAVERNnet_extendedTcpClient_c::readChar(char& value, int blockingType)
{
	int nbytes = char_size;
	return read((char*) &value, &nbytes, blockingType);
}

int CAVERNnet_extendedTcpClient_c::writeChar(char value, int blockingType)
{
	int nbytes = sizeof(char);
	return write((char*) &value, &nbytes, blockingType);
}

int CAVERNnet_extendedTcpClient_c::readInt(int& value, int blockingType)
{
	CAVERNnet_datapack_c packer;
	int nbytes = int_size;
	int status  = read(int_buffer , &nbytes, blockingType);
    packer.initUnpack(int_buffer,nbytes);
    packer.unpackInt(&value);          
	return status;
}

int CAVERNnet_extendedTcpClient_c::readIntArray(int* intArray, int size, int blockingType)
{
    CAVERNnet_datapack_c packer;
	int nbytes = int_size*size;
	char* readBuffer = new char[nbytes];
	int status  = read(readBuffer , &nbytes, blockingType);
    packer.initUnpack(readBuffer,nbytes);
    for (int index=0; index<size; index++)
    {
        packer.unpackInt(&intArray[index]);
    }
    delete readBuffer;
	return status;
}

int CAVERNnet_extendedTcpClient_c::writeInt(int value, int blockingType)
{
	CAVERNnet_datapack_c packer;
	int nbytes = int_size;                          
                                                                 
    packer.initPack(int_buffer, nbytes);             
    packer.packInt(value);
	int status = write(int_buffer, &nbytes, blockingType);
	return status;
}

int CAVERNnet_extendedTcpClient_c::writeIntArray(int* intArray, int size, int blockingType)
{
    CAVERNnet_datapack_c packer;
    int nbytes = int_size*size;
    char*sendBuffer = new char[nbytes];
    
    packer.initPack(sendBuffer, nbytes);
    for (int index=0; index<size; index++)
    {
        packer.packInt(intArray[index]);
    }
    int status = write(sendBuffer, &nbytes, blockingType);
    delete sendBuffer;
    return status;
}

int CAVERNnet_extendedTcpClient_c::readInt32(int32& value, int blockingType)
{
	CAVERNnet_datapack_c packer;
	int nbytes = int32_size;
	int status  = read(int32_buffer , &nbytes, blockingType);
    packer.initUnpack(int32_buffer,nbytes);
    packer.unpackInt32(&value);          
	return status;
}

int CAVERNnet_extendedTcpClient_c::readInt32Array(int32* int32Array, int size, int blockingType)
{
    CAVERNnet_datapack_c packer;
	int nbytes = int32_size*size;
    char* readBuffer = new char[nbytes];
    int status  = read(readBuffer , &nbytes, blockingType);
    packer.initUnpack(readBuffer,nbytes);
    for (int index=0; index<size; index++)
    {
        packer.unpackInt32(&int32Array[index]);
    }
    delete readBuffer;
    return status;
}

int CAVERNnet_extendedTcpClient_c::writeInt32(int32 value, int blockingType)
{
	CAVERNnet_datapack_c packer;
	int nbytes = int32_size;                          
                                                                 
    packer.initPack(int32_buffer, nbytes);             
    packer.packInt32(value);
	int status = write(int32_buffer, &nbytes, blockingType);
	return status;
}

int CAVERNnet_extendedTcpClient_c::writeInt32Array(int32* int32Array, int size, int blockingType)
{
    CAVERNnet_datapack_c packer;
    int nbytes = int32_size*size;
    char* sendBuffer = new char[nbytes];
    packer.initPack(sendBuffer, nbytes);
    for (int index =0; index<size; index++)
    {
        packer.packInt32(int32Array[index]);
    }
    int status = write(sendBuffer, &nbytes, blockingType);
	delete sendBuffer;
    return status;
}

int CAVERNnet_extendedTcpClient_c::readInt64(int64& value, int blockingType)
{
	CAVERNnet_datapack_c packer;
	int nbytes = int64_size;
	int status  = read(int64_buffer , &nbytes, blockingType);
    packer.initUnpack(int64_buffer, nbytes);
    packer.unpackInt64(&value);          
	return status;
}

int CAVERNnet_extendedTcpClient_c::readInt64Array(int64* int64Array, int size, int blockingType)
{
    CAVERNnet_datapack_c packer;
	int nbytes = int64_size*size;
    char* readBuffer = new char[nbytes];
    int status  = read(readBuffer , &nbytes, blockingType);
    packer.initUnpack(readBuffer, nbytes);
    for (int index =0; index<size; index++)
    {
        packer.unpackInt64(&int64Array[index]);
    }
    delete readBuffer;
    return status;
}


int CAVERNnet_extendedTcpClient_c::writeInt64(int64 value, int blockingType)
{
	CAVERNnet_datapack_c packer;
	int nbytes = int64_size;                                                               
    packer.initPack(int64_buffer, nbytes);             
    packer.packInt64(value);
	int status = write(int64_buffer, &nbytes, blockingType);
	return status;
}

int  CAVERNnet_extendedTcpClient_c::writeInt64Array(int64* int64Array, int size, int blockingType)
{
    CAVERNnet_datapack_c packer;
    int nbytes = int64_size *size;   
    char* sendBuffer = new char[nbytes];
    packer.initPack(sendBuffer, nbytes);
    for (int index=0; index<size;index++)
    {
        packer.packInt64(int64Array[index]);
    }
    int status = write(sendBuffer, &nbytes, blockingType);
	delete sendBuffer;
    return status;
}

int CAVERNnet_extendedTcpClient_c::readLong(long& value, int blockingType)
{
	CAVERNnet_datapack_c packer;
	int nbytes = long_size;
	int status  = read(long_buffer , &nbytes, blockingType);
    packer.initUnpack(long_buffer,nbytes);
    packer.unpackLong(&value);          
	return status;
}

int CAVERNnet_extendedTcpClient_c::readLongArray(long* longArray, int size, int blockingType)
{
    CAVERNnet_datapack_c packer;
	int nbytes = long_size*size;
    char* readBuffer = new char[nbytes];
    int status  = read(readBuffer , &nbytes, blockingType);
    packer.initUnpack(readBuffer,nbytes);
    for (int index=0; index<size; index++)
    {
        packer.unpackLong(&longArray[index]);
    }
    delete readBuffer;
    return status;
}

int CAVERNnet_extendedTcpClient_c::writeLong(long value, int blockingType)
{
	CAVERNnet_datapack_c packer;
	int nbytes = long_size;                                            
    packer.initPack(long_buffer, nbytes);             
    packer.packLong(value);
	int status = write(long_buffer, &nbytes, blockingType);
	return status;
}

int CAVERNnet_extendedTcpClient_c::writeLongArray(long* longArray, int size, int blockingType)
{
    CAVERNnet_datapack_c packer;
    int nbytes = long_size*size;
    char* sendBuffer = new char[nbytes];
    packer.initPack(sendBuffer, nbytes); 
    for (int index=0; index<size; index++)
    {
        packer.packLong(longArray[index]);
    }
    int status = write(sendBuffer, &nbytes, blockingType);
    delete sendBuffer;
    return status;
}

int CAVERNnet_extendedTcpClient_c::readFloat(float& value, int blockingType)
{
	CAVERNnet_datapack_c packer;
	int nbytes = float_size;
	int status  = read(float_buffer , &nbytes, blockingType);
    packer.initUnpack(float_buffer,nbytes);
    packer.unpackFloat(&value);          
	return status;
}

int CAVERNnet_extendedTcpClient_c::readFloatArray(float* floatArray, int size, int blockingType)
{
    CAVERNnet_datapack_c packer;
    int nbytes = float_size*size;
    char* readBuffer = new char[nbytes];
    int status  = read(readBuffer , &nbytes, blockingType);
    packer.initUnpack(readBuffer,nbytes);
    for (int index=0; index<size; index++)
    {
        packer.unpackFloat(&floatArray[index]);
    }
    delete readBuffer;
    return status;
}

int CAVERNnet_extendedTcpClient_c::writeFloat(float value, int blockingType)
{
	CAVERNnet_datapack_c packer;
	int nbytes = float_size;                          
                                                                 
    packer.initPack(float_buffer, nbytes);             
    packer.packFloat(value);
	int status = write(float_buffer, &nbytes, blockingType);
	return status;
}

int CAVERNnet_extendedTcpClient_c::writeFloatArray(float* floatArray, int size, int blockingType)
{
    CAVERNnet_datapack_c packer;
    int nbytes = float_size*size;
    char* sendBuffer = new char[nbytes];
    packer.initPack(sendBuffer, nbytes);  
    for (int index=0; index<size; index++)
    {
        packer.packFloat(floatArray[index]);
    }
    int status = write(sendBuffer, &nbytes, blockingType);
    delete sendBuffer;
    return status;
}

int CAVERNnet_extendedTcpClient_c::readDouble(double& value, int blockingType)
{
	CAVERNnet_datapack_c packer;
	int nbytes = double_size;
	int status  = read(double_buffer , &nbytes, blockingType);
    packer.initUnpack(double_buffer,nbytes);
    packer.unpackDouble(&value);          
	return status;
}

int CAVERNnet_extendedTcpClient_c::readDoubleArray(double* doubleArray, int size, int blockingType)
{
    CAVERNnet_datapack_c packer;
	int nbytes = double_size*size;
    char* readBuffer = new char[nbytes];
    int status  = read(readBuffer , &nbytes, blockingType);
    packer.initUnpack(readBuffer,nbytes);
    for(int index=0; index<size; index++)
    {
        packer.unpackDouble(&doubleArray[index]);
    }
    delete readBuffer;
    return status;
}

int CAVERNnet_extendedTcpClient_c::writeDouble(double value, int blockingType)
{
	CAVERNnet_datapack_c packer;
	int nbytes = double_size;                          
                                                                 
    packer.initPack(double_buffer, nbytes);             
    packer.packDouble(value);
	int status = write(double_buffer, &nbytes, blockingType);
	return status;
}

int CAVERNnet_extendedTcpClient_c::writeDoubleArray(double* doubleArray, int size, int blockingType)
{
    CAVERNnet_datapack_c packer;
	int nbytes = double_size*size;
    char* sendBuffer = new char[nbytes];
    packer.initPack(sendBuffer, nbytes);             
    for (int index=0; index<size;index++)
    {
        packer.packDouble(doubleArray[index]);
    }
    int status = write(sendBuffer, &nbytes, blockingType);
	delete sendBuffer;
    return status;
}

CAVERNnet_extendedTcpClient_c *CAVERNnet_extendedTcpServer_c::checkForNewConnections()
{
	struct sockaddr_in clientInfo;
	int clilen;

	// Perform a non-blocking check to see if socket is ready for
	// an incoming connection.
	fd_set read_set;
	struct timeval timeout;
	timeout.tv_sec = 0;
	timeout.tv_usec = 0;
	int ready_fd;

	FD_ZERO(&read_set);
	FD_SET(sockfd, &read_set);
	ready_fd = select( sockfd+1, &read_set, NULL, NULL, &timeout);

	// If it is not ready then return.
	if ((ready_fd <= 0)  ||  (!FD_ISSET( sockfd, &read_set))) return NULL;

	clilen = sizeof(clientInfo);
	int newsockfd = accept(sockfd,
			       (struct sockaddr *) &clientInfo,
			       &clilen);
	if (newsockfd > 0) {
		struct linger lingerData;
		lingerData.l_onoff = 1;
		lingerData.l_linger = 1000;

		int optlen = sizeof(lingerData);

		if (setsockopt(newsockfd,SOL_SOCKET,SO_LINGER, (char *) &lingerData, optlen) < 0)
			printf("CAVERNnet_tcpServer_c::checkForNewConnections: setsockopt: SO_LINGER failed.\n");

		CAVERNnet_extendedTcpClient_c *clientObj;

		clientObj = new CAVERNnet_extendedTcpClient_c;
		clientObj->setTimeOut(timeOutPeriod);
		clientObj->setClientInfo(&clientInfo);
		clientObj->setClientSockFd(newsockfd);

		return clientObj;
	} else return NULL;
}
