/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include "CAVERN.hxx"

#ifndef WIN32
#include <netinet/tcp.h>
#endif

#include <stdio.h>

#define TCPBUFSIZE 4470000 
//#define TCPBUFSIZE 32677
#define SSOCKET_MINPORT 5000
#define SSOCKET_MAXPORT 10000

const int CAVERNnet_parallelTcpServer_c::OK = 1;
const int CAVERNnet_parallelTcpServer_c::FAILED = 0;

#ifdef sun
long gethostid(void);
#endif

#ifdef WIN32
#define MAXHOSTNAME   128   /* Maximum length of a DNS hostname */

// This function is excerpted from "Windows Sockets Network Programming"
// by Bob Quinn and Dave Shute published from Addison-Wesley.
static long gethostid(void)
{
	char szLclHost [MAXHOSTNAME];
	LPHOSTENT lpstHostent;
	SOCKADDR_IN stLclAddr;
	SOCKADDR_IN stRmtAddr;
	int nAddrSize = sizeof(SOCKADDR);
	SOCKET hSock;
	int nRet;

	/* Init local address (to zero) */
	stLclAddr.sin_addr.s_addr = INADDR_ANY;

	/* Get the local hostname */
	nRet = gethostname(szLclHost, MAXHOSTNAME); 
	if (nRet != SOCKET_ERROR) {
		/* Resolve hostname for local address */
		lpstHostent = gethostbyname((LPSTR)szLclHost);
		if (lpstHostent)
			stLclAddr.sin_addr.s_addr = *((u_long FAR*) (lpstHostent->h_addr));
	} 

	/* If still not resolved, then try second strategy */
	if (stLclAddr.sin_addr.s_addr == INADDR_ANY) {
		/* Get a UDP socket */
		hSock = socket(AF_INET, SOCK_DGRAM, 0);
		if (hSock != INVALID_SOCKET)  {
			/* Connect to arbitrary port and address (NOT loopback) */
			stRmtAddr.sin_family = AF_INET;
			stRmtAddr.sin_port   = htons(IPPORT_ECHO);
			stRmtAddr.sin_addr.s_addr = inet_addr("128.127.50.1");
			nRet = connect(hSock, (LPSOCKADDR)&stRmtAddr,
						   sizeof(SOCKADDR));
			if (nRet != SOCKET_ERROR) {
				/* Get local address */
				getsockname(hSock, (LPSOCKADDR)&stLclAddr, 
							(int FAR*)&nAddrSize);
			}
			closesocket(hSock);   /* we're done with the socket */
		}
	}
	return (stLclAddr.sin_addr.s_addr);
}
#endif

int CAVERNnet_parallelTcpServer_c::init(int inPort)
{
	int optlen;
	int iBindReturn = -1;
	int iTcpBufSize = TCPBUFSIZE;

#ifdef WIN32
	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {
		printf("socket = %d: can't open stream socket. Error #: %d\n", 
			   sockfd, WSAGetLastError());
		return CAVERNnet_parallelTcpServer_c::FAILED;
	}
#else
	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
			printf("socket = %d: can't open stream socket\n", sockfd);
			return CAVERNnet_parallelTcpServer_c::FAILED;
	}
#endif

	/* Set no delay to stop Tcp/IP from buffering */
	int noDelay = 0;// Set to 0 for no buffering. Set to 1 for buffering.
	optlen = sizeof(noDelay);

	if (setsockopt(sockfd,IPPROTO_TCP,TCP_NODELAY, (char *) &noDelay, optlen) < 0) 
		printf("CAVERNnet_parallelTcpServer_c::init: Cannot set no delay, you may experience sluggish performance.\n");

	/* Use this to try to eliminate the problem of being unable to rebind to a static port */
	int reuseFlag = 1;
	optlen = sizeof(reuseFlag);
	if (setsockopt(sockfd,SOL_SOCKET,SO_REUSEADDR, (char *) &reuseFlag, optlen) < 0) 
		printf("CAVERNnet_parallelTcpServer_c::init: Cannot reuse port address.\n");

	/***** Bind our local address so that the client can send to us. */

	struct linger lingerData;
	lingerData.l_onoff = 1;
	lingerData.l_linger = 1000;

	optlen = sizeof(lingerData);
	if (setsockopt(sockfd,SOL_SOCKET,SO_LINGER, (char *) &lingerData, optlen) < 0)
		printf("CAVERNnet_parallelTcpServer_c::init: setsockopt: SO_LINGER failed.\n");

	/* Clear out the structure */
	memset((char *) &serverInfo, 0, sizeof(serverInfo));

	serverInfo.sin_family = AF_INET;
	serverInfo.sin_addr.s_addr = htonl(INADDR_ANY);
	serverInfo.sin_port = htons(inPort);
	iBindReturn = bind(sockfd, (struct sockaddr*) &serverInfo, sizeof(struct sockaddr_in));

	if (iBindReturn < 0) {
		printf("CAVERNnet_parallelTcpServer_c::init: can't bind to local address\n");
#ifdef WIN32
		printf("Errno: %d", WSAGetLastError());
#else
		printf("Errno: %d %s\n", errno, strerror(errno));
#endif
		return CAVERNnet_parallelTcpServer_c::FAILED;
	}

#if (defined(WIN32)) || (defined(sun))
	if (setsockopt(sockfd, SOL_SOCKET, SO_RCVBUF, (char*) &iTcpBufSize, sizeof(iTcpBufSize)) < 0)
		printf("Cannot set the receiving buffer size to %d\n", iTcpBufSize); 
	if (setsockopt(sockfd, SOL_SOCKET, SO_SNDBUF, (char*) &iTcpBufSize, sizeof(iTcpBufSize)) < 0)
		printf("Cannot set the sending buffer size to %d\n", iTcpBufSize); 
#else
	if (setsockopt(sockfd, SOL_SOCKET, SO_RCVBUF, &iTcpBufSize, sizeof(iTcpBufSize)) < 0)
		printf("Cannot set the receiving buffer size to %d\n", iTcpBufSize);
	if (setsockopt(sockfd, SOL_SOCKET, SO_SNDBUF, &iTcpBufSize, sizeof(iTcpBufSize)) < 0)
		printf("Cannot set the sending buffer size to %d\n", iTcpBufSize); 
#endif

	int iRetVal = listen(sockfd, 10);
#ifdef WIN32
	if (iRetVal == SOCKET_ERROR) printf("WinSock Error #: %d\n", WSAGetLastError());
#else
	if (iRetVal <0)	printf("%d %s\n",errno,strerror(errno));
#endif

	return CAVERNnet_parallelTcpServer_c::OK;
}

int CAVERNnet_parallelTcpServer_c::connectToClient(unsigned long RemoteAddr, unsigned short RemotePort, 
												   int& Sockfd)
{
	struct sockaddr_in remoteAddr;

	memset(&remoteAddr, 0, sizeof(struct sockaddr_in));

	remoteAddr.sin_family = AF_INET;
	remoteAddr.sin_addr.s_addr = RemoteAddr;
	remoteAddr.sin_port = htons(RemotePort);

#ifdef WIN32
	if ((Sockfd = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {
		printf("CAVERNnet_parallelTcpServer::connectToClient: can't open stream socket\n");
		return CAVERNnet_parallelTcpServer_c::FAILED;
	}
#else
	if ((Sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		printf("CAVERNnet_parallelTcpServer::connectToClient: can't open stream socket\n");
		return CAVERNnet_parallelTcpServer_c::FAILED;
	}
#endif

	if (connect(Sockfd, (struct sockaddr*) &remoteAddr, sizeof(struct sockaddr_in)) < 0) {
#ifdef WIN32
		printf("CAVERNnet_parallelTcpServer::connectToClient: can't connect to client to establish parallel connections. Error code = %d\n", 
			   WSAGetLastError());
		closesocket(Sockfd);
#else
		printf("CAVERNnet_parallelTcpServer::connectToClient: can't connect to client to establish parallel connections. Error code = %d\n", 
			   errno);
		::close(Sockfd);
#endif
		return CAVERNnet_parallelTcpServer_c::FAILED;
	}

	return CAVERNnet_parallelTcpServer_c::OK;
}

CAVERNnet_parallelTcpClient_c *CAVERNnet_parallelTcpServer_c::checkForNewConnections()
{
	struct sockaddr_in clientInfo;
	int cliLen = sizeof(sockaddr_in);
	CAVERNnet_parallelTcpClient_c* newClient;
	CAVERNnet_datapack_c packer;
	unsigned int iBufSize = packer.sizeof_int() + packer.sizeof_int();
	char* pcBuffer = new char[iBufSize];
	int iSize;
	int data1;
	int* sockets;
	int setupSockfd;
	
	memset(pcBuffer, 0, iBufSize);
	memset(&clientInfo, 0, sizeof(struct sockaddr_in));

	setupSockfd = accept(sockfd, (struct sockaddr*) &clientInfo, &cliLen);
#ifdef WIN32
	if (setupSockfd != INVALID_SOCKET) {
#else
	if (setupSockfd >= 0) {
#endif
		int len = recv(setupSockfd, pcBuffer, iBufSize, 0);
		packer.initUnpack(pcBuffer, iBufSize);
		packer.unpackInt(&data1);
		packer.unpackInt(&iSize);
		
		sockets = new int[iSize];
		for (int i = 0; i < iSize; i++) {
			if (connectToClient(clientInfo.sin_addr.s_addr, data1, sockets[i]) == CAVERNnet_parallelTcpServer_c::FAILED) {
#ifdef WIN32
				closesocket(setupSockfd);
				for (int j = 0; j < i; j++) closesocket(sockets[j]);
#else
				::close(setupSockfd);
				for (int j = 0; j < i; j++) ::close(sockets[j]);
#endif
				return NULL;
			}
			else {
                 makeNonBlocking(sockets[i]);
            }
		}
		newClient = new CAVERNnet_parallelTcpClient_c;
		newClient->setSockets(sockets, iSize);

#ifdef WIN32
		closesocket(setupSockfd);
#else
		::close(setupSockfd);
#endif
		return newClient;
	}
	return NULL;
}

unsigned int CAVERNnet_parallelTcpServer_c::getSelfIP()
{
#ifdef linux
    if (serverInfo.sin_addr.s_addr) return serverInfo.sin_addr.s_addr;
    else {
        unsigned short a1, a2;
        unsigned long ip, temp;

        ip = gethostid();
        a1 = (unsigned short) (ip >> 16);
        a2 = (unsigned short) ip;
        temp = (unsigned long) a2;
        temp <<= 16;
        temp |= a1;
        return temp;
    }
#else
	return (serverInfo.sin_addr.s_addr) ? serverInfo.sin_addr.s_addr : gethostid();
#endif
}

void CAVERNnet_parallelTcpServer_c::makeNonBlocking(int Sockfd)
{
#ifdef WIN32
    u_long nonBlockingMode = 1L;

    ioctlsocket(Sockfd, FIONBIO, &nonBlockingMode);
#else
	int iValue;

	if((iValue = fcntl(Sockfd, F_GETFL, 0)) < 0 ) {
		printf("Error in getting options\n"); fflush(stdout); 
		printf("Errorno=%d, %s \n",errno,strerror(errno));
    }
#ifdef __sgi
	iValue |= FNDELAY;
#endif
#ifdef linux
	iValue |= O_NONBLOCK;
#endif
#ifdef sun
	iValue |= O_NDELAY;
#endif
	if(fcntl(Sockfd, F_SETFL, iValue) < 0) {
	    printf("Error in setting options\n"); fflush(stdout);
    }
#endif
}

const int CAVERNnet_parallelTcpClient_c::OK = 1;
const int CAVERNnet_parallelTcpClient_c::FAILED = -1;

CAVERNnet_parallelTcpClient_c::CAVERNnet_parallelTcpClient_c() 
            : sockets(NULL), maxSockDesc(-1), numSockets(0), 
			  pcReadPtr(NULL), pcWritePtr(NULL), piBytesLeft(NULL)
{
}

CAVERNnet_parallelTcpClient_c::~CAVERNnet_parallelTcpClient_c()
{
	if (sockets) close();
}

void CAVERNnet_parallelTcpClient_c::setSockets(int* Sockets, int NumSockets) 
{ 
    sockets = Sockets;
    numSockets = NumSockets; 
    for (int i = 0; i < NumSockets; i++) {
        if (sockets[i] > maxSockDesc) maxSockDesc = sockets[i];
    }
    pcReadPtr = new char *[NumSockets];
    pcWritePtr = new char *[NumSockets];
    piBytesLeft = new int[NumSockets];
}

int CAVERNnet_parallelTcpClient_c::write(char* buffer, int* length)
{
	int iNumSockets = numSockets; 
    int bytesSent = 0;

	if(*length < 5000) {
		iNumSockets = 1;
	}        

	int i;
	int size = *length / iNumSockets;
	piBytesLeft[0] = size;
	pcWritePtr[0] = buffer;
	for(i = 1; i < iNumSockets; i++) {
		piBytesLeft[i] = size; 
		pcWritePtr[i] = pcWritePtr[i - 1] + size;//buffer + (i * size); 
	}
	piBytesLeft[i - 1] += *length%iNumSockets;

	while (*length > 0) {
		FD_ZERO(&fdSet);
		for (i = 0; i < iNumSockets; i++) {
		    if (piBytesLeft[i] > 0)
		        FD_SET(sockets[i], &fdSet);
		}
		int iSelect = select((maxSockDesc+1), (fd_set *)NULL , &fdSet, (fd_set *)NULL, (struct timeval *)NULL);
        for (i = 0; i < iNumSockets; i++) {
			if (FD_ISSET(sockets[i], &fdSet)) {
				//if (piBytesLeft[i] > 0) {
					int iLength = send(sockets[i], pcWritePtr[i], piBytesLeft[i], 0);

					if (iLength < 0) {
#ifdef WIN32
						if (WSAGetLastError() != WSAEWOULDBLOCK) {
							printf("CAVERNnet_parallelTcpClient_c::write: Error code = %d\n", WSAGetLastError());
#else
						if (errno != EWOULDBLOCK) {
							printf("CAVERNnet_parallelTcpClient_c::write: Error code = %d\n", errno);
#endif
							*length = bytesSent;
							return CAVERNnet_parallelTcpClient_c::FAILED;
						}
					}

				//	printf("iLength = %d\n", iLength);
					bytesSent += iLength;	

					piBytesLeft[i] -= iLength;
					pcWritePtr[i] += iLength;
					*length -= iLength;
				}
			//}
		}
	}
	*length = bytesSent;
	return CAVERNnet_parallelTcpClient_c::OK;
}	      
	      
  
int CAVERNnet_parallelTcpClient_c::read(char* buffer, int* length)
{
	int iNumSockets = numSockets;
	int bytesRead = 0;

	if (*length < 5000) {
		iNumSockets = 1;
	}             

	int i;
	int size = *length / iNumSockets;

	piBytesLeft[0] = size;
	pcReadPtr[0] = buffer;
	for (i = 1; i < iNumSockets; i++) {
		piBytesLeft[i] = size;
		pcReadPtr[i] = pcReadPtr[i - 1] + size;//buffer + (i*piBytesLeft[i]); 
	}
	piBytesLeft[i-1] += *length % iNumSockets;


	while (*length > 0) {
		FD_ZERO(&fdSet);
		for (i = 0; i < iNumSockets; i++) {
		    if (piBytesLeft[i] > 0)
		        FD_SET(sockets[i], &fdSet);
		}
		int iSelect = select((maxSockDesc+1), &fdSet , (fd_set *)NULL, (fd_set *)NULL, (struct timeval *)NULL);
		for (i = 0; i < iNumSockets; i++) {
			if (FD_ISSET(sockets[i], &fdSet)) {
				//if (piBytesLeft[i] > 0) {
					int iLength = recv(sockets[i], pcReadPtr[i], piBytesLeft[i], 0);

					if (iLength < 0) {
#ifdef WIN32
						if (WSAGetLastError() != WSAEWOULDBLOCK) {
#else
						if (errno != EWOULDBLOCK) {
#endif
							*length = bytesRead;
							return CAVERNnet_parallelTcpClient_c::FAILED;
						}
					}

					bytesRead += iLength;	

					piBytesLeft[i] -= iLength;
					pcReadPtr[i] += iLength;
					*length -= iLength;
				//}
			}
		}
	}
	*length = bytesRead;
	return CAVERNnet_parallelTcpClient_c::OK;
}

void CAVERNnet_parallelTcpClient_c::close()
{
	for (int i = 0; i < numSockets; i++) {
		if (sockets) {
#ifdef WIN32
			if (sockets[i]) closesocket(sockets[i]);
#else
			if (sockets[i]) ::close(sockets[i]);
#endif
			sockets[i] = 0;
		}
	}
	if (piBytesLeft) {
		delete [] piBytesLeft;
		piBytesLeft = NULL;
	}
	if (pcReadPtr) {
		delete [] pcReadPtr;
		pcReadPtr = NULL;
	}
	if (pcWritePtr) {
		delete [] pcWritePtr;
		pcWritePtr = NULL;
	}
	if (sockets) {
		delete [] sockets;
		sockets = NULL;
	}
}

int CAVERNnet_parallelTcpClient_c::connectToServer(char *RemoteName, int RemotePort, int Size)
{
	int setupSocket;
	int waitSockfd;
    CAVERNnet_datapack_c packer;
	int iTryPort = SSOCKET_MINPORT - 1;
	int iBindReturn = -1;
	int iTcpBufSize = TCPBUFSIZE;

#ifdef WIN32
	if ((waitSockfd = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {
		printf("CAVERNnet_parallelTcpClient::connectToServer: can't open stream socket\n");
		return CAVERNnet_parallelTcpClient_c::FAILED;
	}
#else
	if ((waitSockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		printf("CAVERNnet_parallelTcpClient::connectToServer: can't open stream socket\n");
		return CAVERNnet_parallelTcpClient_c::FAILED;
	}
#endif

	int reuseFlag = 1;
	int optlen = sizeof(reuseFlag);
	if (setsockopt(waitSockfd, SOL_SOCKET, SO_REUSEADDR, (char*) &reuseFlag, optlen) < 0)
		printf("CAVERNnet_parallelTcpClient_c::connectToServer: Cannot reuse port address.\n");

	memset(&localAddr, 0, sizeof(struct sockaddr_in));
	localAddr.sin_family = AF_INET;
	localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	do {
		iTryPort++;
		localAddr.sin_port = htons(iTryPort);
		iBindReturn = bind(waitSockfd, (struct sockaddr*) &localAddr, sizeof(struct sockaddr_in));
	} while (iBindReturn < 0 && iTryPort < SSOCKET_MAXPORT);

	if (iBindReturn < 0) {
		printf("CAVERNnet_parallelTcpClient_c::connectToServer: can't bind to local address\n");
#ifdef WIN32
		printf("Errno: %d", WSAGetLastError());
#else
		printf("Errno: %d %s\n", errno, strerror(errno));
#endif
		return CAVERNnet_parallelTcpClient_c::FAILED;
	}

#if (defined(WIN32)) || (defined(sun))
	if (setsockopt(waitSockfd, SOL_SOCKET, SO_RCVBUF, (char*) &iTcpBufSize, sizeof(iTcpBufSize)) < 0)
		printf("Cannot set the receiving buffer size to %d\n", iTcpBufSize); 
	if (setsockopt(waitSockfd, SOL_SOCKET, SO_SNDBUF, (char*) &iTcpBufSize, sizeof(iTcpBufSize)) < 0)
		printf("Cannot set the sending buffer size to %d\n", iTcpBufSize); 
#else
	if (setsockopt(waitSockfd, SOL_SOCKET, SO_RCVBUF, &iTcpBufSize, sizeof(iTcpBufSize)) < 0)
		printf("Cannot set the receiving buffer size to %d\n", iTcpBufSize);
	if (setsockopt(waitSockfd, SOL_SOCKET, SO_SNDBUF, &iTcpBufSize, sizeof(iTcpBufSize)) < 0)
		printf("Cannot set the sending buffer size to %d\n", iTcpBufSize); 
#endif

	int iRetVal = listen(waitSockfd, 10);
#ifdef WIN32
	if (iRetVal == SOCKET_ERROR) printf("WinSock Error #: %d\n", WSAGetLastError());
#else
	if (iRetVal <0)	printf("%d %s\n",errno,strerror(errno));
#endif

	if (connectToClient(RemoteName, RemotePort, setupSocket) == CAVERNnet_parallelTcpClient_c::FAILED) 
		return CAVERNnet_parallelTcpClient_c::FAILED;

	sockets = new int[Size] ;
	pcReadPtr = new char *[Size];
	pcWritePtr = new char *[Size];
	piBytesLeft = new int[Size];

	int iBufSize = packer.sizeof_int() + packer.sizeof_int();

	char *pcBuffer = new char[iBufSize];  
	memset(pcBuffer, 0, iBufSize);

	packer.initPack(pcBuffer,iBufSize);
	packer.packInt(iTryPort);
	packer.packInt(Size);

	if (send(setupSocket, pcBuffer, iBufSize, 0) < iBufSize) {
#ifdef WIN32
		printf("Cannot send enough data to create parallel connections. Error code = %d\n", WSAGetLastError());
#else
		printf("Cannot send enough data to create parallel connections. Error code = %d\n", errno);
#endif
		delete sockets; delete pcReadPtr; delete pcWritePtr; delete piBytesLeft;
		return CAVERNnet_parallelTcpClient_c::FAILED;
	}
	delete [] pcBuffer;

	int remoteAddrLen = sizeof(struct sockaddr_in);
	for(int i = 0; i < Size; i++) {
		memset(&remoteAddr, 0, remoteAddrLen);
		sockets[i] = accept(waitSockfd, (struct sockaddr*) &remoteAddr, &remoteAddrLen);
#ifdef WIN32
		if (sockets[i] != INVALID_SOCKET) {
#else
		if (sockets[i] >= 0) {
#endif
			makeNonBlocking(sockets[i]);
			if(sockets[i] > maxSockDesc) maxSockDesc = sockets[i];
		}
		else {
			printf("CAVERNnet_parallelTcpClient_c::connectToServer: accept failed\n");
			delete sockets; delete pcReadPtr; delete pcWritePtr; delete piBytesLeft;
			return CAVERNnet_parallelTcpClient_c::FAILED;
		}
	}

	numSockets = Size;

#ifdef WIN32
	closesocket(setupSocket);
	closesocket(waitSockfd);
#else
	::close(setupSocket);
	::close(waitSockfd);
#endif

	return CAVERNnet_parallelTcpClient_c::OK;
}

void CAVERNnet_parallelTcpClient_c::makeNonBlocking(int Sockfd)
{
#ifdef WIN32
    u_long nonBlockingMode = 1L;

    if (ioctlsocket(Sockfd, FIONBIO, &nonBlockingMode) < 0)
		printf("Error in making non-blocking mode socket, errno = %d\n", WSAGetLastError());
#else
	int iValue;

	if((iValue = fcntl(Sockfd, F_GETFL, 0)) < 0 ) {
		printf("Error in getting options\n"); fflush(stdout); 
		printf("Errorno=%d, %s \n",errno,strerror(errno));
    }
#ifdef __sgi
	iValue |= FNDELAY;
#endif
#ifdef linux
	iValue |= O_NONBLOCK;
#endif
#ifdef sun
	iValue |= O_NDELAY;
#endif
	if(fcntl(Sockfd, F_SETFL, iValue) < 0) {
	    printf("Error in setting options\n"); fflush(stdout);
    }
#endif
}

int CAVERNnet_parallelTcpClient_c::connectToClient(char* RemoteName, unsigned short RemotePort, 
												   int& Sockfd)
{
    struct sockaddr_in RemoteAddr;
    char remoteIP[128];
	int status = hostnameToIP(RemoteName, remoteIP);

	if (status == 0) {
		printf("CAVERNnet_parallelTcpClient_c::connectToClient: Host name %s is invalid.\n", RemoteName);
		return CAVERNnet_parallelTcpClient_c::FAILED;
	}

	memset(&RemoteAddr, 0, sizeof(struct sockaddr_in));

	RemoteAddr.sin_family = AF_INET;
	RemoteAddr.sin_addr.s_addr = inet_addr(remoteIP);
	RemoteAddr.sin_port = htons(RemotePort);

#ifdef WIN32
	if ((Sockfd = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {
		printf("CAVERNnet_parallelTcpServer::connectToClient: can't open stream socket\n");
		return CAVERNnet_parallelTcpClient_c::FAILED;
	}
#else
	if ((Sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		printf("CAVERNnet_parallelTcpServer::connectToClient: can't open stream socket\n");
		return CAVERNnet_parallelTcpClient_c::FAILED;
	}
#endif

	if (connect(Sockfd, (struct sockaddr*) &RemoteAddr, sizeof(struct sockaddr_in)) < 0) {
#ifdef WIN32
		printf("CAVERNnet_parallelTcpClient::connectToClient: can't connect to server. Error code = %d\n", WSAGetLastError());
		closesocket(Sockfd);
#else
		printf("CAVERNnet_parallelTcpClient::connectToClient: can't connect to server. Error code = %d\n", errno);
		::close(Sockfd);
#endif
		return CAVERNnet_parallelTcpClient_c::FAILED;
	}

	return CAVERNnet_parallelTcpClient_c::OK;
}

unsigned int CAVERNnet_parallelTcpClient_c :: getRemoteIP() {
	return remoteAddr.sin_addr.s_addr;
}

void CAVERNnet_parallelTcpClient_c :: getRemoteIP(char* name)
{
	ipNumToString(getRemoteIP(),name);

}

unsigned int CAVERNnet_parallelTcpClient_c :: getSelfIP(){
	return (localAddr.sin_addr.s_addr) ? localAddr.sin_addr.s_addr : gethostid();
}

void CAVERNnet_parallelTcpClient_c :: getSelfIP(char *name)
{
	ipNumToString(getSelfIP(),name);
}

int CAVERNnet_parallelTcpClient_c :: getSelfPort() {
	return ntohs(localAddr.sin_port);
}

/* Get the port # of the client connection */
int CAVERNnet_parallelTcpClient_c :: getRemotePort() {
	return ntohs(remoteAddr.sin_port);
}

