/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include "CAVERNnet_perfClient_c.hxx"
#include "CAVERNnet_datapack_c.hxx"
#include "CAVERNnet_tcp_c.hxx"

const int CAVERNnet_perfDaemonClient_c::OK = 1;
const int CAVERNnet_perfDaemonClient_c::FAILED = 2;
const int CAVERNnet_perfDaemonClient_c::MEM_ALLOC_ERR = 3;
const int CAVERNnet_perfDaemonClient_c::NON_BLOCKING_HAS_NO_DATA = 7;
const int CAVERNnet_perfDaemonClient_c::CONNECTION_TERMINATED=8;
const int CAVERNnet_perfDaemonClient_c::NOT_READY=9;
const int CAVERNnet_perfDaemonClient_c::READY_TO_READ=10;
const int CAVERNnet_perfDaemonClient_c::READY_TO_WRITE=11;
const int CAVERNnet_perfDaemonClient_c::READY_TO_READ_AND_WRITE=12;
const int CAVERNnet_perfDaemonClient_c::BLOCKING=CAVERNnet_tcpClient_c::BLOCKING;
const int CAVERNnet_perfDaemonClient_c::NON_BLOCKING=CAVERNnet_tcpClient_c::NON_BLOCKING;
const int CAVERNnet_perfDaemonClient_c::NO_TIME_OUT=CAVERNnet_tcpClient_c::NO_TIME_OUT;

CAVERNnet_perfDaemonClient_c :: CAVERNnet_perfDaemonClient_c(int isPerfClient)
{
	connectionAlive = 0;
	client = new CAVERNnet_tcpClient_c;
    perfClient = isPerfClient;
	// Turn the timeout off. In general this is necessary because in a
	// TCP stream if you lose some data you won't be able to figure out
	// where the head of the next message will be.
	if (client) client->setTimeOut(CAVERNnet_perfDaemonClient_c::NO_TIME_OUT);
	timeOut = CAVERNnet_perfDaemonClient_c::NO_TIME_OUT;
}

CAVERNnet_perfDaemonClient_c::CAVERNnet_perfDaemonClient_c(CAVERNnet_tcpClient_c *aclient, int isPerfClient)
{
	connectionAlive = 1;
	client = aclient;
    perfClient = isPerfClient;
	if (client) client->setTimeOut(CAVERNnet_perfDaemonClient_c::NO_TIME_OUT);
	timeOut = CAVERNnet_perfDaemonClient_c::NO_TIME_OUT;
}

void CAVERNnet_perfDaemonClient_c :: setTimeOut(int timeout)
{
	if (client) {
		client->setTimeOut(timeout);
		timeOut =  timeout;
	}
}

CAVERNnet_perfDaemonClient_c :: ~CAVERNnet_perfDaemonClient_c()
{
	if (client) delete client;
}

int CAVERNnet_perfDaemonClient_c :: connectToServer(char *ip, int port)
{
	if (client && (client->connectToServer(ip,port) >= 0)) {
		connectionAlive = 1;

		if (perfClient) {
            int len = strlen(REGISTER_PERF_CLIENT) + 1;

            if (write(REGISTER_PERF_CLIENT, &len) != CAVERNnet_perfDaemonClient_c::OK) {
			    printf("Cannot connect to perf_daemon\n");
				connectionAlive = 0;
				return CAVERNnet_perfDaemonClient_c::FAILED;
			}
        }

		return CAVERNnet_perfDaemonClient_c::OK;
	} 
	else {
	    printf("Cannot connect to perf_daemon\n");
		connectionAlive = 0;
		return CAVERNnet_perfDaemonClient_c::FAILED;
	}
}


int CAVERNnet_perfDaemonClient_c :: read(char **ret, int *nbytes, int blockingType)
{
	if (connectionAlive == 0 || client == NULL) return CAVERNnet_perfDaemonClient_c::CONNECTION_TERMINATED;
	
	CAVERNnet_datapack_c packer;
	char *read_buffer;
	int dataSize;
	int status;
//	char header[sizeof(int)];

	// Read the header.	
	dataSize = packer.sizeof_int();
	read_buffer = new char[dataSize];
	status = client->read((char*) read_buffer, &dataSize, blockingType);
	if (status == CAVERNnet_tcpClient_c::OK)  {                                                     
		// The header is simply the # of bytes that follows.
		int incomingSize = 0;
                                                  
	    packer.initUnpack((char *)read_buffer,dataSize);
		packer.unpackInt(&incomingSize);
	    delete read_buffer;                           

		char *buffer = new char[incomingSize];
		if (buffer == NULL) {
			*ret = NULL;
			connectionAlive = 0;
			return CAVERNnet_perfDaemonClient_c::MEM_ALLOC_ERR;
		}

		// Read the body
		status = client->read(buffer, &incomingSize, CAVERNnet_tcpClient_c::BLOCKING);
		if (status == CAVERNnet_tcpClient_c::OK)  {
			*nbytes = incomingSize;
			*ret = buffer;
			return CAVERNnet_perfDaemonClient_c::OK;
		} else {
			connectionAlive = 0;
			*nbytes = 0;
			delete buffer;
			*ret = NULL;
			if (status == CAVERNnet_tcpClient_c::CONNECTION_TERMINATED)
				return CAVERNnet_perfDaemonClient_c::CONNECTION_TERMINATED;
			else
				return CAVERNnet_perfDaemonClient_c::FAILED;
		}
	}

	if (read_buffer) delete read_buffer;
	*nbytes = 0;
	*ret = NULL;
	if (status == CAVERNnet_tcpClient_c::NON_BLOCKING_HAS_NO_DATA) 
		return CAVERNnet_perfDaemonClient_c::NON_BLOCKING_HAS_NO_DATA;
	if (status == CAVERNnet_tcpClient_c::CONNECTION_TERMINATED) {
		connectionAlive = 0;
		return CAVERNnet_perfDaemonClient_c::CONNECTION_TERMINATED;
	}
	if (status == CAVERNnet_tcpClient_c::TIMED_OUT) {
		connectionAlive = 0;
		return CAVERNnet_perfDaemonClient_c::CONNECTION_TERMINATED;
	}

	return CAVERNnet_perfDaemonClient_c::FAILED;

}


int CAVERNnet_perfDaemonClient_c :: write(char *ptr, int *nbytes)
{
	CAVERNnet_datapack_c packer;
	char *send_buffer;

	if (connectionAlive == 0 || client == NULL) return CAVERNnet_perfDaemonClient_c::FAILED;

	int  dataSize = packer.sizeof_int();
	int status;

	// Send the header
	send_buffer = new char[dataSize];
	packer.initPack(send_buffer, dataSize);
	packer.packInt(*nbytes);
	status = client->write(send_buffer,&dataSize,CAVERNnet_tcpClient_c::BLOCKING);
	delete send_buffer;
	if (status != CAVERNnet_tcpClient_c::OK) {
		connectionAlive = 0;
		return CAVERNnet_perfDaemonClient_c::CONNECTION_TERMINATED;
	}
				
	status = client->write(ptr, nbytes, CAVERNnet_tcpClient_c::BLOCKING);
	if (status != CAVERNnet_tcpClient_c::OK) {
		connectionAlive = 0;
		return CAVERNnet_perfDaemonClient_c::CONNECTION_TERMINATED;
	}

	return CAVERNnet_perfDaemonClient_c::OK;
}

int CAVERNnet_perfDaemonClient_c :: isReadyToRead()
{

	if (connectionAlive == 0 || client == NULL) return CAVERNnet_perfDaemonClient_c::NOT_READY;

	int status= client->isReadyToRead();

	if (CAVERNnet_tcpClient_c::READY_TO_READ == status)
		return CAVERNnet_perfDaemonClient_c::READY_TO_READ;

	else //we got CAVERNnet_tcpClient_c::NOT_READY
		return CAVERNnet_perfDaemonClient_c::NOT_READY;
}


int CAVERNnet_perfDaemonClient_c :: isReadyToWrite() {

	if (connectionAlive == 0 || client == NULL) return CAVERNnet_perfDaemonClient_c::NOT_READY;

	int status =  client->isReadyToWrite();

	if (CAVERNnet_tcpClient_c::READY_TO_WRITE == status)
		return CAVERNnet_perfDaemonClient_c::READY_TO_WRITE;

	else // we got CAVERNnet_tcpClient_c::NOT_READY
		return CAVERNnet_perfDaemonClient_c::NOT_READY;


}

int CAVERNnet_perfDaemonClient_c :: isReady() {

	if (connectionAlive == 0 || client == NULL) return CAVERNnet_perfDaemonClient_c::NOT_READY;

	int status =  client->isReady();

	if (CAVERNnet_tcpClient_c::READY_TO_READ == status)
		return CAVERNnet_perfDaemonClient_c::READY_TO_READ;

	else if (CAVERNnet_tcpClient_c::READY_TO_WRITE == status)
		return CAVERNnet_perfDaemonClient_c::READY_TO_WRITE;

	else if (CAVERNnet_tcpClient_c::READY_TO_READ_AND_WRITE == status)
		return CAVERNnet_perfDaemonClient_c::READY_TO_READ_AND_WRITE;

	else //we got CAVERNnet_tcpClient_c::NOT_READY
		return CAVERNnet_perfDaemonClient_c::NOT_READY;

}

void CAVERNnet_perfDaemonClient_c::showStatus(int status, int nbytes)
{
	switch (status) {
	case CONNECTION_TERMINATED:
		printf("CAVERNnet_perfDaemonClient_c::showStatus: Connection terminated with %d bytes transacted.\n", nbytes);
		break;
	case NON_BLOCKING_HAS_NO_DATA:
		printf("CAVERNnet_perfDaemonClient_c::showStatus: Non-blocking read found no data on stream.\n");
		break;
	case OK:
		printf("CAVERNnet_perfDaemonClient_c::showStatus: Transaction ok.\n");
		break;
	default:
#ifdef WIN32
		printf("CAVERNnet_perfDaemonClient_c::showStatus: UNIX Socket error no: %d ",WSAGetLastError());
#else
		printf("CAVERNnet_perfDaemonClient_c::showStatus: UNIX Socket error no: %d ",errno);
		perror("");
#endif
		break;
	}
	fflush(stdout);
}

unsigned int CAVERNnet_perfDaemonClient_c::getRemoteIP() 
{ 
	return (client) ? client->getRemoteIP() : 0; 
};

void CAVERNnet_perfDaemonClient_c::getRemoteIP(char* name) 
{ 
	if (client) client->getRemoteIP(name); 
}

unsigned int CAVERNnet_perfDaemonClient_c::getSelfIP() 
{ 
	return (client) ? client->getSelfIP() : 0; 
}

void CAVERNnet_perfDaemonClient_c::getSelfIP(char *name) 
{ 
	if (client) client->getSelfIP(name); 
}

int CAVERNnet_perfDaemonClient_c::getSelfPort() 
{ 
	return (client) ? client->getSelfPort() : 0; 
}

int CAVERNnet_perfDaemonClient_c::getRemotePort() 
{ 
	return (client) ? client->getRemotePort() : 0; 
}

void CAVERNnet_perfDaemonClient_c::close() 
{ 
	connectionAlive = 0; 
	if (client) client->close(); 
}
