/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include "CAVERNnet_perfDaemon_c.hxx"
#include "CAVERNnet_datapack_c.hxx"
#include "CAVERNnet_tcp_c.hxx"

const int CAVERNnet_perfDaemon_c::OK = 0;
const int CAVERNnet_perfDaemon_c::FAILED = 1;
const int CAVERNnet_perfDaemon_c::MEM_ALLOC_ERR = 2;
const int CAVERNnet_perfDaemon_c::NEW_CONNECTION_ESTABLISHED = 4;
const int CAVERNnet_perfDaemon_c::TOO_MANY_CLIENTS = 5;
const int CAVERNnet_perfDaemon_c::NO_NEW_CONNECTION= 6;
const int CAVERNnet_perfDaemon_c::NON_BLOCKING_HAS_NO_DATA=7;
const int CAVERNnet_perfDaemon_c::SKIP_DISTRIBUTION=8;
const int CAVERNnet_perfDaemon_c::DEFAULT_PORT=PERF_DAEMON_DEFAULT_PORT;

CAVERNnet_perfDaemon_c :: CAVERNnet_perfDaemon_c(FILE* File)
{
	server = new CAVERNnet_tcpServer_c;
	interceptCallback = NULL;
	interceptUserData = NULL;
	interceptNewConnectionCallback = NULL;
	interceptNewConnectionUserData = NULL;
	LogFile = File;
}


CAVERNnet_perfDaemon_c :: ~CAVERNnet_perfDaemon_c()
{
	if (server) delete server;
	closeDownSockets();
}

int CAVERNnet_perfDaemon_c :: init(int incomingPort, int maxClients)
{
	maxNumClients = maxClients;

	// Initialize the incoming socket
	if (server && server->init(incomingPort) == 0)	{
		return CAVERNnet_perfDaemon_c::FAILED;
	}

	// Create an array of pointers to track the incoming connections.
	clients = new PerfDaemonClient*[maxNumClients];
	if (clients == NULL) return CAVERNnet_perfDaemon_c::MEM_ALLOC_ERR;

	for(int i = 0; i < maxNumClients; i++) clients[i] = NULL;

	return CAVERNnet_perfDaemon_c::OK;
}

int CAVERNnet_perfDaemon_c :: checkForNewClients()
{
	CAVERNnet_tcpClient_c *aClient;
	/* If a connection is established, accept it and create
	 * a new socket for that connection to communicate with.
	 *
	 */
	aClient = server->checkForNewConnections();

	// If a new client connects then add it to an empty slot in the array of clients.
	if (aClient) {
		CAVERNnet_perfDaemonClient_c *newClient = new CAVERNnet_perfDaemonClient_c(aClient);

		// Turn the timeout off. In general this is necessary because in a
		// TCP stream if you lose some data you won't be able to figure out
		// where the head of the next message will be.
		aClient->setTimeOut(CAVERNnet_tcpClient_c::NO_TIME_OUT);

		for (int i = 0; i < maxNumClients; i++) {
			if (clients && clients[i] == NULL){
                clients[i] = new PerfDaemonClient;
				clients[i]->Client = newClient;
				clients[i]->IsPerfClient = 0;
				clients[i]->Started = 0;

				char remoteIP[256];
				int remotePort;
				newClient->getRemoteIP(remoteIP);
				remotePort = newClient->getRemotePort();
				printf("CAVERNnet_perfDaemon_c: New Client: %s:%d\n",remoteIP, remotePort);

				//sprintf(clients[i]->FileName, "%s_%d", remoteIP, remotePort);

				if (interceptNewConnectionCallback)
					interceptNewConnectionCallback(newClient, 
                                                   interceptNewConnectionUserData);

				return CAVERNnet_perfDaemon_c::NEW_CONNECTION_ESTABLISHED;
			}
		}

		// If user callback specified then call it.

		// If cannot host any more clients then report it.
		return CAVERNnet_perfDaemon_c::TOO_MANY_CLIENTS;
	}
	return CAVERNnet_perfDaemon_c::NO_NEW_CONNECTION;
}

void CAVERNnet_perfDaemon_c :: intercept(int (*callback) (CAVERNnet_perfDaemonClient_c *, char** buffer, int *datasize, void *userData), void* userData)
{
	interceptCallback = callback;
	interceptUserData = userData;
}

void CAVERNnet_perfDaemon_c :: interceptNewConnection(void (*callback) (CAVERNnet_perfDaemonClient_c *newClient, void* userData), void *userData)
{
	interceptNewConnectionCallback = callback;
	interceptNewConnectionUserData = userData;
}


int CAVERNnet_perfDaemon_c :: process()
{
    int status;
    int incomingSize = 0;
    char *buffer;

    // Check for any new clients.
    checkForNewClients();

    // Go through array of clients and read from them.
    for (int i = 0; i < maxNumClients; i++) {
		if (clients && clients[i]) {
		    status = clients[i]->Client->read(&buffer, &incomingSize,
					  CAVERNnet_perfDaemonClient_c::NON_BLOCKING);
		    if (status == CAVERNnet_perfDaemonClient_c::MEM_ALLOC_ERR) 
		        return CAVERNnet_perfDaemon_c::MEM_ALLOC_ERR;
		    if (status == CAVERNnet_perfDaemonClient_c::OK) {
		        if (clients[i]->Started == 0) {
				    if (strncmp(REGISTER_PERF_CLIENT, buffer, REGISTER_PERF_CLIENT_LENGTH) == 0) {
					    clients[i]->IsPerfClient = 1;
						clients[i]->Started = 1;
						delete buffer;
						return CAVERNnet_perfDaemon_c::OK;
				    }
				    else if (strncmp(SEND_STAT_CLIENT, buffer, SEND_STAT_CLIENT_LENGTH) == 0) {
					    clients[i]->IsPerfClient = 0;
						clients[i]->Started = 1;
						delete buffer;
						return CAVERNnet_perfDaemon_c::OK;
				    }
				}
				if (interceptCallback) {
				    if (interceptCallback(clients[i]->Client, &buffer, 
										&incomingSize, interceptUserData) 
						== CAVERNnet_perfDaemon_c::SKIP_DISTRIBUTION) {
					    delete buffer;
						continue;
				    }
				}

				// append to the logfile
				if (LogFile) {
                    fprintf(LogFile, "%s\n", buffer); 
                    fflush(LogFile);
                }

				// Reflect the data to all connected clients except
				// the client that originated the message.
				distributeDataToPerfClients(buffer, incomingSize);
				delete buffer;
		    } 
		    else if (status != CAVERNnet_perfDaemonClient_c::NON_BLOCKING_HAS_NO_DATA){
		        removeClient(i);
                printf("client %d is removed.\n", i);
            }
		}
    }
    return CAVERNnet_perfDaemon_c::OK;
}

int CAVERNnet_perfDaemon_c :: distributeDataToPerfClients(char* buf, int incomingSize)
{
	// Iterate thru the array of clients and send the message to all of them
	// except the one indicated by clientToIgnore
	if (clients) {
		for (int i = 0; i < maxNumClients; i++) {
			if (clients[i]) {
				if (clients[i]->IsPerfClient 
				              && clients[i]->Client->write(buf, &incomingSize) != CAVERNnet_perfDaemonClient_c::OK) {
					removeClient(i);
				}
			}
		}
	}
	return CAVERNnet_perfDaemon_c::OK;
}

int CAVERNnet_perfDaemon_c::sendToAll(char* buf, int incomingSize)
{
    return distributeDataToPerfClients(buf, incomingSize);
}

void CAVERNnet_perfDaemon_c :: removeClient(int i)
{
	if (clients && clients[i])	{
		delete clients[i];
    }
	clients[i] = NULL;
}

void CAVERNnet_perfDaemon_c :: closeDownSockets()
{
	int count;

	// Iterate thru list of clients and close them down.
	if (clients) {
		for (count = 0; count < maxNumClients; count++) {
			if (clients[count]) {
				clients[count]->Client->close();
				delete clients[count];
			}
		}
		delete clients;
	}
	server->close();
	delete server;
}

