/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include "CAVERNnet_perfMonitor_c.hxx"
#include "CAVERNnet_perfClient_c.hxx"

const int CAVERNnet_perfMonitor_c::FAILED = 0;
const int CAVERNnet_perfMonitor_c::OK = 1;

CAVERNnet_perfMonitor_c::CAVERNnet_perfMonitor_c()
{
    resultantStats = new char[DEFAULT_BUFFER_SIZE];
    selfIp = NULL;
    remoteIp = NULL;
    aClient = NULL;	
}

CAVERNnet_perfMonitor_c::~CAVERNnet_perfMonitor_c()
{
    if (resultantStats) delete [] resultantStats;
        if (selfIp) delete [] selfIp;
	if (remoteIp) delete [] remoteIp;
	if (aClient) delete aClient;
}

void CAVERNnet_perfMonitor_c::setIPs(char* _selfIp, char* _remoteIp)
{
    if (_selfIp) {
		selfIp = new char[strlen(_selfIp)+1];
		if (selfIp) strcpy(selfIp, _selfIp);
	}

    if (_remoteIp) {
		remoteIp = new char[strlen(_remoteIp)+1];
		if (remoteIp) strcpy(remoteIp, _remoteIp);
	}
}

void CAVERNnet_perfMonitor_c::setPorts(int _selfPort, int _remotePort)
{
    selfPort = _selfPort;
    remotePort = _remotePort;    
}

void CAVERNnet_perfMonitor_c::showStats(char* streamInfo, char* comment)
{ 
    //update the resultant buffer with the calculated information
    updateStats(streamInfo, comment);
    printf("\n%s\n", resultantStats);    
}

int CAVERNnet_perfMonitor_c::logStats(char* streamInfo, char* comment, FILE* filePtr)
{
	if (filePtr) {
		//update the resultant buffer with the calculated information
		updateStats(streamInfo, comment);
  
		int bytesWritten;
		if ((bytesWritten = fprintf(filePtr, "%s\n", resultantStats)) != (strlen(resultantStats) + 1)) {            
			printf("\nWarning: Error writing to logfile - data may be truncated or  not written at all\n");
			return CAVERNnet_perfMonitor_c::FAILED;
		}    
		fflush(filePtr);
	}
	else {
	    printf("File pointer must be given to use logStats.\n");
		return CAVERNnet_perfMonitor_c::FAILED;
	}
    return CAVERNnet_perfMonitor_c::OK;
}

int CAVERNnet_perfMonitor_c::initSendStats(char* monitorClientIP, int port)
{
	if (monitorClientIP) {
		if (aClient) {
			delete aClient;
			aClient = NULL;// close current connection first.
		}
		aClient = new CAVERNnet_perfDaemonClient_c;
		if (aClient == NULL) {
		    printf("Cannot create perfDaemonClient.\n");
			return CAVERNnet_perfMonitor_c::FAILED;
		}

		if (aClient->connectToServer(monitorClientIP, port) == CAVERNnet_perfDaemonClient_c::FAILED) {
			printf("\nCannot connect to perf_daemon\n");
      		delete aClient;        
			aClient = NULL;
			return CAVERNnet_perfMonitor_c::FAILED;
		}
		int len = SEND_STAT_CLIENT_LENGTH + 1; // including terminating null char
		if (aClient->write(SEND_STAT_CLIENT, &len) != CAVERNnet_perfDaemonClient_c::OK) {
		    printf("Cannot connect to perf_daemon\n");
			delete aClient;
			aClient = NULL;
			return CAVERNnet_perfMonitor_c::FAILED;
		}
	}
	else {
	    printf("Invalid ip address for perf_daemon is given. Please verify the address again.\n");
		return CAVERNnet_perfMonitor_c::FAILED;
	}
    return CAVERNnet_perfMonitor_c::OK;
}

int CAVERNnet_perfMonitor_c::sendStats(char* streamInfo, char* comment)
{
	if (aClient == NULL) {
		printf("PerfDaemonClient is not initialized. No data will be sent to perf_daemon\n");
	    return CAVERNnet_perfMonitor_c::FAILED;
	}

	//update the resultant buffer with the calculated information
	updateStats(streamInfo, comment);

	int nbytes = strlen(resultantStats) + 1;

	//transmitting data to perfdaemon    
	int status = aClient->write(resultantStats, &nbytes);
	if (status != CAVERNnet_perfDaemonClient_c::OK) {
		aClient->showStatus(status, nbytes);
		delete aClient;
		aClient = NULL;
		return CAVERNnet_perfMonitor_c::FAILED;
	}

    return CAVERNnet_perfMonitor_c::OK;
}

void CAVERNnet_perfMonitor_c::exitSendStats()
{
	if (aClient) {
		delete aClient;
		aClient = NULL;
	}
}

void CAVERNnet_perfMonitor_c::updateStats(char* streamInfo, char* comment)
{
    //update the resultant stats buffer
	if (resultantStats) {
	    char *tempStreamInfo, *tempComment;
            char tempMsg[] = "NULL";
        
	    tempComment = (comment == NULL) ? tempMsg : comment;        
	    tempStreamInfo = (streamInfo == NULL) ? tempMsg : streamInfo;

            sprintf(resultantStats, "TIME=%lf SELF_IP=%s REMOTE_IP=%s SELF_PORT=%d REMOTE_PORT=%d STREAM_INFO=%s COMMENT=%s MIN_LAT=%lf AVG_LAT=%lf MAX_LAT=%lf INST_LAT=%lf STAL=%lf JITTER=%lf MIN_IMD=%lf AVG_IMD=%lf MAX_IMD=%lf INST_IMD=%lf AVG_RBW=%lf INST_RBW=%lf AVG_SBW=%lf INST_SBW=%lf STAB=%lf BURSTINESS=%lf TOTAL_READ=%ld TOTAL_SENT=%ld PACKETS_READ=%ld PACKETS_SENT=%ld", getTimeInSecs(), selfIp, remoteIp, selfPort, remotePort, tempStreamInfo, tempComment,  getMinLatency(), getAverageLatency(), getMaxLatency(), getInstantLatency(), getSTALatency(), getJitter(),  getMinInterMesgDelay(), getAverageInterMesgDelay(), getMaxInterMesgDelay(),  getInstantInterMesgDelay(), getAverageReceiveBandwidth(), getInstantReceiveBandwidth(),  getAverageSendBandwidth(),  getInstantSendBandwidth(), getSTABandwidth(), getBurstiness(), getTotalDataRead(),  getTotalDataSent(), getPacketsRead(), getPacketsSent());
	}
}











