/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#if (defined(__sgi) && (_MIPS_SZLONG==64)) || defined(CAVERN_SIXTY_FOUR_BIT_FILE_SYSTEM)

#include "CAVERNnet_remoteParallelFileIO64_c.hxx"
#include "CAVERNts_thread_c.hxx"
#include "CAVERNnet_datapack_c.hxx"

#ifdef WIN32
#include <time.h>
#include <stdlib.h>

#define MAXPATHLEN	MAX_PATH
#else
#include <sys/param.h>
#include <netinet/in.h>
#endif

#include <stdio.h>
#include <sys/stat.h>

/*
Data stream format from client to server:

Upload:
'u' +
filenamelen[int] +
filename +
datasize[int]
data

Download:
'd' +
filenamelen[int] +
filename

From server to client- in response to download message:
'd' +
filesize[int]
data

*/

class ServerData
{
public:
    char* neighborServerName;
	CAVERNnet_extendedParallelTcpClient_c* client;
	CAVERNnet_extendedParallelTcpServer_c* server;
	int store;
	int port;
	int numberOfSockets;

	virtual ~ServerData() { if (client) { delete client; } }
};

/* Minimized client version of remote file IO client that can
   only download.
*/
class CAVERNnet_ParallelServerClient64_c {
	public:

	/// Status ok
	static const int OK/* = 1*/;

	/// Status failed
	static const int FAILED/* = 0*/;

	/// Cannot connect to remote server.
	static const int CANNOT_CONNECT/* = 2*/;

	/// Cannot open local file for reading.
	static const int CANNOT_READ_LOCAL_FILE/* = 3*/;

	/// CAnnot write to local file.
	static const int CANNOT_WRITE_LOCAL_FILE/* = 3*/;

	/// Received invalid command from server.
	static const int INVALID_COMMAND/* = 4*/;

	/// Memory allocation error.
	static const int MEM_ALLOC_ERR/* = 5*/;

	/// Remote file error.
	static const int FILE_ERROR/* = 6*/;

	/// Remote file error.
	static const int DOWNLOAD/* = 7*/;

	CAVERNnet_ParallelServerClient64_c(CAVERNnet_extendedParallelTcpServer_c* Server,
						               int ServerIP = 0, int ServerPort = 0);
	~CAVERNnet_ParallelServerClient64_c();

	/// Set port number to connect to.
	void setRemotePort(int port = CAVERNnet_remoteParallelFileIOServer64_c::DEFAULT_PORT) {
	    neighborPortNumber = port;
	}

	/** Download a file from the remote server. If the remote file is not found or
	    a network error occurs, this function will return with a failed.
	    @param ipAddress is IP address of remote server.
	    @param localFileName is local file write to.
	    @param remoteFileName is remote file to get.
	    @return OK, FAILED, CANNOT_WRITE_LOCAL_FILE, INVALID_COMMAND, CANNOT_CONNECT
	*/
	int initDownload(char *ipAddress, char* localFilename, char* remoteFileName, int numOfSockets);
	int readBlock(char*, int nBytes);
	int getFileSize(int64& fileSize) { return client->readInt64(fileSize); }
	int getRemoteDateTime(char *ipAddress, char* remoteFileName, int *remDay, 
		        int *remMon, int *remYear, int* remHour, int* remMin, int* remSec);

private:
	int neighborPortNumber;
	int store;
	int serverIPNumber;
	int serverPortNumber;
	char neighborServerName[MAXPATHLEN];
	CAVERNnet_extendedParallelTcpServer_c* server;
	CAVERNnet_extendedParallelTcpClient_c* client;
};

const int CAVERNnet_ParallelServerClient64_c::OK = 1;
const int CAVERNnet_ParallelServerClient64_c::FAILED = 0;
const int CAVERNnet_ParallelServerClient64_c::CANNOT_CONNECT = 2;
const int CAVERNnet_ParallelServerClient64_c::CANNOT_READ_LOCAL_FILE = 3;
const int CAVERNnet_ParallelServerClient64_c::CANNOT_WRITE_LOCAL_FILE = 3;
const int CAVERNnet_ParallelServerClient64_c::INVALID_COMMAND = 4;
const int CAVERNnet_ParallelServerClient64_c::MEM_ALLOC_ERR = 5;
const int CAVERNnet_ParallelServerClient64_c::FILE_ERROR = 6;
const int CAVERNnet_ParallelServerClient64_c::DOWNLOAD = 7;

CAVERNnet_ParallelServerClient64_c::CAVERNnet_ParallelServerClient64_c(CAVERNnet_extendedParallelTcpServer_c* Server,
																	   int ServerIP, int ServerPort)
{
	client = NULL;
	server = Server;
	serverIPNumber = ServerIP;
	serverPortNumber = ServerPort;
	neighborPortNumber = CAVERNnet_remoteParallelFileIOServer64_c::DEFAULT_PORT;
}

CAVERNnet_ParallelServerClient64_c::~CAVERNnet_ParallelServerClient64_c()
{
	if (client) delete client;
}

int CAVERNnet_ParallelServerClient64_c::getRemoteDateTime(char *ipAddress, 
						char* remoteFileName, int *remDay, int *remMon, 
						int *remYear, int* remHour, int* remMin, int* remSec)
{
	CAVERNnet_extendedParallelTcpClient_c *client = new CAVERNnet_extendedParallelTcpClient_c;

	if (client == NULL || server == NULL) {
		*remDay = -1; *remMon = -1; *remYear = -1;
		*remHour = -1; *remMin = -1; *remSec = -1;
		return CAVERNnet_ParallelServerClient64_c::MEM_ALLOC_ERR;
	}

	int status = client->connectToServer(ipAddress, neighborPortNumber, 1);
	if (status == CAVERNnet_extendedParallelTcpClient_c::FAILED) {
		*remDay = -1; *remMon = -1; *remYear = -1;
		*remHour = -1; *remMin = -1; *remSec = -1;
		return CAVERNnet_ParallelServerClient64_c::CANNOT_CONNECT;
	}

	status = client->writeChar('t');
	if (status != CAVERNnet_extendedParallelTcpClient_c::OK) 
		return CAVERNnet_ParallelServerClient64_c::FAILED;

	// Send filename length
	int filenamelen = strlen(remoteFileName) + 1;
	status = client->writeInt(filenamelen);
	if (status != CAVERNnet_extendedParallelTcpClient_c::OK) 
		return CAVERNnet_ParallelServerClient64_c::FAILED;

	// Send filename
	int nbytes = filenamelen;
	status = client->write(remoteFileName, &nbytes);
	if (status != CAVERNnet_parallelTcpClient_c::OK) 
		return CAVERNnet_ParallelServerClient64_c::FAILED;

	status = client->writeInt((serverIPNumber) ? serverIPNumber : server->getSelfIP());
	if (status != CAVERNnet_parallelTcpClient_c::OK) 
		return CAVERNnet_ParallelServerClient64_c::FAILED;

	// now sends out server's port number
	status = client->writeInt((serverPortNumber) ? serverPortNumber : server->getSelfPort());
	if (status != CAVERNnet_parallelTcpClient_c::OK) 
		return CAVERNnet_ParallelServerClient64_c::FAILED; 

	// Now wait for response.
	char transferMode = ' ';
	status = client->readChar(transferMode);
	if (status != CAVERNnet_parallelTcpClient_c::OK) 
		return CAVERNnet_ParallelServerClient64_c::FAILED;

	switch (transferMode) {
		case 'i': // invalid command
		case 'n': // File not exist
			*remDay = -1; *remMon = -1; *remYear = -1;
			*remHour = -1; *remMin = -1; *remSec = -1;
			return CAVERNnet_ParallelServerClient64_c::FILE_ERROR;

		case 't': // retrieving date/time
		{
			int remdate[6];
			CAVERNnet_datapack_c packer;

			nbytes = packer.sizeof_int() * 6;
			char* read_buffer = new char[nbytes];
			status = client->read((char*) read_buffer, &nbytes);
	        packer.initUnpack(read_buffer,nbytes);  
			for (int i = 0; i < 6; i++) 
				packer.unpackInt(&(remdate[i]));
	        delete read_buffer;                     

			if (status != CAVERNnet_parallelTcpClient_c::OK) 
				return CAVERNnet_ParallelServerClient64_c::FILE_ERROR;

			*remDay = remdate[0]; *remMon = remdate[1]; *remYear = remdate[2];
			*remHour = remdate[3]; *remMin = remdate[4]; *remSec = remdate[5];
			if ((remdate[0] == -1) && (remdate[1] == -1) && (remdate[2] == -1) &&
				(remdate[3] == -1) && (remdate[4] == -1) && (remdate[5] == -1)) {
				return CAVERNnet_ParallelServerClient64_c::FILE_ERROR;
			}
			return CAVERNnet_ParallelServerClient64_c::OK;
		}

		default:
			break;
	}
	return CAVERNnet_ParallelServerClient64_c::FAILED;
}

int CAVERNnet_ParallelServerClient64_c::initDownload(char *ipAddress, char* localFileName, 
                                               char* remoteFileName, int numOfSockets)
{
	CAVERNnet_datapack_c packer;
	
	client = new CAVERNnet_extendedParallelTcpClient_c;
	if (client == NULL || server == NULL) return CAVERNnet_ParallelServerClient64_c::MEM_ALLOC_ERR;

	int status = client->connectToServer(ipAddress, neighborPortNumber, numOfSockets);
	if (status == CAVERNnet_parallelTcpClient_c::FAILED) {
		return CAVERNnet_ParallelServerClient64_c::CANNOT_CONNECT;
	}

	char transferMode = 'o';
	status = client->writeChar(transferMode);
	if (status != CAVERNnet_parallelTcpClient_c::OK) 
		return CAVERNnet_ParallelServerClient64_c::FAILED;

	// Send filename length
	int filenamelen = strlen(remoteFileName) + 1;

	status = client->writeInt(filenamelen);
	if (status != CAVERNnet_parallelTcpClient_c::OK) 
		return CAVERNnet_ParallelServerClient64_c::FAILED;

	// Send filename
	int nbytes = filenamelen;
	status = client->write(remoteFileName, &nbytes);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {
		delete client; return CAVERNnet_ParallelServerClient64_c::FAILED;
	}

	status = client->writeInt((serverIPNumber) ? serverIPNumber : server->getSelfIP());
	if (status != CAVERNnet_parallelTcpClient_c::OK) 
		return CAVERNnet_ParallelServerClient64_c::FAILED;

	// now sends out server's port number
	status = client->writeInt((serverPortNumber) ? serverPortNumber : server->getSelfPort());
	if (status != CAVERNnet_parallelTcpClient_c::OK) 
		return CAVERNnet_ParallelServerClient64_c::FAILED;

	// Now wait for response.
	transferMode =' ';
	status = client->readChar(transferMode);
	if (status != CAVERNnet_parallelTcpClient_c::OK) 
		return CAVERNnet_ParallelServerClient64_c::FAILED;
	
	switch (transferMode) {
		case 'i': // invalid command
		case 'n': // File not exist
			return CAVERNnet_ParallelServerClient64_c::FAILED;

		case 'o': // Download 
			return CAVERNnet_ParallelServerClient64_c::DOWNLOAD;

		default:
			return CAVERNnet_ParallelServerClient64_c::INVALID_COMMAND;
	}
}

int CAVERNnet_ParallelServerClient64_c::readBlock(char* dataBlock, int nBytes)
{
	if (client) {
		// Read from file and write to socket in chunks
		int status = client->read(dataBlock, &nBytes);
		if (status != CAVERNnet_parallelTcpClient_c::OK)
			return CAVERNnet_ParallelServerClient64_c::FAILED;
	
		return CAVERNnet_ParallelServerClient64_c::OK;
	}
	return CAVERNnet_ParallelServerClient64_c::FAILED;
}

const int CAVERNnet_remoteParallelFileIOServer64_c::DEFAULT_PORT = 7673;
const int CAVERNnet_remoteParallelFileIOServer64_c::THREADED = 1;
const int CAVERNnet_remoteParallelFileIOServer64_c::NON_THREADED = 0;
const int CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE = 30000;

int CAVERNnet_remoteParallelFileIOServer64_c::init(int inPort, char* neighborServer, 
                               int neighborPort, int storeLocally, int numOfSockets)
{
	if (neighborServer) strcpy(neighborServerName, neighborServer);
	else neighborServerName[0] = '\0';
	store = storeLocally;
	numberOfSockets = numOfSockets;
	neighborPortNumber = neighborPort;

	return server.init(inPort);
}

void CAVERNnet_remoteParallelFileIOServer64_c :: process(int threaded)
{
	CAVERNnet_extendedParallelTcpClient_c *newClient = server.checkForNewConnections();
	if (newClient) {
		ServerData* data = new ServerData;

		data->client = newClient;
		data->neighborServerName = &neighborServerName[0];
		data->store = store;
		data->port = neighborPortNumber;
		data->numberOfSockets = numberOfSockets;
		data->server = &server;

		if (threaded== CAVERNnet_remoteParallelFileIOServer64_c::THREADED) {
			CAVERNts_thread_c newThread; 
			newThread.create(threadHandler, (void*) data);
		} 
		else threadHandler((void*) data);
	}
}

#define CLOSE_REMOVE_AND_RETURN 	if (readFile) fclose(readFile); \
									if (writeFile) fclose(writeFile); \
									if (downloadClient) delete downloadClient; \
									delete data; \
									return NULL; 

void* CAVERNnet_remoteParallelFileIOServer64_c :: threadHandler(void* inData)
{
	ServerData* data = (ServerData*) inData;
	if (data == NULL) return NULL;

	CAVERNnet_datapack_c packer;
	CAVERNnet_extendedParallelTcpClient_c *newClient = ((ServerData *) data)->client;
	char* neighborServerName = ((ServerData*) data)->neighborServerName;
	int store = ((ServerData*) data)->store;
	int neighborPort = ((ServerData*) data)->port;
	int numberOfSockets = ((ServerData*) data)->numberOfSockets;
	CAVERNnet_extendedParallelTcpServer_c* server = ((ServerData*) data)->server;
	int nbytes;

	// Check if upload or download
	char transferMode;
	int status = newClient->readChar(transferMode);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {
	    delete data; return NULL;
	}

	// Check filename size
	int filenamelen;
	status = newClient->readInt(filenamelen);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {
	    delete data; return NULL;
	}

	// Check filename
	char* filename = new char[filenamelen];
	if (filename == NULL) { delete data; return NULL;}
	status = newClient->read(filename, &filenamelen);
	if (status != CAVERNnet_parallelTcpClient_c::OK) { delete filename; delete data; return NULL; }
	filename[filenamelen - 1]=0;

	char dataBlock[CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE];

	// If handling check for return date/time stamp
	if (transferMode == 'c') {
		int thedate[6];

		// If file does not exist time stamp will return -1s.
		status = CAVERNnet_remoteParallelFileIOClient64_c::getLocalDateTime(filename,
									  &thedate[0],
									  &thedate[1],
									  &thedate[2],
									  &thedate[3],
									  &thedate[4],
									  &thedate[5]);
		// If file does not exist locally, then check neighbor server if neighbor exists.
		if (status != CAVERNnet_remoteParallelFileIOClient64_c::OK && neighborServerName[0]) {
			if (server->getSelfIP() == getHostIPNumber(neighborServerName) 
							&& server->getSelfPort() == neighborPort) { // trying to connect to itself
#ifdef DBG_MSG
				printf("trying to connecting to itself. server processing 'c'\n");
#endif
				status = newClient->writeChar('n');
				delete filename; delete data; return NULL;
			}

		    CAVERNnet_ParallelServerClient64_c client(server);

			client.setRemotePort(neighborPort);
			status = client.getRemoteDateTime(neighborServerName, filename, 
			                          &thedate[0], &thedate[1],
									  &thedate[2], &thedate[3],
									  &thedate[4], &thedate[5]);
			if (status != CAVERNnet_ParallelServerClient64_c::OK) {
				// Send back command 'n'
				status = newClient->writeChar('n');
				delete filename; delete data; return NULL;
			}
		}

		// Send back command 'c'
		status = newClient->writeChar('c');
		if (status != CAVERNnet_parallelTcpClient_c::OK) {
			delete filename; delete data; return NULL;
		}

		// Send back date/time stamp of file
		nbytes = 6 * packer.sizeof_int();
		char* send_buffer = new char[nbytes];
        packer.initPack(send_buffer, nbytes);  
		for (int i = 0; i < 6; i++)	packer.packInt(thedate[i]);
		status = newClient->write((char*) send_buffer, &nbytes);
		delete send_buffer;

		delete filename;
		delete data;
		return NULL;
	}

	// If handling check for return date/time stamp for the request from a server
	if (transferMode == 't') {
		// Get server IP
		int serverIP;
		status = newClient->readInt(serverIP);
		if (status != CAVERNnet_parallelTcpClient_c::OK) { delete filename; delete data; return NULL; }

		// get port number 
		int serverPort;
		newClient->readInt(serverPort);
		if (status != CAVERNnet_parallelTcpClient_c::OK) { delete filename; delete data; return NULL; }

/*		// Check whether we are getting back the request this server sent before.
		if (serverIP == server->getSelfIP() && serverPort == server->getSelfPort()) { 
#ifdef DBG_MSG
			printf("cycle detected. server processing 't'\n");
#endif
			newClient->writeChar('n');
			delete filename; delete data; return NULL;
		}*/
		
		int thedate[6];

		// If file does not exist time stamp will return -1s.
		status = CAVERNnet_remoteParallelFileIOClient64_c::getLocalDateTime(filename,
									  &thedate[0],
									  &thedate[1],
									  &thedate[2],
									  &thedate[3],
									  &thedate[4],
									  &thedate[5]);

		// If file does not exist locally, then check neighbor server if neighbor exists.
		if (status != CAVERNnet_remoteParallelFileIOClient64_c::OK && neighborServerName[0]) {
			if (serverIP == getHostIPNumber(neighborServerName) 
							&& serverPort == neighborPort) { // detected cycle
#ifdef DBG_MSG
				printf("Cycle detected. Server processing 't'\n");
#endif
				status = newClient->writeChar('n');
				delete filename; delete data; return NULL;
			}

		    CAVERNnet_ParallelServerClient64_c client(server, serverIP, serverPort);

			client.setRemotePort(neighborPort);
			status = client.getRemoteDateTime(neighborServerName, filename, 
			                          &thedate[0], &thedate[1],
									  &thedate[2], &thedate[3],
									  &thedate[4], &thedate[5]);
			if (status != CAVERNnet_ParallelServerClient64_c::OK) {
				newClient->writeChar('n');
				delete filename; delete data; return NULL;
			}
		}

		// Send back command 't'
		status = newClient->writeChar('t');
		if (status != CAVERNnet_parallelTcpClient_c::OK) {
			delete filename; delete data; return NULL;
		}

		// Send back date/time stamp of file
		nbytes = 6 * packer.sizeof_int();
		char* send_buffer = new char[nbytes];
        packer.initPack(send_buffer,nbytes);
		for (int i = 0; i < 6; i++) packer.packInt(thedate[i]);
		status = newClient->write((char*) send_buffer, &nbytes);
		delete send_buffer;

		delete filename;
		delete data;
		return NULL;
	}

	// If handling upload
	if (transferMode == 'u') {
		// Get datasize
		int64 dataSize;

		status = newClient->readInt64(dataSize);
		if (status != CAVERNnet_parallelTcpClient_c::OK) {delete filename; delete data; return NULL;}

		FILE *writeFile = fopen(filename,"wb");
		if (writeFile == NULL) {
			delete filename;
			delete data;
			return NULL;
		}

		// Read from file and write to socket in 8K chunks
		int64 numChunks = dataSize / CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE;
		for (int64 i = 0; i < numChunks; i++) {
			nbytes = CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE;
			status = newClient->read(dataBlock, &nbytes);

			if (status != CAVERNnet_parallelTcpClient_c::OK) {fclose(writeFile); delete filename; delete data; return NULL;}

			fwrite(dataBlock, nbytes, 1, writeFile);
		}

		int remain = (int) (dataSize - (int64) ((int64) numChunks * (int64) CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE));
		if (remain) {
			nbytes = remain;
			status = newClient->read(dataBlock, &nbytes);

			if (status != CAVERNnet_parallelTcpClient_c::OK) {fclose(writeFile); delete filename; delete data; return NULL;}

			nbytes = remain;
			fwrite(dataBlock, nbytes, 1, writeFile);
		}
		fclose(writeFile);
		delete filename;
		delete data;

		return NULL;
	}

	/* If handling download... */
	if (transferMode == 'd') {
		int64 fileSize;

		FILE *readFile = NULL, *writeFile = NULL;
		CAVERNnet_ParallelServerClient64_c* downloadClient = NULL;
	
		readFile = fopen(filename, "rb");
		if (readFile == NULL) {
			if (neighborServerName[0]) {
				if (server->getSelfIP() == getHostIPNumber(neighborServerName) 
								&& server->getSelfPort() == neighborPort) { // trying to connect to itself
#ifdef DBG_MSG
					printf("Trying to connecting to itself. server processing 'd'\n");
#endif
					status = newClient->writeChar('n');
					delete filename; delete data; return NULL;
				}

				downloadClient = new CAVERNnet_ParallelServerClient64_c(server);
				downloadClient->setRemotePort(neighborPort);

				if (downloadClient == NULL) {
					delete filename; delete data; return NULL;
				}
				if (downloadClient->initDownload(neighborServerName, filename, filename, numberOfSockets) !=
					                                  CAVERNnet_ParallelServerClient64_c::DOWNLOAD) {
					newClient->writeChar('i');
					delete filename; delete downloadClient; 
					delete data; return NULL;
				}

				if (store) writeFile = fopen(filename, "wb");
			}
			else { 
				status = newClient->writeChar('n');
				delete filename; delete data; return NULL; 
			}
		}

		delete filename;

		status = newClient->writeChar('d');
		if (status != CAVERNnet_parallelTcpClient_c::OK) {
			CLOSE_REMOVE_AND_RETURN
		}

		// Find length of file.
		if (readFile) { // if file exists locally
			fseek64(readFile, 0, SEEK_END);
			fileSize = ftell64(readFile);
			fseek64(readFile , 0L, SEEK_SET);
		}
		else { // get the length of file from the neighbor server
			status = downloadClient->getFileSize(fileSize);
			if (status != CAVERNnet_ParallelServerClient64_c::OK) {
				CLOSE_REMOVE_AND_RETURN
			}
		}

		// Send length of file
		status = newClient->writeInt64(fileSize);
		if (status != CAVERNnet_parallelTcpClient_c::OK) {
			CLOSE_REMOVE_AND_RETURN
		}

		// Send file
		int64 numChunks = fileSize / (int64) CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE;
		for(int64 i = 0; i < numChunks; i++) {
			nbytes = CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE;
			if (readFile) fread(dataBlock, nbytes, 1, readFile);
			else {
				if (downloadClient->readBlock(dataBlock, nbytes) != CAVERNnet_ParallelServerClient64_c::OK) {
					CLOSE_REMOVE_AND_RETURN
				}
				if (writeFile) fwrite(dataBlock, nbytes, 1, writeFile);
			}
			status = newClient->write(dataBlock, &nbytes);
			if (status != CAVERNnet_parallelTcpClient_c::OK) {
				CLOSE_REMOVE_AND_RETURN
			}
		}

		int remain = (int) (fileSize - (int64) ((int64) numChunks * (int64) CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE));		
		if (remain) {
			nbytes = remain;
			if (readFile) fread(dataBlock, nbytes, 1, readFile);
			else {
				if (downloadClient->readBlock(dataBlock, nbytes) != CAVERNnet_ParallelServerClient64_c::OK) {
					CLOSE_REMOVE_AND_RETURN
				}
				if (writeFile) fwrite(dataBlock, nbytes, 1, writeFile);
			}
			status = newClient->write(dataBlock, &nbytes);
			if (status != CAVERNnet_parallelTcpClient_c::OK) {
				CLOSE_REMOVE_AND_RETURN
			}
		}
		
		CLOSE_REMOVE_AND_RETURN
	}

	// If handling download for the request from a server... 
	if (transferMode == 'o') {
		int64 fileSize;
		FILE *readFile = NULL, *writeFile = NULL;
		CAVERNnet_ParallelServerClient64_c* downloadClient = NULL;

		// Get server name's length
		int serverIP;
		if (newClient->readInt(serverIP) != CAVERNnet_parallelTcpClient_c::OK) { 
			delete filename; delete data; return NULL; 
		}

		// get port number 
		int serverPort;
		if (newClient->readInt(serverPort) != CAVERNnet_parallelTcpClient_c::OK) { 
			delete filename; delete data; return NULL; 
		}
		
		// Check whether we are getting back the request this server sent before.
/*		if (serverIP == server->getSelfIP() && serverPort == server->getSelfPort()) { 
			printf("cycle detected\n");
			newClient->writeChar('i');
			delete data;
			return NULL;
		}*/

		readFile = fopen(filename, "rb");
		if (readFile == NULL) {
			if (neighborServerName[0]) {
				if (serverIP == getHostIPNumber(neighborServerName) 
								&& serverPort == neighborPort) { // detected cycle
#ifdef DBG_MSG
					printf("Cycle detected in server processing 'o'\n");
#endif
					status = newClient->writeChar('n');
					delete filename; delete data; return NULL;
				}

				downloadClient = new CAVERNnet_ParallelServerClient64_c(server, serverIP, serverPort);
				downloadClient->setRemotePort(neighborPort);

				if (downloadClient == NULL) {
					newClient->writeChar('n');
					delete filename; delete data; return NULL;
				}
				if (downloadClient->initDownload(neighborServerName, filename, filename, numberOfSockets) !=
                 					CAVERNnet_ParallelServerClient64_c::DOWNLOAD) {
					newClient->writeChar('n');
					delete filename; delete downloadClient; 
					delete data; return NULL;
				}

				if (store) writeFile = fopen(filename, "wb");
			}
			else { 
				newClient->writeChar('n');
				delete filename; delete data; return NULL; 
			}
		}

		delete filename;

		if (newClient->writeChar('o') != CAVERNnet_parallelTcpClient_c::OK) {
			CLOSE_REMOVE_AND_RETURN
		}

		// Find length of file.
		if (readFile) { // if file exists locally
			fseek64(readFile, 0L, SEEK_END);
			fileSize = ftell64(readFile);
			fseek64(readFile , 0L, SEEK_SET);
		}
		else { // get the length of file from the neighbor server
			if (downloadClient->getFileSize(fileSize) != CAVERNnet_ParallelServerClient64_c::OK) {
				newClient->writeChar('n');
				CLOSE_REMOVE_AND_RETURN
			}
		}

		// Send length of file
		if (newClient->writeInt(fileSize) != CAVERNnet_parallelTcpClient_c::OK) {
			CLOSE_REMOVE_AND_RETURN
		}

		// Send file
		int64 numChunks = fileSize / (int64) CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE;
		for (int64 i = 0; i < numChunks; i++) {
			nbytes = CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE;
			if (readFile) fread(dataBlock, nbytes, 1, readFile);
			else {
				if (downloadClient->readBlock(dataBlock, nbytes) != CAVERNnet_ParallelServerClient64_c::OK) {
					CLOSE_REMOVE_AND_RETURN
				}
				if (writeFile) fwrite(dataBlock, nbytes, 1, writeFile);
			}
			status = newClient->write(dataBlock, &nbytes);
			if (status != CAVERNnet_parallelTcpClient_c::OK) {
				CLOSE_REMOVE_AND_RETURN
			}
		}

		int remain = (int) (fileSize - ((int64) numChunks * (int64) CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE));
		if (remain) {
			nbytes = remain;
			if (readFile) fread(dataBlock, nbytes, 1, readFile);
			else {
				if (downloadClient->readBlock(dataBlock, nbytes) != CAVERNnet_ParallelServerClient64_c::OK) {
					CLOSE_REMOVE_AND_RETURN
				}
				if (writeFile) fwrite(dataBlock, nbytes, 1, writeFile);
			}
			status = newClient->write(dataBlock, &nbytes);
			if (status != CAVERNnet_parallelTcpClient_c::OK) {
				CLOSE_REMOVE_AND_RETURN
			}
		}
		
		CLOSE_REMOVE_AND_RETURN
	}

	return NULL;
}
#undef CLOSE_REMOVE_AND_RETURN 	

int CAVERNnet_remoteParallelFileIOServer64_c::getHostIPNumber(char* hostNameOrIP)
{
	struct hostent* hostEnt;

	unsigned ip = inet_addr(hostNameOrIP);
	if ((ip != INADDR_NONE) || (strcmp(hostNameOrIP, "255.255.255.255") == 0))
		return (int) ip;
	else {
		hostEnt = gethostbyname(hostNameOrIP);
		if (hostEnt) 
			return *((int*) hostEnt->h_addr);
		else {
		    printf("Invalid address is given: %s\n", hostNameOrIP);
			return 0;
		}
	}
}

const int CAVERNnet_remoteParallelFileIOClient64_c::OK = 1;
const int CAVERNnet_remoteParallelFileIOClient64_c::FAILED = 0;
const int CAVERNnet_remoteParallelFileIOClient64_c::CANNOT_CONNECT = 2;
const int CAVERNnet_remoteParallelFileIOClient64_c::CANNOT_READ_LOCAL_FILE = 3;
const int CAVERNnet_remoteParallelFileIOClient64_c::CANNOT_WRITE_LOCAL_FILE = 3;
const int CAVERNnet_remoteParallelFileIOClient64_c::INVALID_COMMAND = 4;
const int CAVERNnet_remoteParallelFileIOClient64_c::MEM_ALLOC_ERR = 5;
const int CAVERNnet_remoteParallelFileIOClient64_c::FILE_ERROR = 6;
const int CAVERNnet_remoteParallelFileIOClient64_c::EARLIER = 7;
const int CAVERNnet_remoteParallelFileIOClient64_c::LATER = 8;
const int CAVERNnet_remoteParallelFileIOClient64_c::SAME = 9;

CAVERNnet_remoteParallelFileIOClient64_c :: CAVERNnet_remoteParallelFileIOClient64_c()
{
	remotePort = CAVERNnet_remoteParallelFileIOServer64_c::DEFAULT_PORT;
}

void CAVERNnet_remoteParallelFileIOClient64_c :: setRemotePort(int port) {remotePort = port;}

int CAVERNnet_remoteParallelFileIOClient64_c :: getRemoteDateTime(char *ipAddress, char* remoteFileName, int *remDay, int *remMon, int *remYear, int* remHour, int* remMin, int* remSec, int numberOfSockets)
{
	CAVERNnet_extendedParallelTcpClient_c* client = new CAVERNnet_extendedParallelTcpClient_c;
	CAVERNnet_datapack_c packer;
	char *read_buffer;
	int nbytes;
	int status;

	if (client == NULL) return CAVERNnet_remoteParallelFileIOClient64_c :: MEM_ALLOC_ERR;

	status = client->connectToServer(ipAddress, remotePort, numberOfSockets);
	if (status == CAVERNnet_parallelTcpClient_c::FAILED) {
		return CAVERNnet_remoteParallelFileIOClient64_c :: CANNOT_CONNECT;
	}

	status = client->writeChar('c');
	if (status != CAVERNnet_parallelTcpClient_c::OK) {
	    delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;
	}
    
    //Increment the data sent
    pmonitor.incrementDataSent(packer.sizeof_char());
        
	// Send filename length
	int filenamelen = strlen(remoteFileName) + 1;
	status = client->writeInt(filenamelen);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}
    
    nbytes = packer.sizeof_int();
    //Increment the data sent
    pmonitor.incrementDataSent(nbytes);
        
	// Send filename
	nbytes = filenamelen;
	status = client->write(remoteFileName, &nbytes);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}

    pmonitor.incrementDataSent(nbytes);
    
	// Now wait for response.
	char transferMode = ' ';
	status = client->readChar(transferMode);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}

    //Increment the data read
    nbytes = sizeof(char);
    pmonitor.incrementDataRead(nbytes);
     
	if (transferMode == 'c') {
		int remdate[6];

		nbytes = packer.sizeof_int() * 6;
		read_buffer = new char[nbytes];
		status = client->read(read_buffer, &nbytes);
        packer.initUnpack(read_buffer,nbytes);  
		for (int i = 0; i < 6; i++)
			packer.unpackInt(&(remdate[i]));
        delete read_buffer;                     

		if (status != CAVERNnet_parallelTcpClient_c::OK) {
			delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FILE_ERROR;
		}
        //Increment the data read
        pmonitor.incrementDataRead(nbytes);
        
        //set IP and port info for the performance monitoring statistics        
        char selfIp[256], remoteIp[256];
    
        client->getSelfIP(selfIp);
        client->getRemoteIP(remoteIp);
        pmonitor.setIPs(selfIp, remoteIp);
        pmonitor.setPorts(client->getSelfPort(), client->getRemotePort());
     
		*remDay = remdate[0]; *remMon = remdate[1]; *remYear = remdate[2];
		*remHour = remdate[3]; *remMin = remdate[4]; *remSec = remdate[5];
		if ((remdate[0] == -1) && (remdate[1] == -1) && (remdate[2] == -1) &&
		    (remdate[3] == -1) && (remdate[4] == -1) && (remdate[5] == -1)) {
			delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FILE_ERROR;
		}
		delete client;
		return CAVERNnet_remoteParallelFileIOClient64_c::OK;
	}
	else {
		int result = CAVERNnet_remoteParallelFileIOClient64_c::FAILED;

		if (transferMode == 'i')
			result = CAVERNnet_remoteParallelFileIOClient64_c::INVALID_COMMAND;
		else if (transferMode == 'n')
			result = CAVERNnet_remoteParallelFileIOClient64_c::FILE_ERROR;

		*remDay = -1; *remMon = -1; *remYear = -1;
		*remHour = -1; *remMin = -1; *remSec = -1;
		return result;
	}
	return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;
}

int CAVERNnet_remoteParallelFileIOClient64_c :: upload(char *ipAddress, char* localFileName, char* remoteFileName, int numberOfSockets)
{
	CAVERNnet_extendedParallelTcpClient_c* client = new CAVERNnet_extendedParallelTcpClient_c;
	CAVERNnet_datapack_c packer;

	if (client == NULL) return CAVERNnet_remoteParallelFileIOClient64_c :: MEM_ALLOC_ERR;

	int status = client->connectToServer(ipAddress, remotePort, numberOfSockets);
	if (status == CAVERNnet_parallelTcpClient_c::FAILED) {
		return CAVERNnet_remoteParallelFileIOClient64_c :: CANNOT_CONNECT;
	}

	char transferMode = 'u';

	status = client->writeChar(transferMode);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}

    //Increment the data sent
    int nbytes = sizeof(char);
    pmonitor.incrementDataSent(nbytes);
            
	// Send filename length
	int filenamelen = strlen(remoteFileName) + 1;

	status = client->writeInt(filenamelen);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}

    //increment data sent
    nbytes = packer.sizeof_int();
    pmonitor.incrementDataSent(nbytes);
                
	// Send filename
	nbytes = filenamelen;
	status = client->write(remoteFileName, &nbytes);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}

    pmonitor.incrementDataSent(nbytes);

	// Send data size
	// Find length of file.
	FILE *readFile = fopen(localFileName, "rb");

	if (readFile==NULL) {
		delete client;
		return CAVERNnet_remoteParallelFileIOClient64_c::CANNOT_READ_LOCAL_FILE;
	}

	fseek64(readFile, 0, SEEK_END);
	int fileSize = ftell64(readFile);
	fseek64(readFile, 0L, SEEK_SET);

	status = client->writeInt64(fileSize);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; fclose(readFile); return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}
    
    //Increment the data sent
    nbytes = packer.sizeof_int();
    pmonitor.incrementDataSent(nbytes);
        
	// Send data
	char dataBlock[CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE];

	int numChunks = fileSize / CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE;

	for (int64 i = 0; i < numChunks; i++) {
		fread(dataBlock, CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE, 1, readFile);
		nbytes = CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE;
		status = client->write(dataBlock, &nbytes);
		if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; fclose(readFile); return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}
	    //Increment the data sent
        pmonitor.incrementDataSent(nbytes);
    }
    
    int remain = (int) (fileSize - (int64) ((int64) numChunks * (int64) CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE));
	if (remain) {
		fread(dataBlock, remain,1,readFile);
		status = client->write(dataBlock, &remain);
		if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; fclose(readFile); return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}
        //Increment the data sent
        pmonitor.incrementDataSent(nbytes);
    }
    //set IP and port info for the performance monitoring statistics        
    char selfIp[256], remoteIp[256];
    client->getSelfIP(selfIp);
    client->getRemoteIP(remoteIp);
    pmonitor.setIPs(selfIp, remoteIp);
    pmonitor.setPorts(client->getSelfPort(), client->getRemotePort());
 
	fclose(readFile);	
	delete client;

	return CAVERNnet_remoteParallelFileIOClient64_c :: OK;
}


int CAVERNnet_remoteParallelFileIOClient64_c :: download(char *ipAddress, char* localFileName, char* remoteFileName, int numberOfSockets)
{
	CAVERNnet_extendedParallelTcpClient_c* client = new CAVERNnet_extendedParallelTcpClient_c;
	CAVERNnet_datapack_c packer;

	if (client == NULL) return CAVERNnet_remoteParallelFileIOClient64_c :: MEM_ALLOC_ERR;

	int status = client->connectToServer(ipAddress, remotePort, numberOfSockets);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {
		return CAVERNnet_remoteParallelFileIOClient64_c :: CANNOT_CONNECT;
	}

	int nbytes;
	char transferMode = 'd';
	status = client->writeChar(transferMode);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}

    //Increment the data sent
    nbytes = packer.sizeof_char();
    pmonitor.incrementDataSent(nbytes);
            
	// Send filename length
	int filenamelen = strlen(remoteFileName) + 1;
	status = client->writeInt(filenamelen);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}

    //Increment the data sent
    nbytes = packer.sizeof_int();
    pmonitor.incrementDataSent(nbytes);
            
	// Send filename
	nbytes = filenamelen;
	status = client->write(remoteFileName, &nbytes);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}
    pmonitor.incrementDataSent(nbytes);
    
	transferMode =' ';
	status = client->readChar(transferMode);
	if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}

    //Increment the data read
    nbytes = sizeof(char);
    pmonitor.incrementDataRead(nbytes);

	if (transferMode == 'd') {
		int64 dataSize;

		status = client->readInt64(dataSize);                                                     
		if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}
        
        //Increment the data read
        nbytes = packer.sizeof_int64();
		pmonitor.incrementDataRead(nbytes);

        FILE *writeFile = fopen(localFileName,"wb");
		if (writeFile == NULL) {
			delete client;
			return CAVERNnet_remoteParallelFileIOClient64_c :: CANNOT_WRITE_LOCAL_FILE;
		}
		// Read from file and write to socket in chunks
		int64 numChunks = dataSize / (int) CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE;
		char dataBlock[CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE];

		for (int64 i = 0; i < numChunks; i++) {
			nbytes = CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE;
			status = client->read(dataBlock, &nbytes);
			if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; fclose(writeFile); return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}
            pmonitor.incrementDataRead(nbytes);
            
			fwrite(dataBlock, nbytes, 1, writeFile);
		}

		int remain = (int) (dataSize - ((int64) numChunks * (int64) CAVERNnet_remoteParallelFileIOServer64_c::TRANSFERSIZE));
		if (remain) {
			nbytes = remain;
			status = client->read(dataBlock, &nbytes);
			if (status != CAVERNnet_parallelTcpClient_c::OK) {delete client; fclose(writeFile); return CAVERNnet_remoteParallelFileIOClient64_c::FAILED;}
            pmonitor.incrementDataRead(nbytes);
            
			nbytes = remain;
			fwrite(dataBlock, nbytes, 1, writeFile);
		}
       
        //set IP and port info for the performance monitoring statistics        
        char selfIp[256], remoteIp[256];
    
        client->getSelfIP(selfIp);
        client->getRemoteIP(remoteIp);
        pmonitor.setIPs(selfIp, remoteIp);
        pmonitor.setPorts(client->getSelfPort(), client->getRemotePort());
  
		fclose(writeFile);
		delete client;
		return CAVERNnet_remoteParallelFileIOClient64_c :: OK;
	} 
	else {
		int result;

		if (transferMode == 'i') 
			result = CAVERNnet_remoteParallelFileIOClient64_c::INVALID_COMMAND;
		else if (transferMode == 'n')
			result = CAVERNnet_remoteParallelFileIOClient64_c::FILE_ERROR;

		delete client;	
		return result;
	}
}

// Given a filename return its modification time
int CAVERNnet_remoteParallelFileIOClient64_c :: getLocalDateTime(char* filename,
		 int *day, int *month, int *year, int *hour, int *minute, int *second)
{

	struct stat buf;

	*day = *month = *year = *hour = *minute = *second = -1;

	if (stat(filename, &buf) != 0) return CAVERNnet_remoteParallelFileIOClient64_c::FILE_ERROR;

	time_t tim;

#ifdef __sgi
    tim = buf.st_mtim.tv_sec; 
#else
    tim = buf.st_mtime;       
#endif                            
                                  
	struct tm *gmt;
	gmt = gmtime(&tim);

	*day = gmt->tm_mday;
	*month = gmt->tm_mon + 1;
	*year = gmt->tm_year + 1900;
	*hour = gmt->tm_hour;
	*minute = gmt->tm_min;
	*second = gmt->tm_sec;

	return CAVERNnet_remoteParallelFileIOClient64_c::OK;
}

int CAVERNnet_remoteParallelFileIOClient64_c :: compareDateTime(int day1, int month1, int year1,
						      int hour1, int minute1, int second1,
						      int day2, int month2, int year2,
						      int hour2, int minute2, int second2)
{
	if (year1 < year2) return CAVERNnet_remoteParallelFileIOClient64_c::EARLIER;
	if (year1 > year2) return CAVERNnet_remoteParallelFileIOClient64_c::LATER;

	if (month1 < month2) return CAVERNnet_remoteParallelFileIOClient64_c::EARLIER;
	if (month1 > month2) return CAVERNnet_remoteParallelFileIOClient64_c::LATER;

	if (day1 < day2) return CAVERNnet_remoteParallelFileIOClient64_c::EARLIER;
	if (day1 > day2) return CAVERNnet_remoteParallelFileIOClient64_c::LATER;

	if (hour1 < hour2) return CAVERNnet_remoteParallelFileIOClient64_c::EARLIER;
	if (hour1 > hour2) return CAVERNnet_remoteParallelFileIOClient64_c::LATER;

	if (minute1 < minute2) return CAVERNnet_remoteParallelFileIOClient64_c::EARLIER;
	if (minute1 > minute2) return CAVERNnet_remoteParallelFileIOClient64_c::LATER;

	if (second1 < second2) return CAVERNnet_remoteParallelFileIOClient64_c::EARLIER;
	if (second1 > second2) return CAVERNnet_remoteParallelFileIOClient64_c::LATER;

	return CAVERNnet_remoteParallelFileIOClient64_c::SAME;
}

void CAVERNnet_remoteParallelFileIOClient64_c::showStats(char* streamInfo, char* comment)        
{
    pmonitor.showStats(streamInfo, comment);    
}

int CAVERNnet_remoteParallelFileIOClient64_c::logStats(char* streamInfo, char* comment, FILE* filePtr)
{
    return pmonitor.logStats(streamInfo, comment, filePtr);
}


int CAVERNnet_remoteParallelFileIOClient64_c::sendStats(char* streamInfo, char* comment)
{
    return pmonitor.sendStats(streamInfo, comment);    
}

int CAVERNnet_remoteParallelFileIOClient64_c::initSendStats(char* monitorClientIP, int port)
{
    return pmonitor.initSendStats(monitorClientIP, port);
}

void CAVERNnet_remoteParallelFileIOClient64_c::exitSendStats()
{
    pmonitor.exitSendStats();
}
#endif
