/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include "CAVERNinit.hxx"
#include "CAVERNnet_socketbase_c.hxx"
#include <sys/timeb.h>

#ifdef WIN32
#include "gettimeofday.hxx"
#endif

const int CAVERNnet_socketbase_c::OK = 1;
const int CAVERNnet_socketbase_c::FAILED = 0;

CAVERNnet_socketbase_c :: CAVERNnet_socketbase_c() {
    initialReadTime = initialSentTime = getTimeInSecs();
	previousSentTime = initialSentTime;
	previousReadTime = initialReadTime;

	totalDataRead = totalDataSent = prevTotalDataRead = prevTotalDataSent = 0;
	latCount = readCount = sentCount =0;
	instLat = latTotal = maxLat = minLat = avgLat = 0;
	instDelay = maxDelay = minDelay = avgDelay = totalDelay = 0;
   instReadBandwidth= instSendBandwidth= avgReadBandwidth= avgSendBandwidth =0;
    
    jitter = burstiness = STALatency =STABandwidth =0;
    for (int i=0; i<LATENCY_BUF_SIZE; i++, latencyBuffer[i] = bandwidthBuffer[i] =0);
    STlatencySum = STbandwidthSum =0;    
    tempLatCounter = tempBWCounter = 0;  
}

double CAVERNnet_socketbase_c :: getTimeInSecs()
{
/*#ifdef WIN32 //using either high performance counter or _ftime in Windows 
	if (UseFTime) { // Use _ftime. Precision is limited to milliseconds level.
	    const double oneMilliSec = 0.001;	
		struct _timeb timeSec;

		_ftime(&timeSec);

		return (((double) timeSec.time) + (((double) timeSec.millitm) * oneMilliSec));
	}
	else { // Use high performance counter. Precision is in microseconds level.
		LARGE_INTEGER endCounter;	
		double tempL;

		QueryPerformanceCounter(&endCounter);
		return StartTime + (double) ((endCounter.QuadPart - StartCounter.QuadPart) / (double) Frequency.QuadPart);
	}
#else //using gettimeofday() otherwise - gives time with microsecond level precision*/
    const double oneMicroSec = 0.000001;
	struct timeval timeSec;

	gettimeofday(&timeSec, NULL);

	return (((double) timeSec.tv_sec) +
		(((double) timeSec.tv_usec) * oneMicroSec));
//#endif
}

void CAVERNnet_socketbase_c::ipNumToString(unsigned int ipnum, char *ipstring)
{
	char *name = (char*) &ipnum;

	int a,b,c,d;
	a = (unsigned char) name[0];
	b = (unsigned char) name[1];
	c = (unsigned char) name[2];
	d = (unsigned char)  name[3];
	sprintf(ipstring,"%d.%d.%d.%d",a,b,c,d);
}

void CAVERNnet_socketbase_c :: expandHostName(char *src, char* newname, int len)
{
	struct hostent *	hp_ptr;

        hp_ptr = gethostbyname(src);
        if (hp_ptr != NULL) {
		struct in_addr      addr;

		memcpy(&(addr.s_addr), hp_ptr->h_addr, hp_ptr->h_length);
		hp_ptr = gethostbyaddr((char *)&addr,sizeof(addr),AF_INET);
		if (hp_ptr != NULL && strcmp(hp_ptr->h_name, src) != 0){
			strncpy(newname, hp_ptr->h_name,len);
		}
		else {
			hp_ptr = NULL;
		}
	}
	if (hp_ptr == NULL) {
		strncpy(newname,src,len);
	}

}

void CAVERNnet_socketbase_c :: getHostName(char *name, int len)
{
	static char hostname[256];

	if (gethostname(hostname,256) < 0) {
		name[0] = '\0';
		return;
	}

	expandHostName(hostname, name, len);
}

int CAVERNnet_socketbase_c::hostnameToIP(char *hostname, char *hostIP)
{
#ifndef WIN32
	extern int h_errno;
#endif
	struct hostent* host;

	host = gethostbyname(hostname);

	if (!host) {
		printf("Unable to resolve hostname: %s\n",hostname);

		/* This is needed becoz on the Sun
		 * gethostbyname returns nothing if
		 * the explicit IP address is specified,
		 * whereas on the SGI the same IP is
		 * returned.
		 */

		sprintf(hostIP,"?.?.?.?");

		return FAILED;
	}

	unsigned int* addr;
	addr = (unsigned int*) host->h_addr_list[0];
	ipNumToString(*addr, hostIP);
	return OK;
}

unsigned int CAVERNnet_socketbase_c::getTotalDataSent()
{
	return totalDataSent;
}

unsigned int CAVERNnet_socketbase_c::getTotalDataRead()
{
	return totalDataRead;
}

double CAVERNnet_socketbase_c::getInstantSendBandwidth(){
	return instSendBandwidth;
}

double CAVERNnet_socketbase_c::getInstantReceiveBandwidth(){
	return instReadBandwidth;
}

double CAVERNnet_socketbase_c::getAverageSendBandwidth()
{
	return avgSendBandwidth;
}


double CAVERNnet_socketbase_c::getAverageReceiveBandwidth()
{
	return avgReadBandwidth;
}

void CAVERNnet_socketbase_c::computeInstantSendBandwidth()
{
	/* Figure out what time it is */

	currentTime = getTimeInSecs();
    if (!sentCount)
    {
        initialSentTime = currentTime;
    }
    else
    {
         double sentDifference = currentTime - previousSentTime ;
        if (sentDifference > 0)
            instSendBandwidth = double(totalDataSent - prevTotalDataSent) / (currentTime-previousSentTime);
        //else instSendBandwidth is the previous instantaneous SBW value
    }
    prevTotalDataSent = totalDataSent;		
	previousSentTime = currentTime;
}

void CAVERNnet_socketbase_c::computeInstantReceiveBandwidth()
{
	/* Figure out what time it is */
	currentTime = getTimeInSecs();
    if (!readCount)
    {
        initialReadTime = currentTime;
    }
    else
    {
        double readDifference = currentTime - previousReadTime ;
        if (readDifference > 0)
            instReadBandwidth = double(totalDataRead - prevTotalDataRead) / (currentTime-previousReadTime);
        //else instReadBandwidth is the previous inst.RBW
    }
    prevTotalDataRead = totalDataRead;
    previousReadTime = currentTime;
}

void CAVERNnet_socketbase_c::computeAverageReceiveBandwidth()
{
    //Average bandwidth calculated from the second packet received
    if (readCount)
    {        
        currentTime = getTimeInSecs();
        avgReadBandwidth = double(totalDataRead)/(currentTime-initialReadTime);
    }
    //calculate the burstiness with the inst read bandwidth
    calculateBurstiness(instReadBandwidth);
}

void CAVERNnet_socketbase_c::computeAverageSendBandwidth()
{
    //Average bandwidth calculated from the second packet sent
	if (sentCount)
    {
        currentTime = getTimeInSecs();
        avgSendBandwidth = double(totalDataSent)/(currentTime-initialSentTime);
    } 
}

double CAVERNnet_socketbase_c :: getInstantLatency()
{
	return instLat;
}

double CAVERNnet_socketbase_c :: getAverageLatency()
{
    return avgLat;
}

void CAVERNnet_socketbase_c::setInstantLatency(double lat)
{
    double curTime = getTimeInSecs();
	if (latCount == 0){
		minLat = lat;
	}
	instLat = lat;
	if (lat > maxLat) maxLat = lat;
	if (lat < minLat) minLat = lat;

    //Calculate the jitter with the inst.latency -also increment latency count and total latency calculated
    calculateJitter(lat);
}

double CAVERNnet_socketbase_c::getMinLatency()
{
	return minLat;
}

double CAVERNnet_socketbase_c::getMaxLatency()
{
	return maxLat;
}

void CAVERNnet_socketbase_c::incrementDataSent(unsigned long size)
{
    totalDataSent+=size;
    computeInstantSendBandwidth();
    computeAverageSendBandwidth();
    sentCount++;
}

void CAVERNnet_socketbase_c::incrementDataRead(unsigned long size)
{
    double curTime = getTimeInSecs();
    totalDataRead+=size;
	if (!readCount)
    {
        prevLocalTime = curTime;
    }
    if (readCount)
    { 
        instDelay = curTime - prevLocalTime; 
        if (readCount == 1) minDelay = instDelay;
        totalDelay+=instDelay;
        prevLocalTime = curTime; 
        if (instDelay>maxDelay) maxDelay = instDelay;
        if (instDelay<minDelay) minDelay = instDelay; 
    }

    computeInstantReceiveBandwidth();
    computeAverageReceiveBandwidth();
}

double CAVERNnet_socketbase_c::getInstantInterMesgDelay()
{
	return instDelay;
}

double CAVERNnet_socketbase_c::getMaxInterMesgDelay()
{
	return maxDelay;
}

double CAVERNnet_socketbase_c::getMinInterMesgDelay()
{
	return minDelay;
}

double CAVERNnet_socketbase_c::getAverageInterMesgDelay()
{
    return avgDelay;
}

unsigned long CAVERNnet_socketbase_c :: getPacketsRead()
{
    return readCount;
}

unsigned long CAVERNnet_socketbase_c :: getPacketsSent()
{
    return sentCount;
}

double CAVERNnet_socketbase_c::getSTALatency()
{
	return STALatency;
}

double CAVERNnet_socketbase_c::getJitter()
{
	return jitter;
}

double CAVERNnet_socketbase_c::getSTABandwidth()
{
	return STABandwidth;
}

double CAVERNnet_socketbase_c::getBurstiness()
{
	return burstiness;
}

void CAVERNnet_socketbase_c :: calculateJitter (double lat)
{
    int lastnumber;
    int i;
    double devn;    
    //find the absolute latency
    lat=fabs(lat);
    
    if (latCount <  LATENCY_BUF_SIZE)
         lastnumber = tempLatCounter+1;
    else 
        lastnumber = LATENCY_BUF_SIZE;
    
    
    //calculate the cumulative sum of the latencies
    latTotal+= lat;        
    latencyBuffer[tempLatCounter] = lat;                     
    
    STlatencySum =0;
    for (i=0; i<lastnumber; i++)
    {
        STlatencySum+= latencyBuffer[i];        
    }
    
    STALatency = STlatencySum / lastnumber;
    
    //calculating jitter for the latencies in the buffer
    
    jitter =0;    
    for (i=0; i<lastnumber; i++)
    {
        devn = fabs(latencyBuffer[i] - STALatency);
        jitter+= devn;
    }
    jitter = jitter/lastnumber;        
    
    latCount++;
    avgLat = (double)latTotal/(double)latCount;

    if (tempLatCounter ==  LATENCY_BUF_SIZE -1)    
        tempLatCounter =0;        
    else
        tempLatCounter++;
}


void CAVERNnet_socketbase_c :: calculateBurstiness (double bw)
{
    int i;
    int lastnumber; 
    double devn;    
    //find the absolute bandwidth
    bw = fabs(bw);
    if (readCount)
    {        
        if (readCount <=  BANDWIDTH_BUF_SIZE)
            lastnumber = tempBWCounter+1;
        else 
            lastnumber = BANDWIDTH_BUF_SIZE;
        
        bandwidthBuffer[tempBWCounter] = bw;    
        
        STbandwidthSum =0;
        for (i=0; i<lastnumber; i++)
        {
            STbandwidthSum+= bandwidthBuffer[i];        
        }
    
        STABandwidth = STbandwidthSum / lastnumber;
        
        //calculating burstiness for the bandwitdh values in the buffer
        
        burstiness =0;    
        for (i=0; i<lastnumber; i++)
        {
            devn = fabs(bandwidthBuffer[i] - STABandwidth);
            burstiness+= devn;
        }
        burstiness = burstiness/lastnumber;
    }
    
    readCount++;

    //Delay is calculated from the second packet onwards
    if (readCount >1)
    {
        avgDelay = (double )(totalDelay /(readCount -1));
        if (tempBWCounter == BANDWIDTH_BUF_SIZE -1)    
            tempBWCounter = 0;        
        else
            tempBWCounter++;
    }
    else
        avgDelay = 0;
}

