/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include "CAVERNnet_tcpReflector_c.hxx"
#include "CAVERNnet_datapack_c.hxx"

const int CAVERNnet_tcpReflector_c::OK = 0;
const int CAVERNnet_tcpReflector_c::FAILED = 1;
const int CAVERNnet_tcpReflector_c::MEM_ALLOC_ERR = 2;
const int CAVERNnet_tcpReflector_c::NEW_CONNECTION_ESTABLISHED = 4;
const int CAVERNnet_tcpReflector_c::TOO_MANY_CLIENTS = 5;
const int CAVERNnet_tcpReflector_c::NO_NEW_CONNECTION= 6;
const int CAVERNnet_tcpReflector_c::NON_BLOCKING_HAS_NO_DATA=7;
const int CAVERNnet_tcpReflector_c::SKIP_DISTRIBUTION=8;
const int CAVERNnet_tcpReflector_c::DEFAULT_PORT=TCP_REFLECTOR_DEFAULT_PORT;

const int CAVERNnet_tcpReflectorClient_c::OK = 1;
const int CAVERNnet_tcpReflectorClient_c::FAILED = 2;
const int CAVERNnet_tcpReflectorClient_c::MEM_ALLOC_ERR = 3;
const int CAVERNnet_tcpReflectorClient_c::NON_BLOCKING_HAS_NO_DATA = 7;
const int CAVERNnet_tcpReflectorClient_c::CONNECTION_TERMINATED=8;
const int CAVERNnet_tcpReflectorClient_c::NOT_READY=9;
const int CAVERNnet_tcpReflectorClient_c::READY_TO_READ=10;
const int CAVERNnet_tcpReflectorClient_c::READY_TO_WRITE=11;
const int CAVERNnet_tcpReflectorClient_c::READY_TO_READ_AND_WRITE=12;
const int CAVERNnet_tcpReflectorClient_c::BLOCKING=CAVERNnet_tcpClient_c::BLOCKING;
const int CAVERNnet_tcpReflectorClient_c::NON_BLOCKING=CAVERNnet_tcpClient_c::NON_BLOCKING;
const int CAVERNnet_tcpReflectorClient_c::NO_TIME_OUT=CAVERNnet_tcpClient_c::NO_TIME_OUT;

CAVERNnet_tcpReflector_c :: CAVERNnet_tcpReflector_c()
{
	server = new CAVERNnet_tcpServer_c;
	interceptCallback = NULL;
	interceptUserData = NULL;
	interceptNewConnectionCallback = NULL;
	interceptNewConnectionUserData = NULL;
    perfDaemonIP = NULL;
    perfDaemonPort = PERF_DAEMON_DEFAULT_PORT;
    perfDaemonStatus = PERFDAEMON_NOT_ALIVE ;
}

CAVERNnet_tcpReflector_c :: ~CAVERNnet_tcpReflector_c()
{
    if (perfDaemonIP)
    {
    	exitSendStats();
    	delete perfDaemonIP;
    }
	closeDownSockets();
}

int CAVERNnet_tcpReflector_c :: init(int incomingPort, int maxClients, char* forwardAddress, unsigned short forwardPort)
{
	maxNumClients = maxClients;

	// Initialize the incoming socket
	if (server->init(incomingPort) == 0)	{
		return CAVERNnet_tcpReflector_c::FAILED;
	}

	// Create an array of pointers to track the incoming connections.
	clients = new CAVERNnet_tcpReflectorClient_c*[maxNumClients];
	if (clients == NULL) return CAVERNnet_tcpReflector_c::MEM_ALLOC_ERR;

	for(int i = 0; i < maxNumClients; i++) clients[i] = NULL;

	if (forwardAddress && forwardPort) setForcedDestination(forwardAddress, forwardPort);
	return CAVERNnet_tcpReflector_c::OK;
}

int CAVERNnet_tcpReflector_c :: checkForNewClients()
{
	CAVERNnet_tcpClient_c *aClient;
	/* If a connection is established, accept it and create
	 * a new socket for that connection to communicate with.
	 *
	 */
	aClient = server->checkForNewConnections();

	// If a new client connects then add it to an empty slot in the array of clients.
	if (aClient) {
		CAVERNnet_tcpReflectorClient_c *newClient = new CAVERNnet_tcpReflectorClient_c(aClient);

		// Turn the timeout off. In general this is necessary because in a
		// TCP stream if you lose some data you won't be able to figure out
		// where the head of the next message will be.
		aClient->setTimeOut(CAVERNnet_tcpClient_c::NO_TIME_OUT);

		int status = addNewClient(newClient);

		if (status == CAVERNnet_tcpReflector_c::NEW_CONNECTION_ESTABLISHED) {
			char remoteIP[256];
			int remotePort;

			newClient->getRemoteIP(remoteIP);
			remotePort = newClient->getRemotePort();
			printf("CAVERNnet_tcpReflector_c: New Client: %s:%d\n",remoteIP, remotePort);
/*
If perfdaemon is alive and initSendStats has been done already by the other clients, then this is a new client which has connected to the reflector and a initSendStats has to be done for this client too
 */ 

           if (perfDaemonIP) 
            {
                if (newClient->initSendStats(perfDaemonIP, perfDaemonPort) != CAVERNnet_perfMonitor_c::OK) 
                    perfDaemonStatus = PERFDAEMON_NOT_ALIVE;
            }
            
			if (interceptNewConnectionCallback)
				interceptNewConnectionCallback(newClient, interceptNewConnectionUserData);
		}
		else delete newClient;

		return status;
	}
	return CAVERNnet_tcpReflector_c::NO_NEW_CONNECTION;
}

int CAVERNnet_tcpReflector_c :: setForcedDestination(char *ipAddr, unsigned short port)
{
	CAVERNnet_tcpReflectorClient_c* newClient = new CAVERNnet_tcpReflectorClient_c;

	if (newClient == NULL) return CAVERNnet_tcpReflector_c::MEM_ALLOC_ERR;
	if (newClient->connectToServer(ipAddr, port) != CAVERNnet_tcpReflectorClient_c::OK) {
	    printf("Cannot connect to the forced destination %s.\n", ipAddr);
		delete newClient;
	}
	int status = addNewClient(newClient);

	if (status != CAVERNnet_tcpReflector_c::NEW_CONNECTION_ESTABLISHED) delete newClient;

        /*
	If perfdaemon is alive and initSendStats has been done already by the other clients, then this is a new client which has connected to the reflector and a initSendStats has to be done for this client too
 	*/

        if (perfDaemonIP) {
                if (newClient->initSendStats(perfDaemonIP, perfDaemonPort) != CAVERNnet_perfMonitor_c::OK)
                    perfDaemonStatus = PERFDAEMON_NOT_ALIVE;
        }

	return status;
}

void CAVERNnet_tcpReflector_c :: sendToAll(char* buffer, int incomingSize)
{
	distributeDataExceptTo(-1,buffer,incomingSize);
}

void CAVERNnet_tcpReflector_c :: intercept(int (*callback) (CAVERNnet_tcpReflectorClient_c *, char** buffer, int *datasize, void *userData), void* userData)
{
	interceptCallback = callback;
	interceptUserData = userData;
}

void CAVERNnet_tcpReflector_c :: interceptNewConnection(void (*callback) (CAVERNnet_tcpReflectorClient_c *newClient, void* userData), void *userData)
{
	interceptNewConnectionCallback = callback;
	interceptNewConnectionUserData = userData;
}

int CAVERNnet_tcpReflector_c :: process()
{
	int status;
	int incomingSize = 0;
	char *buffer;

	// Check for any new clients.
	checkForNewClients();

	// Go through array of clients and read from them.
	for (int i = 0; i < maxNumClients; i++) {
		if (clients[i]) {
 			status = clients[i]->read(&buffer, &incomingSize,CAVERNnet_tcpReflectorClient_c::NON_BLOCKING);
			if (status == CAVERNnet_tcpReflectorClient_c::MEM_ALLOC_ERR) return CAVERNnet_tcpReflector_c::MEM_ALLOC_ERR;
			if (status == CAVERNnet_tcpReflectorClient_c::OK) {
				if (interceptCallback) {
					if (interceptCallback(clients[i], &buffer, &incomingSize, interceptUserData) == CAVERNnet_tcpReflector_c::SKIP_DISTRIBUTION) {
						delete buffer;
						continue;
					}
				}

				// Reflect the data to all connected clients except
				// the client that originated the message.
				distributeDataExceptTo(i, buffer, incomingSize);
				delete buffer;
			} 
			else if (status != CAVERNnet_tcpReflectorClient_c::NON_BLOCKING_HAS_NO_DATA)
				removeClient(i);
		}
	}
	return CAVERNnet_tcpReflector_c::OK;
}

int CAVERNnet_tcpReflector_c :: distributeDataExceptTo(int clientToIgnore, char* buf, int incomingSize)
{
	// Iterate thru the array of clients and send the message to all of them
	// except the one indicated by clientToIgnore
	for (int i = 0; i < maxNumClients; i++) {
		if (i != clientToIgnore) {
			if (clients[i] && clients[i]->write(buf, &incomingSize) != CAVERNnet_tcpReflectorClient_c::OK) {
			 	removeClient(i);
			}
		}
	}

	return CAVERNnet_tcpReflector_c::OK;
}

int CAVERNnet_tcpReflector_c::addNewClient(CAVERNnet_tcpReflectorClient_c* newClient)
{
	// If a new client connects then add it to an empty slot in the array of clients.
	if (newClient) {
		for (int i = 0; i < maxNumClients; i++) {
			if (clients[i] == NULL){
				clients[i] = newClient;

				return CAVERNnet_tcpReflector_c::NEW_CONNECTION_ESTABLISHED;
			}
		}

		// If cannot host any more clients then report it.
		return CAVERNnet_tcpReflector_c::TOO_MANY_CLIENTS;
	}
	return CAVERNnet_tcpReflector_c::NO_NEW_CONNECTION;
}

void CAVERNnet_tcpReflector_c :: removeClient(int i)
{
	if (clients[i])
    {
        if (perfDaemonIP)  //if an initSendStats has been done
            clients[i]->exitSendStats();
		delete clients[i];
    }
	clients[i] = NULL;
}

void CAVERNnet_tcpReflector_c :: closeDownSockets()
{
	int count;

	// Iterate thru list of clients and close them down.
	for (count = 0; count < maxNumClients; count++) {
		if (clients[count]) {
			clients[count]->close();
			delete clients[count];
		}
	}
	delete clients;
	server->close();
	delete server;
}

void CAVERNnet_tcpReflector_c::showStats(char* streamInfo, char* comment)
{
    for (int count = 0; count < maxNumClients; count++)
    {
        if (clients[count])
            clients[count]->showStats(streamInfo, comment);        
    }
}

int CAVERNnet_tcpReflector_c::logStats(char* streamInfo, char* comment, FILE* filePtr)
{
    int status=1;    
    for (int count = 0; count < maxNumClients; count++)
    {
        if (clients[count])
            status = ((clients[count]->logStats(streamInfo, comment, filePtr))&&(status));
    }
     if (status)
        return CAVERNnet_perfMonitor_c::OK;
     else return CAVERNnet_perfMonitor_c::FAILED;   
}


int CAVERNnet_tcpReflector_c::sendStats(char* streamInfo, char* comment)
{
    int status =1;    
    
    for (int count = 0; count < maxNumClients; count++)
    {
        if (clients[count])
            status = clients[count]->sendStats(streamInfo, comment)&& status;
    }
    if (status)
        return CAVERNnet_perfMonitor_c::OK;
    else return CAVERNnet_perfMonitor_c::FAILED;
}

int CAVERNnet_tcpReflector_c::initSendStats(char* remoteIP, int port)
{
    int status = 1;    
    if (remoteIP == NULL) return CAVERNnet_perfMonitor_c::FAILED;

    perfDaemonIP = new char[strlen(remoteIP) + 1];
    strcpy(perfDaemonIP, remoteIP);
    perfDaemonPort = port;
    
    for (int count = 0; count < maxNumClients; count++)
    {
        if (clients[count])
            status = clients[count]->initSendStats(remoteIP, port)&& status;
    }
    if (status)   //the perfDameon is alive and the clients have successfully connected to it
    {
        perfDaemonStatus = PERFDAEMON_ALIVE;
        return CAVERNnet_perfMonitor_c::OK;
    }
    else 
        {
            perfDaemonStatus = PERFDAEMON_NOT_ALIVE;            
            return CAVERNnet_perfMonitor_c::FAILED;
        }    
}

void CAVERNnet_tcpReflector_c::exitSendStats()
{
    for (int count = 0; count < maxNumClients; count++)
    {
        if (clients[count])
             clients[count]->exitSendStats();
    }
}
 
// ******************************************************************
CAVERNnet_tcpReflectorClient_c :: CAVERNnet_tcpReflectorClient_c(CAVERNnet_tcpClient_c *aclient)
{
	connectionAlive = 1;
	client = aclient;
	client->setTimeOut(CAVERNnet_tcpReflectorClient_c::NO_TIME_OUT);
	timeOut = CAVERNnet_tcpReflectorClient_c::NO_TIME_OUT;
}

CAVERNnet_tcpReflectorClient_c :: CAVERNnet_tcpReflectorClient_c()
{
	connectionAlive = 0;
	client = new CAVERNnet_tcpClient_c;

	// Turn the timeout off. In general this is necessary because in a
	// TCP stream if you lose some data you won't be able to figure out
	// where the head of the next message will be.
	client->setTimeOut(CAVERNnet_tcpReflectorClient_c::NO_TIME_OUT);
	timeOut = CAVERNnet_tcpReflectorClient_c::NO_TIME_OUT;
}

void CAVERNnet_tcpReflectorClient_c :: setTimeOut(int timeout)
{
	client->setTimeOut(timeout);
	timeOut =  timeout;
}

CAVERNnet_tcpReflectorClient_c :: ~CAVERNnet_tcpReflectorClient_c()
{
	if (client) delete client;
}

int CAVERNnet_tcpReflectorClient_c :: connectToServer(char *ip, int port)
{
	if (client->connectToServer(ip,port) >=0) {
		connectionAlive = 1;
		return CAVERNnet_tcpReflectorClient_c::OK;
	} else {
		connectionAlive = 0;
		return CAVERNnet_tcpReflectorClient_c::FAILED;
	}
}

int CAVERNnet_tcpReflectorClient_c :: read(char **ret, int *nbytes, int blockingType)
{
	if (connectionAlive == 0) return CAVERNnet_tcpReflectorClient_c::CONNECTION_TERMINATED;
	
	CAVERNnet_datapack_c packer;
	char *read_buffer = NULL;
	int dataSize;
	int status;

	// Read the header.	
	dataSize = packer.sizeof_int()  + packer.sizeof_double();
	read_buffer = new char[dataSize];
	status = client->read((char*) read_buffer, &dataSize, blockingType);
	if (status == CAVERNnet_tcpClient_c::OK)  {                                                     
		// The header is simply the # of bytes that follows and the timestamp
		int incomingSize = 0;
        double remoteTimeStamp;
                           
	    packer.initUnpack((char *)read_buffer,dataSize);
		packer.unpackInt(&incomingSize);
        packer.unpackDouble(&remoteTimeStamp);

        //Calculating latency  - for performance monitoring
        double currenttime = client->getTimeInSecs();
        double latency = fabs(currenttime - remoteTimeStamp);
        client->setInstantaneousLatency(latency);

	    delete read_buffer;                           

		char *buffer = new char[incomingSize];
		if (buffer == NULL) {
			*ret = NULL;
			connectionAlive = 0;
			return CAVERNnet_tcpReflectorClient_c::MEM_ALLOC_ERR;
		}

		// Read the body
		status = client->read(buffer, &incomingSize, CAVERNnet_tcpClient_c::BLOCKING);
		if (status == CAVERNnet_tcpClient_c::OK)  {
			*nbytes = incomingSize;
			*ret = buffer;
			return CAVERNnet_tcpReflectorClient_c::OK;
		} else {
			connectionAlive = 0;
			*nbytes = 0;
			delete buffer;
			*ret = NULL;
			if (status == CAVERNnet_tcpClient_c::CONNECTION_TERMINATED)
				return CAVERNnet_tcpReflectorClient_c::CONNECTION_TERMINATED;
			else
				return CAVERNnet_tcpReflectorClient_c::FAILED;
		}
	}

	*nbytes = 0;
	*ret = NULL;
	if (read_buffer) delete read_buffer;

	if (status == CAVERNnet_tcpClient_c::NON_BLOCKING_HAS_NO_DATA) 
		return CAVERNnet_tcpReflectorClient_c::NON_BLOCKING_HAS_NO_DATA;
	if (status == CAVERNnet_tcpClient_c::CONNECTION_TERMINATED) {
		connectionAlive = 0;
		return CAVERNnet_tcpReflectorClient_c::CONNECTION_TERMINATED;
	}
	if (status == CAVERNnet_tcpClient_c::TIMED_OUT) {
		connectionAlive = 0;
		return CAVERNnet_tcpReflectorClient_c::CONNECTION_TERMINATED;
	}

	return CAVERNnet_tcpReflectorClient_c::FAILED;

}

int CAVERNnet_tcpReflectorClient_c :: write(char *ptr, int *nbytes)
{
	CAVERNnet_datapack_c packer;
	char *send_buffer;

	if (connectionAlive == 0) return CAVERNnet_tcpReflectorClient_c::FAILED;

	int  dataSize = packer.sizeof_int()+ packer.sizeof_double();
	int status;
    double timeStamp = client->getTimeInSecs();
	// Send the header
	send_buffer = new char[dataSize];
	packer.initPack(send_buffer, dataSize);
	packer.packInt(*nbytes);
    packer.packDouble(timeStamp);

	status = client->write(send_buffer,&dataSize,CAVERNnet_tcpClient_c::BLOCKING);
	delete send_buffer;
	if (status != CAVERNnet_tcpClient_c::OK) {
		connectionAlive = 0;
		return CAVERNnet_tcpReflectorClient_c::CONNECTION_TERMINATED;
	}
				
	status = client->write(ptr, nbytes, CAVERNnet_tcpClient_c::BLOCKING);
	if (status != CAVERNnet_tcpClient_c::OK) {
		connectionAlive = 0;
		return CAVERNnet_tcpReflectorClient_c::CONNECTION_TERMINATED;
	}

	return CAVERNnet_tcpReflectorClient_c::OK;
}

int CAVERNnet_tcpReflectorClient_c :: isReadyToRead()
{
	if (connectionAlive == 0) return CAVERNnet_tcpReflectorClient_c::NOT_READY;

	int status= client->isReadyToRead();

	if (CAVERNnet_tcpClient_c::READY_TO_READ == status)
		return CAVERNnet_tcpReflectorClient_c::READY_TO_READ;

	else //we got CAVERNnet_tcpClient_c::NOT_READY
		return CAVERNnet_tcpReflectorClient_c::NOT_READY;
}

int CAVERNnet_tcpReflectorClient_c :: isReadyToWrite() 
{
	if (connectionAlive == 0) return CAVERNnet_tcpReflectorClient_c::NOT_READY;

	int status =  client->isReadyToWrite();

	if (CAVERNnet_tcpClient_c::READY_TO_WRITE == status)
		return CAVERNnet_tcpReflectorClient_c::READY_TO_WRITE;
	else // we got CAVERNnet_tcpClient_c::NOT_READY
		return CAVERNnet_tcpReflectorClient_c::NOT_READY;
}

int CAVERNnet_tcpReflectorClient_c :: isReady() 
{
	if (connectionAlive == 0) return CAVERNnet_tcpReflectorClient_c::NOT_READY;

	int status =  client->isReady();

	if (CAVERNnet_tcpClient_c::READY_TO_READ == status)
		return CAVERNnet_tcpReflectorClient_c::READY_TO_READ;
	else if (CAVERNnet_tcpClient_c::READY_TO_WRITE == status)
		return CAVERNnet_tcpReflectorClient_c::READY_TO_WRITE;
	else if (CAVERNnet_tcpClient_c::READY_TO_READ_AND_WRITE == status)
		return CAVERNnet_tcpReflectorClient_c::READY_TO_READ_AND_WRITE;
	else //we got CAVERNnet_tcpClient_c::NOT_READY
		return CAVERNnet_tcpReflectorClient_c::NOT_READY;
}


void CAVERNnet_tcpReflectorClient_c::showStatus(int status, int nbytes)
{
	switch (status) {
	case CONNECTION_TERMINATED:
		printf("CAVERNnet_tcpReflectorClient_c::showStatus: Connection terminated with %d bytes transacted.\n", nbytes);
		break;
	case NON_BLOCKING_HAS_NO_DATA:
		printf("CAVERNnet_tcpReflectorClient_c::showStatus: Non-blocking read found no data on stream.\n");
		break;
	case OK:
		printf("CAVERNnet_tcpReflectorClient_c::showStatus: Transaction ok.\n");
		break;
	default:
#ifdef WIN32
		printf("CAVERNnet_tcpReflectorClient_c::showStatus: UNIX Socket error no: %d ",WSAGetLastError());
#else
		printf("CAVERNnet_tcpReflectorClient_c::showStatus: UNIX Socket error no: %d ",errno);
		perror("");
#endif
		break;
	}
	fflush(stdout);
}

unsigned int CAVERNnet_tcpReflectorClient_c::getRemoteIP() 
{ 
	return client->getRemoteIP(); 
};

void CAVERNnet_tcpReflectorClient_c::getRemoteIP(char* name) 
{ 
	client->getRemoteIP(name); 
}

unsigned int CAVERNnet_tcpReflectorClient_c::getSelfIP() 
{ 
	return client->getSelfIP(); 
}

void CAVERNnet_tcpReflectorClient_c::getSelfIP(char *name) 
{ 
	client->getSelfIP(name); 
}

int CAVERNnet_tcpReflectorClient_c::getSelfPort() 
{ 
	return client->getSelfPort(); 
}

int CAVERNnet_tcpReflectorClient_c::getRemotePort() 
{ 
	return client->getRemotePort(); 
}

void CAVERNnet_tcpReflectorClient_c::close() 
{ 
	connectionAlive = 0; 
	client->close(); 
}

void CAVERNnet_tcpReflectorClient_c::showStats(char* streamInfo, char* comment)
{
    if (client) client->showStats(streamInfo, comment);    
}

int CAVERNnet_tcpReflectorClient_c::logStats(char* streamInfo, char* comment, FILE* filePtr)
{
    return (client) ? client->logStats(streamInfo, comment, filePtr) : CAVERNnet_perfMonitor_c::FAILED;
}

int CAVERNnet_tcpReflectorClient_c::initSendStats(char* remoteIP, int port)
{
    return (client) ? client->initSendStats(remoteIP, port) : CAVERNnet_perfMonitor_c::FAILED;
}

void CAVERNnet_tcpReflectorClient_c::exitSendStats()
{
    if (client) client->exitSendStats();
}

int CAVERNnet_tcpReflectorClient_c::sendStats(char* streamInfo, char* comment)
{
    return (client) ? client->sendStats(streamInfo, comment) : CAVERNnet_perfMonitor_c::FAILED;
}
