/******************************************************************
 * CAVERNsoft
 * Copyright (C) 1994-2000 Electronic Visualization Laboratory,
 * all rights reserved
 * By Jason Leigh, Yong-joo Cho, Naveen Krishnaprasad, Chris Scharver,
 * Stuart Bailey, Atul Nayak, Shalini Venkataraman
 * University of Illinois at Chicago
 * 
 * This publication and its text and code may not be copied for commercial 
 * use without the express written permission of the University of Illinois
 * at Chicago.
 * The contributors disclaim any representation of warranty: use this
 * code at your own risk.
 * Direct questions, comments etc to cavern@evl.uic.edu
 ******************************************************************/

#include "CAVERN.hxx"

CAVERNts_mutex_c :: CAVERNts_mutex_c()
{
#ifdef CAVERN_USE_GLOBUS_THREADS
	globus_mutex_init(&mutex,NULL);
#endif
#ifdef CAVERN_USE_PTHREADS
	pthread_mutex_init(&mutex,NULL);
#endif
#ifdef CAVERN_USE_WINDOWSTHREADS
    mutex = CreateMutex(NULL, FALSE, NULL);
#endif
	debugMsg = NULL;
	debugStat = CAVERNts_mutex_c::DEBUG_OFF;
}

void CAVERNts_mutex_c :: lock()
{
	if (debugStat == DEBUG_ON) {
		cvrnMesg(debugMsg,"BEFORE LOCK");
	}

#ifdef CAVERN_USE_GLOBUS_THREADS
	globus_mutex_lock(&mutex);
#endif
#ifdef CAVERN_USE_PTHREADS
	pthread_mutex_lock(&mutex);
#endif
#ifdef CAVERN_USE_WINDOWSTHREADS
    WaitForSingleObject(mutex, INFINITE);
#endif

	if (debugStat == DEBUG_ON) {
		cvrnMesg(debugMsg,"AFTER LOCK");
	}
}

void CAVERNts_mutex_c :: unlock()
{
	if (debugStat == DEBUG_ON) {
		cvrnMesg(debugMsg,"BEFORE UNLOCK");
	}
#ifdef CAVERN_USE_GLOBUS_THREADS
	globus_mutex_unlock(&mutex);
#endif
#ifdef CAVERN_USE_PTHREADS
	pthread_mutex_unlock(&mutex);
#endif
#ifdef CAVERN_USE_WINDOWSTHREADS
    ReleaseMutex(mutex);
#endif

	if (debugStat == DEBUG_ON) {
		cvrnMesg(debugMsg,"AFTER UNLOCK");
	}
}

void CAVERNts_mutex_c :: setMutexDebug(CAVERNts_mutex_c::mutexDebug_t stat)
{
	debugStat = stat;
}

void CAVERNts_mutex_c :: setMutexDebugMesg(char *msg)
{
	if (debugMsg) delete debugMsg;
	debugMsg = new char[strlen(msg)+1];
	strcpy(debugMsg,msg);
}

CAVERNts_mutex_c :: ~CAVERNts_mutex_c()
{
	if (debugMsg) delete debugMsg;
#ifdef CAVERN_USE_GLOBUS_THREADS
	globus_mutex_destroy(&mutex);
#endif
#ifdef CAVERN_USE_PTHREADS
	pthread_mutex_destroy(&mutex);
#endif
#ifdef CAVERN_USE_WINDOWSTHREADS
    if (mutex) CloseHandle(mutex);
#endif
}

CAVERN_MUTEX_T * CAVERNts_mutex_c :: getMutex()
{
	return &mutex;
}

